@@ -135,13 +135,13 @@ for op in (:<,:<=,:isless)
     @eval ($op)(a::Float16, b::Float16) = ($op)(float32(a), float32(b))
 end
 for func in (:sin,:cos,:tan,:asin,:acos,:atan,:sinh,:cosh,:tanh,:asinh,:acosh,
-             :atanh,:exp,:log,:sqrt)
+             :atanh,:exp,:log,:log2,:log10,:sqrt,:lgamma,:log1p)
     @eval begin
         $func(a::Float16) = float16($func(float32(a)))
         $func(a::Complex32) = complex32($func(complex64(a)))
     end
 end
-atan2(a::Float16, b::Float16) = float16(atan2(float32(a), float32(b)))
+atan2(y::Float16, x::Float16) = float16(atan2(float32(y), float32(x)))
 hypot(a::Float16, b::Float16) = float16(hypot(float32(a), float32(b)))
 ldexp(a::Float16, b::Integer) = float16(ldexp(float32(a), b))
 exponent(x::Float16) = exponent(float32(x))
