@@ -12,12 +12,12 @@ export                                  # types
  etree
 
 using Base.LinAlg.UMFPACK               # for decrement, increment, etc.
- 
+
 import Base: (*), convert, copy, ctranspose, eltype, findnz, getindex, hcat,
              isvalid, nnz, show, size, sort!, transpose, vcat
 
 import ..LinAlg: (\), A_mul_Bc, A_mul_Bt, Ac_ldiv_B, Ac_mul_B, At_ldiv_B, At_mul_B,
-                 cholfact, cholfact!, copy, det, diag,
+                 A_ldiv_B!, cholfact, cholfact!, copy, det, diag,
                  full, isposdef!, logdet, norm, scale, scale!, sparse
 
 include("cholmod_h.jl")
@@ -34,30 +34,40 @@ else
 end
 const chm_com_sz = ccall((:jl_cholmod_common_size,:libsuitesparse_wrapper),Int,())
 const chm_com    = fill(0xff, chm_com_sz)
-const chm_l_com  = fill(0xff, chm_com_sz)             
+const chm_l_com  = fill(0xff, chm_com_sz)
 ## chm_com and chm_l_com must be initialized at runtime because they contain pointers
-## to functions in libc.so, whose addresses can change             
+## to functions in libc.so, whose addresses can change
 function cmn(::Type{Int32})
     if isnan(reinterpret(Float64,chm_com[1:8])[1])
         @isok ccall((:cholmod_start, :libcholmod), Cint, (Ptr{Uint8},), chm_com)
     end
     chm_com
 end
-function cmn(::Type{Int64})             
+function cmn(::Type{Int64})
     if isnan(reinterpret(Float64,chm_l_com[1:8])[1])
         @isok ccall((:cholmod_l_start, :libcholmod), Cint, (Ptr{Uint8},), chm_l_com)
     end
     chm_l_com
 end
 
-typealias CHMITypes Union(Int32,Int64)
-typealias CHMVTypes Union(Complex64, Complex128, Float32, Float64)
+# check the size of SuiteSparse_long
+if int(ccall((:jl_cholmod_sizeof_long,:libsuitesparse_wrapper),Csize_t,())) == 4
+    const CholmodIndexTypes = (:Int32, )
+    typealias CHMITypes Union(Int32)
+else
+    const CholmodIndexTypes = (:Int32, :Int64)
+    typealias CHMITypes Union(Int32, Int64)
+end
+
+
+typealias CHMVTypes Union(Complex128, Float64)
+typealias CHMVRealTypes Union(Float64)
 
 type CholmodException <: Exception end
 
 ## macro to generate the name of the C function according to the integer type
 macro chm_nm(nm,typ) string("cholmod_", eval(typ) == Int64 ? "l_" : "", nm) end
-             
+
 ### A way of examining some of the fields in chm_com
 ### Probably better to make this a Dict{ASCIIString,Tuple} and
 ### save the offsets and the lengths and the types.  Then the names can be checked.
@@ -102,7 +112,7 @@ end
 function set_chm_prt_lev(cm::Array{Uint8}, lev::Integer) # can probably be removed
     cm[(1:4) + chm_com_offsets[13]] = reinterpret(Uint8, [int32(lev)])
 end
-             
+
 ## cholmod_dense pointers passed to or returned from C functions are of Julia type
 ## Ptr{c_CholmodDense}.  The CholmodDense type contains a c_CholmodDense object and other
 ## fields then ensure the memory pointed to is freed when it should be and not before.
@@ -262,9 +272,9 @@ else
 end
 
 type c_CholmodSparse{Tv<:CHMVTypes,Ti<:CHMITypes}
-    m::Int
-    n::Int
-    nzmax::Int
+    m::Csize_t
+    n::Csize_t
+    nzmax::Csize_t
     ppt::Ptr{Ti}
     ipt::Ptr{Ti}
     nzpt::Ptr{Void}
@@ -321,7 +331,7 @@ function CholmodDense!{T<:CHMVTypes}(aa::VecOrMat{T}) # uses the memory from Jul
                  length(size(aa)) == 2 ? aa : reshape(aa, (m,n)))
 end
 
-## The CholmodDense constructor copies the contents             
+## The CholmodDense constructor copies the contents
 function CholmodDense{T<:CHMVTypes}(aa::VecOrMat{T})
     m = size(aa,1); n = size(aa,2)
     acp = length(size(aa)) == 2 ? copy(aa) : reshape(copy(aa), (m,n))
@@ -346,7 +356,7 @@ function isvalid{T<:CHMVTypes}(cd::CholmodDense{T})
                (Ptr{c_CholmodDense{T}}, Ptr{Uint8}), &cd.c, cmn(Int32)))
 end
 
-function chm_eye{T<:Union(Float64,Complex128)}(m::Integer, n::Integer, t::T)
+function chm_eye{T<:CHMVTypes}(m::Integer, n::Integer, t::T)
     CholmodDense(ccall((:cholmod_eye, :libcholmod), Ptr{c_CholmodDense{T}},
                        (Int, Int, Cint, Ptr{Uint8}),
                        m, n,xtyp(T),cmn(Int32)))
@@ -354,14 +364,14 @@ end
 chm_eye(m::Integer, n::Integer) = chm_eye(m, n, 1.)
 chm_eye(n::Integer) = chm_eye(n, n, 1.)
 
-function chm_ones{T<:Union(Float64,Complex128)}(m::Integer, n::Integer, t::T)
+function chm_ones{T<:CHMVTypes}(m::Integer, n::Integer, t::T)
     CholmodDense(ccall((:cholmod_ones, :libcholmod), Ptr{c_CholmodDense{T}},
                        (Int, Int, Cint, Ptr{Uint8}),
                        m, n, xtyp(T), cmn(Int32)))
 end
 chm_ones(m::Integer, n::Integer) = chm_ones(m, n, 1.)
 
-function chm_zeros{T<:Union(Float64,Complex128)}(m::Integer, n::Integer, t::T)
+function chm_zeros{T<:CHMVTypes}(m::Integer, n::Integer, t::T)
     CholmodDense(ccall((:cholmod_zeros, :libcholmod), Ptr{c_CholmodDense{T}},
                        (Int, Int, Cint, Ptr{Uint8}),
                        m, n, xtyp(T), cmn(Int32)))
@@ -369,6 +379,8 @@ end
 chm_zeros(m::Integer, n::Integer) = chm_zeros(m, n, 1.)
 
 function chm_print{T<:CHMVTypes}(cd::CholmodDense{T}, lev::Integer, nm::ASCIIString)
+    @isok isvalid(cd)
+
     cm = cmn(Int32)
     orig = cm[chm_prt_inds]
     cm[chm_prt_inds] = reinterpret(Uint8, [int32(lev)])
@@ -387,7 +399,7 @@ function copy{Tv<:CHMVTypes}(B::CholmodDense{Tv})
 end
 
 function norm{Tv<:CHMVTypes}(D::CholmodDense{Tv},p::Real=1)
-    ccall((:cholmod_norm_dense, :libcholmod), Float64, 
+    ccall((:cholmod_norm_dense, :libcholmod), Float64,
           (Ptr{c_CholmodDense{Tv}}, Cint, Ptr{Uint8}),
           &D.c, p == 1 ? 1 :(p == Inf ? 1 : throw(ArgumentError("p must be 1 or Inf"))),cmn(Int32))
 end
@@ -402,7 +414,7 @@ function CholmodSparse!{Tv<:CHMVTypes,Ti<:CHMITypes}(colpt::Vector{Ti},
     if bb != 0 && bb != 1 throw(DimensionMismatch("colpt[1] is $bb, must be 0 or 1")) end
     if any(diff(colpt) .< 0) throw(ArgumentError("elements of colpt must be non-decreasing")) end
     if length(colpt) != n + 1 throw(DimensionMismatch("length(colptr) = $(length(colpt)), should be $(n+1)")) end
-    if bool(bb)                         # one-based 
+    if bool(bb)                         # one-based
         decrement!(colpt)
         decrement!(rowval)
     end
@@ -414,13 +426,19 @@ function CholmodSparse!{Tv<:CHMVTypes,Ti<:CHMITypes}(colpt::Vector{Ti},
         throw(ArgumentError("all elements of rowval must be in the range [0,$(m-1)]"))
     end
     it = ityp(Ti)
-    sort!(CholmodSparse(c_CholmodSparse{Tv,Ti}(m,n,int(nz),convert(Ptr{Ti},colpt),
+    cs = CholmodSparse(c_CholmodSparse{Tv,Ti}(m,n,int(nz),convert(Ptr{Ti},colpt),
                                                convert(Ptr{Ti},rowval), C_NULL,
                                                convert(Ptr{Tv},nzval), C_NULL,
                                                int32(stype), ityp(Ti),
                                                xtyp(Tv),dtyp(Tv),
                                                CHOLMOD_FALSE,CHOLMOD_TRUE),
-                        colpt,rowval,nzval))
+                        colpt,rowval,nzval)
+
+    @isok isvalid(cs)
+
+    cs = sort!(cs)
+
+    return cs
 end
 function CholmodSparse{Tv<:CHMVTypes,Ti<:CHMITypes}(colpt::Vector{Ti},
                                                     rowval::Vector{Ti},
@@ -456,7 +474,7 @@ CholmodSparse{Tv<:CHMVTypes}(D::CholmodDense{Tv}) = CholmodSparse(D,1) # default
 function CholmodTriplet{Tv<:CHMVTypes,Ti<:CHMITypes}(tp::Ptr{c_CholmodTriplet{Tv,Ti}})
     ctp = unsafe_load(tp)
     i = pointer_to_array(ctp.i, (ctp.nnz,), true)
-    j = pointer_to_array(ctp.j, (ctp.nnz,), true)    
+    j = pointer_to_array(ctp.j, (ctp.nnz,), true)
     x = pointer_to_array(ctp.x, (ctp.x == C_NULL ? 0 : ctp.nnz), true)
     ct = CholmodTriplet{Tv,Ti}(ctp, i, j, x)
     c_free(tp)
@@ -471,101 +489,111 @@ function chm_rdsp(fnm::String)
     CholmodSparse(res)
 end
 
-for Ti in (:Int32,:Int64)
+for Ti in CholmodIndexTypes
     @eval begin
-        function (*){Tv<:CHMVTypes}(A::CholmodSparse{Tv,$Ti},
+        function (*){Tv<:CHMVRealTypes}(A::CholmodSparse{Tv,$Ti},
                                     B::CholmodSparse{Tv,$Ti})
             CholmodSparse(ccall((@chm_nm "ssmult" $Ti
                                  , :libcholmod), Ptr{c_CholmodSparse{Tv,$Ti}},
                                 (Ptr{c_CholmodSparse{Tv,$Ti}},Ptr{c_CholmodSparse{Tv,$Ti}},
                                  Cint,Cint,Cint,Ptr{Uint8}), &A.c,&B.c,0,true,true,cmn($Ti)))
         end
-        function A_mul_Bc{Tv<:Union(Float32,Float64)}(A::CholmodSparse{Tv,$Ti},
+        function A_mul_Bc{Tv<:CHMVRealTypes}(A::CholmodSparse{Tv,$Ti},
                                                       B::CholmodSparse{Tv,$Ti})
             cm = cmn($Ti)
             aa = Array(Ptr{c_CholmodSparse{Tv,$Ti}}, 2)
             if !is(A,B)
                 aa[1] = ccall((@chm_nm "transpose" $Ti
                                ,:libcholmod), Ptr{c_CholmodSparse{Tv,$Ti}},
-                              (Ptr{c_CholmodSparse{Tv,$Ti}}, Ptr{Uint8}),
-                              &B.c,cm)
+                              (Ptr{c_CholmodSparse{Tv,$Ti}}, Cint, Ptr{Uint8}),
+                              &B.c, 1, cm)
+                ## result of ssmult will have stype==0, contain numerical values and be sorted
                 aa[2] = ccall((@chm_nm "ssmult" $Ti
                                ,:libcholmod), Ptr{c_CholmodSparse{Tv,$Ti}},
                               (Ptr{c_CholmodSparse{Tv,$Ti}},
-                               Ptr{c_CholmodSparse{Tv,$Ti}}, Ptr{Uint8}),
-                              &A.c, aa[1], cmn($Ti))
+                               Ptr{c_CholmodSparse{Tv,$Ti}},Cint,Cint,Cint,Ptr{Uint8}),
+                              &A.c, aa[1], 0, true, true, cm)
                 @isok ccall((@chm_nm "free_sparse" $Ti
                                 ,:libcholmod), Cint,
                                (Ptr{Ptr{c_CholmodSparse{Tv,$Ti}}}, Ptr{Uint8}), aa, cm)
-                CholmodSparse(aa[2])
+                return CholmodSparse(aa[2])
             end
-            ## The A*A' case is handled by cholmod_aat. Strangely the matrix returned by
-            ## cholmod_aat is not marked as symmetric.  The code following the call to
-            ## cholmod_aat is to create the symmetric-storage version of the result then
-            ## transpose it to provide sorted columns.  The result is stored in the upper
-            ## triangle
-            aa[1] = ccall((@chm_nm "aat" $Ti
-                           , :libcholmod), Ptr{c_CholmodSparse{Tv,$Ti}},
-                          (Ptr{c_CholmodSparse{Tv,$Ti}}, Ptr{Void}, Int, Cint, Ptr{Uint8}),
-                          &A.c, C_NULL, 0, 1, cm)
-            ## Create the lower triangle unsorted
-            aa[2] = ccall((@chm_nm "copy" $Ti
-                           , :libcholmod), Ptr{c_CholmodSparse{Tv,$Ti}},
-                          (Ptr{c_CholmodSparse{Tv,$Ti}}, Cint, Cint, Ptr{Uint8}),
-                          aa[1], -1, 1, cm)
-            @isok ccall((@chm_nm "free_sparse" $Ti
-                            , :libcholmod), Cint,
-                           (Ptr{Ptr{c_CholmodSparse{Tv,$Ti}}}, Ptr{Uint8}), aa, cm)
-            aa[1] = aa[2]
-            r = unsafe_load(aa[1])
-            ## Now transpose the lower triangle to the upper triangle to do the sorting
-            rpt = ccall((@chm_nm "allocate_sparse" $Ti
-                         ,:libcholmod),Ptr{c_CholmodSparse{Tv,$Ti}},
-                        (Csize_t,Csize_t,Csize_t,Cint,Cint,Cint,Cint,Ptr{Cuchar}),
-                        r.m,r.n,r.nzmax,r.sorted,r.packed,-r.stype,r.xtype,cm)
-            @isok ccall((@chm_nm "transpose_sym" $Ti
-                            ,:libcholmod),Cint,
-                           (Ptr{c_CholmodSparse{Tv,$Ti}}, Cint, Ptr{$Ti},
-                            Ptr{c_CholmodSparse{Tv,$Ti}}, Ptr{Uint8}),
-                           aa[1],1,C_NULL,rpt,cm)
-            @isok ccall((@chm_nm "free_sparse" $Ti
-                            , :libcholmod), Cint,
-                           (Ptr{Ptr{c_CholmodSparse{Tv,$Ti}}}, Ptr{Uint8}), aa, cm)
-            CholmodSparse(rpt)
+
+            ## The A*A' case is handled by cholmod_aat. This routine requires
+            ## A->stype == 0 (storage of upper and lower parts). If neccesary
+            ## the matrix A is first converted to stype == 0
+            if A.c.stype != 0
+                aa[1] = ccall((@chm_nm "copy" $Ti
+                               , :libcholmod), Ptr{c_CholmodSparse{Tv,$Ti}},
+                              (Ptr{c_CholmodSparse{Tv,$Ti}}, Cint, Cint, Ptr{Uint8}),
+                              &A.c, 0, 1, cm)
+
+                aa[2] = ccall((@chm_nm "aat" $Ti
+                               , :libcholmod), Ptr{c_CholmodSparse{Tv,$Ti}},
+                              (Ptr{c_CholmodSparse{Tv,$Ti}}, Ptr{Void}, Int, Cint, Ptr{Uint8}),
+                              aa[1], C_NULL, 0, 1, cm)
+
+                @isok ccall((@chm_nm "free_sparse" $Ti
+                             , :libcholmod), Cint,
+                            (Ptr{Ptr{c_CholmodSparse{Tv,$Ti}}}, Ptr{Uint8}), aa, cm)
+
+            else
+                aa[2] = ccall((@chm_nm "aat" $Ti
+                               , :libcholmod), Ptr{c_CholmodSparse{Tv,$Ti}},
+                              (Ptr{c_CholmodSparse{Tv,$Ti}}, Ptr{Void}, Int, Cint, Ptr{Uint8}),
+                              &A.c, C_NULL, 0, 1, cm)
+            end
+
+            cs = CholmodSparse(aa[2])
+
+            ## According to the docs conversion to symmetric matrix can be done
+            ## by changing the stype of the result (stype == 1: upper part).
+            cs.c.stype = 1
+
+            return sort!(cs)
         end
-        function Ac_mul_B{Tv<:Union(Float32,Float64)}(A::CholmodSparse{Tv,$Ti},
-                                                      B::CholmodSparse{Tv,$Ti})
+        function Ac_mul_B{Tv<:CHMVRealTypes}(A::CholmodSparse{Tv,$Ti},
+                                              B::CholmodSparse{Tv,$Ti})
             cm = cmn($Ti)
             aa = Array(Ptr{c_CholmodSparse{Tv,$Ti}}, 2)
             aa[1] = ccall((@chm_nm "transpose" $Ti
                            ,:libcholmod), Ptr{c_CholmodSparse{Tv,$Ti}},
-                          (Ptr{c_CholmodSparse{Tv,$Ti}}, Ptr{Uint8}),
-                          &B.c,cm)
+                          (Ptr{c_CholmodSparse{Tv,$Ti}}, Cint, Ptr{Uint8}),
+                          &A.c, 1, cm)
             if is(A,B)
                 Ac = CholmodSparse(aa[1])
                 return A_mul_Bc(Ac,Ac)
             end
+            ## result of ssmult will have stype==0, contain numerical values and be sorted
             aa[2] = ccall((@chm_nm "ssmult" $Ti
                            ,:libcholmod), Ptr{c_CholmodSparse{Tv,$Ti}},
                           (Ptr{c_CholmodSparse{Tv,$Ti}},
-                           Ptr{c_CholmodSparse{Tv,$Ti}}, Ptr{Uint8}),
-                          aa[1],&B.c,cm)
+                           Ptr{c_CholmodSparse{Tv,$Ti}},Cint,Cint,Cint,Ptr{Uint8}),
+                          aa[1],&B.c,0,true,true,cm)
             @isok ccall((@chm_nm "free_sparse" $Ti
                             , :libcholmod), Cint,
                            (Ptr{Ptr{c_CholmodSparse{Tv,$Ti}}}, Ptr{Uint8}), aa, cm)
             CholmodSparse(aa[2])
         end
+        function A_mul_Bt{Tv<:CHMVRealTypes}(A::CholmodSparse{Tv,$Ti},
+                                              B::CholmodSparse{Tv,$Ti})
+            A_mul_Bc(A,B) # in the unlikely event of writing A*B.' instead of A*B'
+        end
+        function At_mul_B{Tv<:CHMVRealTypes}(A::CholmodSparse{Tv,$Ti},
+                                              B::CholmodSparse{Tv,$Ti})
+            Ac_mul_B(A,B) # in the unlikely event of writing A.'*B instead of A'*B
+        end
         function CholmodDense{Tv<:CHMVTypes}(A::CholmodSparse{Tv,$Ti})
             CholmodDense(ccall((@chm_nm "sparse_to_dense" $Ti
-                                ,:libcholmod), Ptr{c_CholmodDense{Tv,$Ti}},
+                                ,:libcholmod), Ptr{c_CholmodDense{Tv}},
                                (Ptr{c_CholmodSparse{Tv,Ti}},Ptr{Uint8}),
                                &A.c,cmn($Ti)))
         end
         function CholmodSparse{Tv<:CHMVTypes}(D::CholmodDense{Tv},i::$Ti)
             CholmodSparse(ccall((@chm_nm "dense_to_sparse" $Ti
                                  ,:libcholmod), Ptr{c_CholmodSparse{Tv,$Ti}},
-                                (Ptr{c_CholmodDense{Tv,$Ti}},Ptr{Uint8}),
-                                &D.c,cmn($Ti)))
+                                (Ptr{c_CholmodDense{Tv}},Cint,Ptr{Uint8}),
+                                &D.c,true,cmn($Ti)))
         end
         function CholmodSparse{Tv<:CHMVTypes}(L::CholmodFactor{Tv,$Ti})
             if bool(L.c.is_ll)
@@ -574,7 +602,7 @@ for Ti in (:Int32,:Int64)
                                            (Ptr{c_CholmodFactor{Tv,$Ti}},Ptr{Uint8}),
                                            &L.c,cmn($Ti)))
             end
-            cm = cmn($Ti)                               
+            cm = cmn($Ti)
             Lcll = ccall((@chm_nm "copy_factor" $Ti
                           ,:libcholmod), Ptr{c_CholmodFactor{Tv,$Ti}},
                          (Ptr{c_CholmodFactor{Tv,$Ti}},Ptr{Uint8}),
@@ -681,6 +709,8 @@ for Ti in (:Int32,:Int64)
             L
         end
         function chm_print{Tv<:CHMVTypes}(L::CholmodFactor{Tv,$Ti},lev,nm)
+            @isok isvalid(L)
+
             cm = cmn($Ti)
             orig = cm[chm_prt_inds]
             cm[chm_prt_inds] = reinterpret(Uint8, [int32(lev)])
@@ -691,6 +721,8 @@ for Ti in (:Int32,:Int64)
             cm[chm_prt_inds] = orig
         end
         function chm_print{Tv<:CHMVTypes}(A::CholmodSparse{Tv,$Ti},lev,nm)
+            @isok isvalid(A)
+
             cm = cmn($Ti)
             orig = cm[chm_prt_inds]
             cm[chm_prt_inds] = reinterpret(Uint8, [int32(lev)])
@@ -784,7 +816,7 @@ for Ti in (:Int32,:Int64)
         end
         function norm{Tv<:CHMVTypes}(A::CholmodSparse{Tv,$Ti},p::Real)
             ccall((@chm_nm "norm_sparse" $Ti
-                   , :libcholmod), Float64, 
+                   , :libcholmod), Float64,
                   (Ptr{c_CholmodSparse{Tv,$Ti}}, Cint, Ptr{Uint8}),
                   &A.c,p == 1 ? 1 :(p == Inf ? 1 : throw(ArgumentError("p must be 1 or Inf"))),cmn($Ti))
         end
@@ -834,21 +866,27 @@ for Ti in (:Int32,:Int64)
         end
     end
 end
+
 (*){Tv<:CHMVTypes}(A::CholmodSparse{Tv},B::CholmodDense{Tv}) = chm_sdmult(A,false,1.,0.,B)
 (*){Tv<:CHMVTypes}(A::CholmodSparse{Tv},B::VecOrMat{Tv}) = chm_sdmult(A,false,1.,0.,CholmodDense(B))
 
-(\){T<:CHMVTypes}(L::CholmodFactor{T},B::CholmodDense{T}) = solve(L,B,CHOLMOD_A)
-(\){T<:CHMVTypes}(L::CholmodFactor{T},B::VecOrMat{T}) = solve(L,CholmodDense!(B),CHOLMOD_A).mat
+function Ac_mul_B{Tv<:CHMVTypes}(A::CholmodSparse{Tv},B::CholmodDense{Tv})
+    chm_sdmult(A,true,1.,0.,B)
+end
+function Ac_mul_B{Tv<:CHMVTypes}(A::CholmodSparse{Tv},B::VecOrMat{Tv})
+    chm_sdmult(A,true,1.,0.,CholmodDense(B))
+end
+
+
+A_ldiv_B!(L::CholmodFactor, B) = L\B # Revisit this to see if allocation can be avoided. It should be possible at least for the right hand side.
+(\){T<:CHMVTypes}(L::CholmodFactor{T}, B::CholmodDense{T}) = solve(L, B, CHOLMOD_A)
+(\){T<:CHMVTypes}(L::CholmodFactor{T}, b::Vector{T}) = reshape(solve(L, CholmodDense!(b), CHOLMOD_A).mat, length(b))
+(\){T<:CHMVTypes}(L::CholmodFactor{T}, B::Matrix{T}) = solve(L, CholmodDense!(B),CHOLMOD_A).mat
 function (\){Tv<:CHMVTypes,Ti<:CHMITypes}(L::CholmodFactor{Tv,Ti},B::CholmodSparse{Tv,Ti})
     solve(L,B,CHOLMOD_A)
 end
 function (\){Tv<:CHMVTypes,Ti<:CHMITypes}(L::CholmodFactor{Tv,Ti},B::SparseMatrixCSC{Tv,Ti})
-    sparse!(solve(L,CholmodSparse(B),CHOLMOD_A))
-end
-
-function A_mul_Bt{Tv<:Union(Float32,Float64),Ti<:CHMITypes}(A::CholmodSparse{Tv,Ti},
-                                                            B::CholmodSparse{Tv,Ti})
-    A_mul_Bc(A,B) # in the unlikely event of writing A*B.' instead of A*B'
+    sparse!(solve(L,CholmodSparse(B, 0),CHOLMOD_A))
 end
 
 Ac_ldiv_B{T<:CHMVTypes}(L::CholmodFactor{T},B::CholmodDense{T}) = solve(L,B,CHOLMOD_A)
@@ -859,21 +897,10 @@ end
 function Ac_ldiv_B{Tv<:CHMVTypes,Ti<:CHMITypes}(L::CholmodFactor{Tv,Ti},B::SparseMatrixCSC{Tv,Ti})
     sparse!(solve(L,CholmodSparse(B),CHOLMOD_A))
 end
- 
-function Ac_mul_B{Tv<:CHMVTypes}(A::CholmodSparse{Tv},B::CholmodDense{Tv})
-    chm_sdmult(A,true,1.,0.,B)
-end
-function Ac_mul_B{Tv<:CHMVTypes}(A::CholmodSparse{Tv},B::VecOrMat{Tv})
-    chm_sdmult(A,true,1.,0.,CholmodDense(B))
-end
 
-function At_mul_B{Tv<:Union(Float32,Float64),Ti<:CHMITypes}(A::CholmodSparse{Tv,Ti},
-                                                            B::CholmodSparse{Tv,Ti})
-    Ac_mul_B(A,B) # in the unlikely event of writing A.'*B instead of A'*B
-end
 
 cholfact{T<:CHMVTypes}(A::CholmodSparse{T},beta::T) = cholfact(A,beta,false)
-cholfact(A::CholmodSparse) = cholfact(A,false) 
+cholfact(A::CholmodSparse) = cholfact(A,false)
 cholfact(A::SparseMatrixCSC,ll::Bool) = cholfact(CholmodSparse(A),ll)
 cholfact(A::SparseMatrixCSC) = cholfact(CholmodSparse(A),false)
 function cholfact!{T<:CHMVTypes}(L::CholmodFactor{T},A::CholmodSparse{T},beta::Number)
@@ -912,7 +939,7 @@ end
 scale{T<:CHMVTypes}(A::CholmodSparse{T},b::Vector{T}) = scale!(copy(A), b)
 
 norm(A::CholmodSparse) = norm(A,1)
-                          
+
 show(io::IO,L::CholmodFactor) = chm_print(L,int32(4),"")
 show(io::IO,A::CholmodSparse) = chm_print(A,int32(4),"")
 
@@ -1030,7 +1057,7 @@ function sparse(A::CholmodSparse)
 end
 function sparse!(A::CholmodSparse)
     SparseMatrixCSC(A.c.m, A.c.n, increment!(A.colptr0), increment!(A.rowval0), A.nzval)
-end    
+end
 sparse(L::CholmodFactor) = sparse!(CholmodSparse(L))
 sparse(D::CholmodDense) = sparse!(CholmodSparse(D))
 sparse(T::CholmodTriplet) = sparse!(CholmodSparse(T))
