@@ -32,10 +32,13 @@ function getindex(D::Diagonal, i::Integer)
     zero(eltype(D.diag))
 end
 
-ishermitian(D::Diagonal) = true
+ishermitian{T<:Real}(D::Diagonal{T}) = true
+ishermitian(D::Diagonal) = all(D.diag .== real(D.diag))
 issym(D::Diagonal) = true
 isposdef(D::Diagonal) = all(D.diag .> 0)
 
+factorize(D::Diagonal) = D
+
 tril!(D::Diagonal,i::Integer) = i == 0 ? D : zeros(D)
 triu!(D::Diagonal,i::Integer) = i == 0 ? D : zeros(D)
 
