@@ -199,12 +199,12 @@ end
 function A_ldiv_B!(A::SparseMatrixCSC, b::AbstractVecOrMat)
     if iseltype(b, Complex); A = complex(A); end
 
-    if istril(A) 
-        # TODO: Fix diagonal case. Diagonal(A.nzval) needs to handle 
+    if istril(A)
+        # TODO: Fix diagonal case. Diagonal(A.nzval) needs to handle
         # the case where there are zeros on the diagonal and error out.
         # It also does not work in the complex case. VBS.
         #if istriu(A); return A_ldiv_B!(Diagonal(A.nzval), b); end
-        return fwdTriSolve!(A, b) 
+        return fwdTriSolve!(A, b)
     end
     if istriu(A); return bwdTriSolve!(A, b); end
     return A_ldiv_B!(lufact(A),b)
@@ -223,11 +223,11 @@ function fwdTriSolve!(A::SparseMatrixCSC, B::AbstractVecOrMat)
     if nrowB != ncol
         throw(DimensionMismatch("A is $(ncol)X$(ncol) and B has length $(n)"))
     end
-   
+
     aa = A.nzval
     ja = A.rowval
     ia = A.colptr
-   
+
     joff = 0
     for k = 1:ncolB
         for j = 1:(nrowB-1)
@@ -261,7 +261,7 @@ function bwdTriSolve!(A::SparseMatrixCSC, B::AbstractVecOrMat)
     aa = A.nzval
     ja = A.rowval
     ia = A.colptr
-    
+
     joff = 0
     for k = 1:ncolB
         for j = nrowB:-1:2
@@ -298,7 +298,7 @@ function triu{Tv,Ti}(S::SparseMatrixCSC{Tv,Ti}, k::Integer)
     end
     rowval = Array(Ti, nnz)
     nzval = Array(Tv, nnz)
-    A = SparseMatrixCSC{Tv,Ti}(m, n, colptr, rowval, nzval)
+    A = SparseMatrixCSC(m, n, colptr, rowval, nzval)
     for col = max(k+1,1) : n
         c1 = S.colptr[col]
         for c2 = A.colptr[col] : A.colptr[col+1]-1
@@ -328,7 +328,7 @@ function tril{Tv,Ti}(S::SparseMatrixCSC{Tv,Ti}, k::Integer)
     end
     rowval = Array(Ti, nnz)
     nzval = Array(Tv, nnz)
-    A = SparseMatrixCSC{Tv,Ti}(m, n, colptr, rowval, nzval)
+    A = SparseMatrixCSC(m, n, colptr, rowval, nzval)
     for col = 1 : min(n, m+k)
         c1 = S.colptr[col+1]-1
         l2 = A.colptr[col+1]-1
@@ -345,7 +345,7 @@ end
 
 function sparse_diff1{Tv,Ti}(S::SparseMatrixCSC{Tv,Ti})
     m,n = size(S)
-    m > 1 || return SparseMatrixCSC{Tv,Ti}(0, n, ones(n+1), Ti[], Tv[])
+    m > 1 || return SparseMatrixCSC(0, n, ones(Ti,n+1), Ti[], Tv[])
     colptr = Array(Ti, n+1)
     numnz = 2 * nnz(S) # upper bound; will shrink later
     rowval = Array(Ti, numnz)
@@ -380,7 +380,7 @@ function sparse_diff1{Tv,Ti}(S::SparseMatrixCSC{Tv,Ti})
     end
     deleteat!(rowval, numnz+1:length(rowval))
     deleteat!(nzval, numnz+1:length(nzval))
-    return SparseMatrixCSC{Tv,Ti}(m-1, n, colptr, rowval, nzval)
+    return SparseMatrixCSC(m-1, n, colptr, rowval, nzval)
 end
 
 function sparse_diff2{Tv,Ti}(a::SparseMatrixCSC{Tv,Ti})
@@ -400,7 +400,7 @@ function sparse_diff2{Tv,Ti}(a::SparseMatrixCSC{Tv,Ti})
     ptrS = 1
     colptr[1] = 1
 
-    n == 0 && return SparseMatrixCSC{Tv,Ti}(m, n, colptr, rowval, nzval)
+    n == 0 && return SparseMatrixCSC(m, n, colptr, rowval, nzval)
 
     startA = colptr_a[1]
     stopA = colptr_a[2]
@@ -470,7 +470,7 @@ function sparse_diff2{Tv,Ti}(a::SparseMatrixCSC{Tv,Ti})
     end
     deleteat!(rowval, ptrS:length(rowval))
     deleteat!(nzval, ptrS:length(nzval))
-    return SparseMatrixCSC{Tv,Ti}(m, n-1, colptr, rowval, nzval)
+    return SparseMatrixCSC(m, n-1, colptr, rowval, nzval)
 end
 
 diff(a::SparseMatrixCSC, dim::Integer)= dim==1 ? sparse_diff1(a) : sparse_diff2(a)
@@ -589,14 +589,25 @@ inv(A::SparseMatrixCSC) = error("The inverse of a sparse matrix can often be den
 
 ## scale methods
 
+# Copy colptr and rowval from one SparseMatrix to another
+function copyinds!(C::SparseMatrixCSC, A::SparseMatrixCSC)
+    if C.colptr !== A.colptr
+        resize!(C.colptr, length(A.colptr))
+        copy!(C.colptr, A.colptr)
+    end
+    if C.rowval !== A.rowval
+        resize!(C.rowval, length(A.rowval))
+        copy!(C.rowval, A.rowval)
+    end
+end
+
 # multiply by diagonal matrix as vector
 function scale!{Tv,Ti}(C::SparseMatrixCSC{Tv,Ti}, A::SparseMatrixCSC, b::Vector)
     m, n = size(A)
     (n==length(b) && size(A)==size(C)) || throw(DimensionMismatch(""))
     numnz = nnz(A)
-    C.colptr = convert(Array{Ti}, A.colptr)
-    C.rowval = convert(Array{Ti}, A.rowval)
-    C.nzval = Array(Tv, numnz)
+    copyinds!(C, A)
+    resize!(C.nzval, length(A.nzval))
     for col = 1:n, p = A.colptr[col]:(A.colptr[col+1]-1)
         C.nzval[p] = A.nzval[p] * b[col]
     end
@@ -605,11 +616,10 @@ end
 
 function scale!{Tv,Ti}(C::SparseMatrixCSC{Tv,Ti}, b::Vector, A::SparseMatrixCSC)
     m, n = size(A)
-    (n==length(b) && size(A)==size(C)) || throw(DimensionMismatch(""))
+    (m==length(b) && size(A)==size(C)) || throw(DimensionMismatch(""))
     numnz = nnz(A)
-    C.colptr = convert(Array{Ti}, A.colptr)
-    C.rowval = convert(Array{Ti}, A.rowval)
-    C.nzval = Array(Tv, numnz)
+    copyinds!(C, A)
+    resize!(C.nzval, length(A.nzval))
     for col = 1:n, p = A.colptr[col]:(A.colptr[col+1]-1)
         C.nzval[p] = A.nzval[p] * b[A.rowval[p]]
     end
