@@ -15,7 +15,7 @@ type MatrixIllConditionedException <: Exception
     message :: String
 end
 
-function umferror(status::Int)
+function umferror(status::Integer)
      if status==UMFPACK_OK
          return
      elseif status==UMFPACK_WARNING_singular_matrix
@@ -68,8 +68,16 @@ function increment!{T<:Integer}(A::AbstractArray{T})
 end
 increment{T<:Integer}(A::AbstractArray{T}) = increment!(copy(A))
 
+# check the size of SuiteSparse_long
+if int(ccall((:jl_cholmod_sizeof_long,:libsuitesparse_wrapper),Csize_t,())) == 4
+    const UmfpackIndexTypes = (:Int32, )
+    typealias UMFITypes Union(Int32)
+else
+    const UmfpackIndexTypes = (:Int32, :Int64)
+    typealias UMFITypes Union(Int32, Int64)
+end
+
 typealias UMFVTypes Union(Float64,Complex128)
-typealias UMFITypes Union(Int32,Int64)
 
 ## UMFPACK
 
@@ -107,7 +115,6 @@ type UmfpackLU{Tv<:UMFVTypes,Ti<:UMFITypes} <: Factorization{Tv}
 end
 
 function lufact{Tv<:UMFVTypes,Ti<:UMFITypes}(S::SparseMatrixCSC{Tv,Ti})
-    S.m == S.n || error("argument matrix must be square")
 
     zerobased = S.colptr[1] == 0
     res = UmfpackLU(C_NULL, C_NULL, S.m, S.n,
@@ -137,17 +144,22 @@ end
 
 ## Wrappers for UMFPACK functions
 
-for (sym_r,sym_c,num_r,num_c,sol_r,sol_c,det_r,det_z,lunz_r,lunz_z,get_num_r,get_num_z,itype) in
-    (("umfpack_di_symbolic","umfpack_zi_symbolic",
-      "umfpack_di_numeric","umfpack_zi_numeric",
-      "umfpack_di_solve","umfpack_zi_solve",
-      "umfpack_di_get_determinant","umfpack_zi_get_determinant",
-      "umfpack_di_get_lunz","umfpack_zi_get_lunz","umfpack_di_get_numeric","umfpack_zi_get_numeric",:Int32),
-     ("umfpack_dl_symbolic","umfpack_zl_symbolic",
-      "umfpack_dl_numeric","umfpack_zl_numeric",
-      "umfpack_dl_solve","umfpack_zl_solve",
-      "umfpack_dl_get_determinant","umfpack_zl_get_determinant",
-      "umfpack_dl_get_lunz","umfpack_zl_get_lunz","umfpack_dl_get_numeric","umfpack_zl_get_numeric",:Int64))
+# generate the name of the C function according to the value and integer types
+umf_nm(nm,Tv,Ti) = "umfpack_" * (Tv == :Float64 ? "d" : "z") * (Ti == :Int64 ? "l_" : "i_") * nm
+
+for itype in UmfpackIndexTypes
+    sym_r = umf_nm("symbolic", :Float64, itype)
+    sym_c = umf_nm("symbolic", :Complex128, itype)
+    num_r = umf_nm("numeric", :Float64, itype)
+    num_c = umf_nm("numeric", :Complex128, itype)
+    sol_r = umf_nm("solve", :Float64, itype)
+    sol_c = umf_nm("solve", :Complex128, itype)
+    det_r = umf_nm("get_determinant", :Float64, itype)
+    det_z = umf_nm("get_determinant", :Complex128, itype)
+    lunz_r = umf_nm("get_lunz", :Float64, itype)
+    lunz_z = umf_nm("get_lunz", :Complex128, itype)
+    get_num_r = umf_nm("get_numeric", :Float64, itype)
+    get_num_z = umf_nm("get_numeric", :Complex128, itype)
     @eval begin
         function umfpack_symbolic!(U::UmfpackLU{Float64,$itype})
             if U.symbolic != C_NULL return U end
@@ -176,7 +188,7 @@ for (sym_r,sym_c,num_r,num_c,sol_r,sol_c,det_r,det_z,lunz_r,lunz_z,get_num_r,get
             if U.symbolic == C_NULL umfpack_symbolic!(U) end
             tmp = Array(Ptr{Void}, 1)
             status = ccall(($num_r, :libumfpack), $itype,
-                           (Ptr{$itype}, Ptr{$itype}, Ptr{Float64}, Ptr{Void}, Ptr{Void}, 
+                           (Ptr{$itype}, Ptr{$itype}, Ptr{Float64}, Ptr{Void}, Ptr{Void},
                             Ptr{Float64}, Ptr{Float64}),
                            U.colptr, U.rowval, U.nzval, U.symbolic, tmp,
                            umf_ctrl, umf_info)
@@ -190,7 +202,7 @@ for (sym_r,sym_c,num_r,num_c,sol_r,sol_c,det_r,det_z,lunz_r,lunz_z,get_num_r,get
             if U.symbolic == C_NULL umfpack_symbolic!(U) end
             tmp = Array(Ptr{Void}, 1)
             status = ccall(($num_c, :libumfpack), $itype,
-                           (Ptr{$itype}, Ptr{$itype}, Ptr{Float64}, Ptr{Float64}, Ptr{Void}, Ptr{Void}, 
+                           (Ptr{$itype}, Ptr{$itype}, Ptr{Float64}, Ptr{Float64}, Ptr{Void}, Ptr{Void},
                             Ptr{Float64}, Ptr{Float64}),
                            U.colptr, U.rowval, real(U.nzval), imag(U.nzval), U.symbolic, tmp,
                            umf_ctrl, umf_info)
@@ -282,8 +294,8 @@ for (sym_r,sym_c,num_r,num_c,sol_r,sol_c,det_r,det_z,lunz_r,lunz_z,get_num_r,get
         end
         function umf_extract(lu::UmfpackLU{Float64,$itype})
             umfpack_numeric!(lu)        # ensure the numeric decomposition exists
-            (lnz,unz,n_row,n_col,nz_diag) = umf_lunz(lu)
-            Lp = Array($itype, n_col + 1)
+            (lnz, unz, n_row, n_col, nz_diag) = umf_lunz(lu)
+            Lp = Array($itype, n_row + 1)
             Lj = Array($itype, lnz) # L is returned in CSR (compressed sparse row) format
             Lx = Array(Float64, lnz)
             Up = Array($itype, n_col + 1)
@@ -301,14 +313,14 @@ for (sym_r,sym_c,num_r,num_c,sol_r,sol_c,det_r,det_z,lunz_r,lunz_z,get_num_r,get
                         Up,Ui,Ux,
                         P, Q, C_NULL,
                         &0, Rs, lu.numeric)
-            (transpose(SparseMatrixCSC(n_row,n_row,increment!(Lp),increment!(Lj),Lx)),
-             SparseMatrixCSC(n_row,n_col,increment!(Up),increment!(Ui),Ux),
+            (transpose(SparseMatrixCSC(min(n_row, n_col), n_row, increment!(Lp), increment!(Lj), Lx)),
+             SparseMatrixCSC(min(n_row, n_col), n_col, increment!(Up), increment!(Ui), Ux),
              increment!(P), increment!(Q), Rs)
         end
         function umf_extract(lu::UmfpackLU{Complex128,$itype})
             umfpack_numeric!(lu)        # ensure the numeric decomposition exists
-            (lnz,unz,n_row,n_col,nz_diag) = umf_lunz(lu)
-            Lp = Array($itype, n_col + 1)
+            (lnz, unz, n_row, n_col, nz_diag) = umf_lunz(lu)
+            Lp = Array($itype, n_row + 1)
             Lj = Array($itype, lnz) # L is returned in CSR (compressed sparse row) format
             Lx = Array(Float64, lnz)
             Lz = Array(Float64, lnz)
@@ -328,8 +340,8 @@ for (sym_r,sym_c,num_r,num_c,sol_r,sol_c,det_r,det_z,lunz_r,lunz_z,get_num_r,get
                         Up,Ui,Ux,Uz,
                         P, Q, C_NULL, C_NULL,
                         &0, Rs, lu.numeric)
-            (transpose(SparseMatrixCSC(n_row,n_row,increment!(Lp),increment!(Lj),complex(Lx,Lz))),
-             SparseMatrixCSC(n_row,n_col,increment!(Up),increment!(Ui),complex(Ux,Uz)),
+            (transpose(SparseMatrixCSC(min(n_row, n_col), n_row, increment!(Lp), increment!(Lj), complex(Lx, Lz))),
+             SparseMatrixCSC(min(n_row, n_col), n_col, increment!(Up), increment!(Ui), complex(Ux, Uz)),
              increment!(P), increment!(Q), Rs)
         end
     end
@@ -368,36 +380,40 @@ function getindex(lu::UmfpackLU, d::Symbol)
         (d == :(:) ? (L,U,p,q,Rs) :
          throw(KeyError(d)))))))
 end
- 
-## The C functions called by these Julia functions do not depend on
-## the numeric and index types, even though the umfpack names indicate
-## they do.  The umfpack_free_* functions can be called on C_NULL without harm.
-function umfpack_free_symbolic(symb::Ptr{Void})
-    tmp = [symb]
-    ccall((:umfpack_dl_free_symbolic, :libumfpack), Void, (Ptr{Void},), tmp)
+
+for Tv in (:Float64, :Complex128), Ti in UmfpackIndexTypes
+    f = symbol(umf_nm("free_symbolic", Tv, Ti))
+    @eval begin
+        function ($f)(symb::Ptr{Void})
+            tmp = [symb]
+            ccall(($(string(f)), :libumfpack), Void, (Ptr{Void},), tmp)
+        end
+
+        function umfpack_free_symbolic(lu::UmfpackLU{$Tv,$Ti})
+            if lu.symbolic == C_NULL return lu end
+            umfpack_free_numeric(lu)
+            ($f)(lu.symbolic)
+            lu.symbolic = C_NULL
+            return lu
+        end
+    end
+
+    f = symbol(umf_nm("free_numeric", Tv, Ti))
+    @eval begin
+        function ($f)(num::Ptr{Void})
+            tmp = [num]
+            ccall(($(string(f)), :libumfpack), Void, (Ptr{Void},), tmp)
+        end
+        function umfpack_free_numeric(lu::UmfpackLU{$Tv,$Ti})
+            if lu.numeric == C_NULL return lu end
+            ($f)(lu.numeric)
+            lu.numeric = C_NULL
+            return lu
+        end
+    end
 end
 show_umf_info() = show_umf_info(2.)
 
-function umfpack_free_symbolic(lu::UmfpackLU)
-    if lu.symbolic == C_NULL return lu end
-    umfpack_free_numeric(lu)
-    umfpack_free_symbolic(lu.symbolic)
-    lu.symbolic = C_NULL
-    return lu
-end
-
-function umfpack_free_numeric(num::Ptr{Void})
-    tmp = [num]
-    ccall((:umfpack_dl_free_numeric, :libumfpack), Void, (Ptr{Void},), tmp)
-end
-
-function umfpack_free_numeric(lu::UmfpackLU)
-    if lu.numeric == C_NULL return lu end
-    umfpack_free_numeric(lu.numeric)
-    lu.numeric = C_NULL
-    return lu
-end
-
 function umfpack_report_symbolic(symb::Ptr{Void}, level::Real)
     old_prl::Float64 = umf_ctrl[UMFPACK_PRL]
     umf_ctrl[UMFPACK_PRL] = float64(level)
@@ -429,4 +445,4 @@ end
 umfpack_report_numeric(lu::UmfpackLU) = umfpack_report_numeric(lu,4.)
 
 end # UMFPACK module
- 
+
