@@ -153,9 +153,11 @@ function clone(url::String, pkg::String)
         Base.rm(pkg, recursive=true)
         rethrow()
     end
-    isempty(Reqs.parse("$pkg/REQUIRE")) && return
     info("Computing changes...")
-    resolve()
+    if !edit(Reqs.add, pkg)
+        isempty(Reqs.parse("$pkg/REQUIRE")) && return
+        resolve()
+    end
 end
 
 function clone(url_or_pkg::String)
@@ -310,9 +312,7 @@ function publish(branch::String)
     ahead_remote > 0 && error("METADATA is behind origin/$branch – run `Pkg.update()` before publishing")
     ahead_local == 0 && error("There are no METADATA changes to publish")
 
-    info("Validating METADATA")
-    check_metadata()
-    tags = Dict{ASCIIString,Vector{ASCIIString}}()
+    tags = Dict{ByteString,Vector{ASCIIString}}()
     Git.run(`update-index -q --really-refresh`, dir="METADATA")
     cmd = `diff --name-only --diff-filter=AMR origin/$branch HEAD --`
     for line in eachline(Git.cmd(cmd, dir="METADATA"))
@@ -333,6 +333,8 @@ function publish(branch::String)
         end || error("$pkg v$ver is incorrectly tagged – $sha1 expected")
     end
     isempty(tags) && info("No new package versions to publish")
+    info("Validating METADATA")
+    check_metadata(Set(keys(tags)))
     @sync for pkg in sort!([keys(tags)...])
         @async begin
             forced = ASCIIString[]
@@ -370,7 +372,8 @@ function resolve(
     for pkg in keys(reqs)
         if !haskey(deps,pkg)
             if "julia" in conflicts[pkg]
-                error("$pkg can't be installed because it has no versions that support ", VERSION, " of julia")
+                error("$pkg can't be installed because it has no versions that support ", VERSION, " of julia. " *
+                   "You may need to update METADATA by running `Pkg.update()`")
             else
                 error("$pkg's requirements can't be satisfied because of the following fixed packages: ",
                    join(conflicts[pkg], ", ", " and "))
@@ -580,7 +583,7 @@ function tag(pkg::String, ver::Union(Symbol,VersionNumber), force::Bool=false, c
     end
 end
 
-function check_metadata()
+function check_metadata(pkgs::Set{ByteString} = Set{ByteString}())
     avail = Read.available()
     deps, conflicts = Query.dependencies(avail)
 
@@ -588,7 +591,7 @@ function check_metadata()
         haskey(deps, p) || error("package $dp v$v requires a non-registered package: $p")
     end
 
-    problematic = Resolve.sanity_check(deps)
+    problematic = Resolve.sanity_check(deps, pkgs)
     if !isempty(problematic)
         msg = "packages with unsatisfiable requirements found:\n"
         for (p, vn, rp) in problematic
@@ -634,7 +637,7 @@ function build(pkgs::Vector)
     WARNING: $(join(map(x->x[1],errs),", "," and ")) had build errors.
 
      - packages with build errors remain installed in $(pwd())
-     - build a package and all its dependencies with `Pkg.build(pkg)`
+     - build the package(s) and all dependencies with `Pkg.build("$(join(map(x->x[1],errs),"\", \""))")`
      - build a single package by running its `deps/build.jl` script
     """)
 end
