@@ -54,6 +54,7 @@ function package(
             Generate.entrypoint(pkg,force=force)
             Generate.tests(pkg,force=force)
             Generate.travis(pkg,force=force)
+            Generate.gitignore(pkg,force=force)
 
             msg = """
             $pkg.jl $(isnew ? "generated" : "regenerated") files.
@@ -136,23 +137,28 @@ end
 function travis(pkg::String; force::Bool=false)
     genfile(pkg,".travis.yml",force) do io
         print(io, """
-        language: cpp
-        compiler:
-          - clang
+        language: julia
+        os:
+          - linux
+          - osx
+        julia:
+          - release
+          - nightly
         notifications:
           email: false
-        env:
-          matrix:
-            - JULIAVERSION="juliareleases"
-            - JULIAVERSION="julianightlies"
-        before_install:
-          - sudo add-apt-repository ppa:staticfloat/julia-deps -y
-          - sudo add-apt-repository ppa:staticfloat/\${JULIAVERSION} -y
-          - sudo apt-get update -qq -y
-          - sudo apt-get install libpcre3-dev julia -y
-          - if [[ -a .git/shallow ]]; then git fetch --unshallow; fi
-        script:
-          - julia -e 'Pkg.init(); Pkg.clone(pwd()); Pkg.test("$pkg")'
+        # uncomment the following lines to override the default test script
+        #script:
+        #  - if [[ -a .git/shallow ]]; then git fetch --unshallow; fi
+        #  - julia --check-bounds=yes -e 'Pkg.clone(pwd()); Pkg.build("$pkg"); Pkg.test("$pkg"; coverage=true)'
+        """)
+    end
+end
+
+function gitignore(pkg::String; force::Bool=false)
+    genfile(pkg,".gitignore",force) do io
+        print(io, """
+        *.jl.cov
+        *.jl.mem
         """)
     end
 end
@@ -175,7 +181,7 @@ function genfile(f::Function, pkg::String, file::String, force::Bool=false)
         info("Generating $file")
         mkpath(dirname(path))
         open(f, path, "w")
-        Git.run(`add $file`, dir=pkg)
+        Git.run(`add -f $file`, dir=pkg)
         return true
     end
     return false
