@@ -7,18 +7,20 @@ type FileMonitor
         handle = c_malloc(_sizeof_uv_fs_event)
         err = ccall(:jl_fs_event_init,Int32, (Ptr{Void}, Ptr{Void}, Ptr{Uint8}, Int32), eventloop(),handle,file,0)
         if err < 0
-            c_free(handle)
+            ccall(:uv_fs_event_stop, Int32, (Ptr{Void},), handle)
+            disassociate_julia_struct(handle)
+            ccall(:jl_forceclose_uv, Void, (Ptr{Void},), handle)
             throw(UVError("FileMonitor",err))
         end
         this = new(handle,cb,false,Condition())
         associate_julia_struct(handle,this)
         finalizer(this,uvfinalize)
-        this        
+        this
     end
     FileMonitor(file) = FileMonitor(false,file)
 end
 
-function close(t::FileMonitor) 
+function close(t::FileMonitor)
     if t.handle != C_NULL
         ccall(:jl_close_uv,Void,(Ptr{Void},),t.handle)
     end
@@ -82,11 +84,11 @@ type PollingFileWatcher <: UVPollingWatcher
         associate_julia_struct(handle,this)
         finalizer(this,uvfinalize)
         this
-    end  
+    end
     PollingFileWatcher(file) =  PollingFileWatcher(false,file)
 end
 
-@unix_only typealias FDW_FD RawFD 
+@unix_only typealias FDW_FD RawFD
 @windows_only typealias FDW_FD WindowsRawSocket
 
 @unix_only _get_osfhandle(fd::RawFD) = fd
@@ -165,7 +167,7 @@ function _wait(fdw::FDWatcher,readable,writable)
     events
 end
 
-# On Unix we can only have one watcher per FD, so we need to keep an explicit 
+# On Unix we can only have one watcher per FD, so we need to keep an explicit
 # list of them. On Windows, I think it is techincally possible to have more than one
 # watcher per FD, but in order to keep compatibility, we do the same on windows as we do
 # on unix
@@ -187,7 +189,7 @@ let
                 fdwatcher_array[fd.fd+1] = FDWatcher(fd)
             end
             _wait(fdwatcher_array[fd.fd+1],readable,writable)
-        end 
+        end
     end
     @windows_only begin
         local fdwatcher_array
@@ -204,15 +206,15 @@ let
                 fdwatcher_array[socket] = FDWatcher(socket)
             end
             _wait(fdwatcher_array[socket],readable,writable)
-        end 
+        end
     end
 end
 
-function pfw_wait_cb(pfw::PollingFileWatcher, prev, cur, status)
+function pfw_wait_cb(pfw::PollingFileWatcher, events, status)
     if status < 0
         notify_error(pfw.notify,UVError("PollingFileWatcher",status))
     else
-        notify(pfw.notify,(prev,cur))
+        notify(pfw.notify,events)
     end
 end
 
@@ -220,11 +222,13 @@ function wait(pfw::PollingFileWatcher; interval=2.0)
     if !pfw.open
         start_watching(pfw_wait_cb,pfw,interval)
     end
-    prev,curr = stream_wait(pfw,pfw.notify)
+    events = stream_wait(pfw,pfw.notify)
     if isempty(pfw.notify.waitq)
         stop_watching(pfw)
     end
-    (prev,curr)
+    # return basename for consistency with something libuv appears to do
+    # in the FileMonitor case.
+    basename(pfw.file), events
 end
 
 function wait(m::FileMonitor)
@@ -246,13 +250,13 @@ function start_watching(t::FDWatcher, events::FDEvent)
 end
 start_watching(f::Function, t::FDWatcher, events::FDEvent) = (t.cb = f; start_watching(t,events))
 
-function start_watching(t::PollingFileWatcher, interval)
+function start_watching(t::PollingFileWatcher, interval=2.0)
     associate_julia_struct(t.handle, t)
     uv_error("start_watching (File)",
              ccall(:jl_fs_poll_start, Int32, (Ptr{Void},Ptr{Uint8},Uint32),
                    t.handle, t.file, iround(interval*1000)))
 end
-start_watching(f::Function, t::PollingFileWatcher, interval) = (t.cb = f;start_watching(t,interval))
+start_watching(f::Function, t::PollingFileWatcher, interval=2.0) = (t.cb = f;start_watching(t,interval))
 
 function stop_watching(t::FDWatcher)
     disassociate_julia_struct(t.handle)
@@ -287,7 +291,8 @@ end
 
 function _uv_hook_fspollcb(t::PollingFileWatcher,status::Int32,prev::Ptr,cur::Ptr)
     if isa(t.cb,Function)
-        t.cb(t, StatStruct(convert(Ptr{Uint8},prev)), StatStruct(convert(Ptr{Uint8},cur)), status)
+        s = StatStruct(convert(Ptr{Uint8},cur))
+        t.cb(t, FDEvent(isreadable(s), iswritable(s), false), status)
     end
 end
 
@@ -325,7 +330,7 @@ function watch_file(cb, s; poll=false)
         pfw = PollingFileWatcher(cb,s)
         start_watching(pfw)
         return pfw
-    else 
+    else
         return FileMonitor(cb,s)
     end
 end
