@@ -700,7 +700,7 @@ function fix_dec(x::SmallFloatingPoint, n::Int)
     @grisu_ccall x Grisu.FIXED n
     if LEN[1] == 0
         DIGITS[1] = '0'
-        return (int32(1), int32(1), NEG[1]) 
+        return (int32(1), int32(1), NEG[1])
     end
     return LEN[1], POINT[1], NEG[1]
 end
@@ -801,9 +801,10 @@ function bigfloat_printf(out, d, flags::ASCIIString, width::Int, precision::Int,
     write(fmt, uint8(0))
     printf_fmt = takebuf_array(fmt)
     @assert length(printf_fmt) == fmt_len
-    lng = ccall((:mpfr_snprintf,:libmpfr), Int32, (Ptr{Uint8}, Culong, Ptr{Uint8}, Ptr{BigFloat}...), DIGITS, BUFLEN-1, printf_fmt, &d)
+    bufsiz = length(DIGITS) - 1
+    lng = ccall((:mpfr_snprintf,:libmpfr), Int32, (Ptr{Uint8}, Culong, Ptr{Uint8}, Ptr{BigFloat}...), DIGITS, bufsiz, printf_fmt, &d)
     lng > 0 || error("invalid printf formatting for BigFloat")
-    write(out, pointer(DIGITS), lng)
+    write(out, pointer(DIGITS), min(lng,bufsiz))
     return (false, ())
 end
 
@@ -840,7 +841,7 @@ end
 
 macro sprintf(args...)
     !isempty(args) || error("@sprintf: called with zero arguments")
-    isa(args[1], String) || is_str_expr(args[1]) || 
+    isa(args[1], String) || is_str_expr(args[1]) ||
         error("@sprintf: first argument must be a format string")
     blk = _printf("@sprintf", :(IOBuffer()), args[1], args[2:end])
     push!(blk.args, :(takebuf_string(out)))
