@@ -24,7 +24,7 @@ type MersenneTwister <: AbstractRNG
     MersenneTwister(seed=0) = MersenneTwister(make_seed(seed))
 end
 
-function srand(r::MersenneTwister, seed) 
+function srand(r::MersenneTwister, seed)
     r.seed = seed
     dsfmt_init_gen_rand(r.state, seed)
     return r
@@ -212,7 +212,7 @@ rand{T}(r::Range{T}) = r[rand(1:(length(r)))]
 function rand!(g::RandIntGen, A::AbstractArray)
     for i = 1 : length(A)
         @inbounds A[i] = rand(g)
-    end    
+    end
     return A
 end
 
@@ -243,76 +243,76 @@ randbool!(B::BitArray) = rand!(B)
 ## randn() - Normally distributed random numbers using Ziggurat algorithm
 
 # The Ziggurat Method for generating random variables - Marsaglia and Tsang
-# Paper and reference code: http://www.jstatsoft.org/v05/i08/ 
+# Paper and reference code: http://www.jstatsoft.org/v05/i08/
 
 # randmtzig (covers also exponential variates)
 ## Tables for normal variates
-const ki = 
-    Uint64[0x0007799ec012f7b3,                 0,0x0006045f4c7de363,0x0006d1aa7d5ec0a6,
-           0x000728fb3f60f778,0x0007592af4e9fbc0,0x000777a5c0bf655d,0x00078ca3857d2256,
-           0x00079bf6b0ffe58c,0x0007a7a34ab092ae,0x0007b0d2f20dd1cb,0x0007b83d3aa9cb52,
-           0x0007be597614224e,0x0007c3788631abea,0x0007c7d32bc192ef,0x0007cb9263a6e86d,
+const ki =
+    Uint64[0x0007799ec012f7b2,0x0000000000000000,0x0006045f4c7de363,0x0006d1aa7d5ec0a5,
+           0x000728fb3f60f777,0x0007592af4e9fbc0,0x000777a5c0bf655d,0x00078ca3857d2256,
+           0x00079bf6b0ffe58b,0x0007a7a34ab092ad,0x0007b0d2f20dd1cb,0x0007b83d3aa9cb52,
+           0x0007be597614224d,0x0007c3788631abe9,0x0007c7d32bc192ee,0x0007cb9263a6e86d,
            0x0007ced483edfa84,0x0007d1b07ac0fd39,0x0007d437ef2da5fc,0x0007d678b069aa6e,
-           0x0007d87db38c5c87,0x0007da4fc6a9ba63,0x0007dbf611b37f3c,0x0007dd7674d0f286,
-           0x0007ded5ce8205f7,0x0007e018307fb62c,0x0007e141081bd124,0x0007e2533d712de9,
-           0x0007e3514bbd7718,0x0007e43d54944b52,0x0007e5192f25ef43,0x0007e5e67481118d,
-           0x0007e6a6897c1ce2,0x0007e75aa6c7f64d,0x0007e803df8ee498,0x0007e8a326eb6273,
-           0x0007e93954717a28,0x0007e9c727f8648f,0x0007ea4d4cc85a3d,0x0007eacc5c4907a9,
-           0x0007eb44e0474cf6,0x0007ebb754e47419,0x0007ec242a3d8474,0x0007ec8bc5d69646,
-           0x0007ecee83d3d6e9,0x0007ed4cb8082f46,0x0007eda6aee01710,0x0007edfcae2dfe68,
-           0x0007ee4ef5dccd3f,0x0007ee9dc08c394f,0x0007eee9441a17c8,0x0007ef31b21b4fb2,
-           0x0007ef773846a8a7,0x0007efba00d35a17,0x0007effa32ccf69f,0x0007f037f25e1279,
-           0x0007f0736112d12c,0x0007f0ac9e145c26,0x0007f0e3c65e1fcc,0x0007f118f4ed8e55,
-           0x0007f14c42ed0dc9,0x0007f17dc7daa0c3,0x0007f1ad99aac6a5,0x0007f1dbcce80015,
-           0x0007f20874cf56bf,0x0007f233a36a3b9a,0x0007f25d69a604ae,0x0007f285d7694a92,
-           0x0007f2acfba75e3c,0x0007f2d2e4720909,0x0007f2f79f09c345,0x0007f31b37ec883c,
-           0x0007f33dbae36abc,0x0007f35f330f08d5,0x0007f37faaf2fa79,0x0007f39f2c805381,
-           0x0007f3bdc11f4f1d,0x0007f3db71b83851,0x0007f3f846bba121,0x0007f4144829f846,
-           0x0007f42f7d9a8b9e,0x0007f449ee420432,0x0007f463a0f8675e,0x0007f47c9c3ea77b,
-           0x0007f494e643cd8e,0x0007f4ac84e9c475,0x0007f4c37dc9cd51,0x0007f4d9d638a433,
-           0x0007f4ef934a5b6b,0x0007f504b9d5f33e,0x0007f5194e78b352,0x0007f52d55994a96,
-           0x0007f540d36aba0d,0x0007f553cbef0e78,0x0007f56642f9ec90,0x0007f5783c32f31e,
-           0x0007f589bb17f609,0x0007f59ac2ff1525,0x0007f5ab5718b15a,0x0007f5bb7a71427d,
-           0x0007f5cb2ff3100a,0x0007f5da7a67cebe,0x0007f5e95c7a24e8,0x0007f5f7d8b7171f,
-           0x0007f605f18f5ef4,0x0007f613a958ad0a,0x0007f621024ed7ea,0x0007f62dfe94f8cb,
+           0x0007d87db38c5c87,0x0007da4fc6a9ba62,0x0007dbf611b37f3b,0x0007dd7674d0f286,
+           0x0007ded5ce8205f6,0x0007e018307fb62b,0x0007e141081bd124,0x0007e2533d712de8,
+           0x0007e3514bbd7718,0x0007e43d54944b52,0x0007e5192f25ef42,0x0007e5e67481118d,
+           0x0007e6a6897c1ce2,0x0007e75aa6c7f64c,0x0007e803df8ee498,0x0007e8a326eb6272,
+           0x0007e93954717a28,0x0007e9c727f8648f,0x0007ea4d4cc85a3c,0x0007eacc5c4907a9,
+           0x0007eb44e0474cf6,0x0007ebb754e47419,0x0007ec242a3d8474,0x0007ec8bc5d69645,
+           0x0007ecee83d3d6e9,0x0007ed4cb8082f45,0x0007eda6aee0170f,0x0007edfcae2dfe68,
+           0x0007ee4ef5dccd3e,0x0007ee9dc08c394e,0x0007eee9441a17c7,0x0007ef31b21b4fb1,
+           0x0007ef773846a8a7,0x0007efba00d35a17,0x0007effa32ccf69f,0x0007f037f25e1278,
+           0x0007f0736112d12c,0x0007f0ac9e145c25,0x0007f0e3c65e1fcc,0x0007f118f4ed8e54,
+           0x0007f14c42ed0dc8,0x0007f17dc7daa0c3,0x0007f1ad99aac6a5,0x0007f1dbcce80015,
+           0x0007f20874cf56bf,0x0007f233a36a3b9a,0x0007f25d69a604ad,0x0007f285d7694a92,
+           0x0007f2acfba75e3b,0x0007f2d2e4720909,0x0007f2f79f09c344,0x0007f31b37ec883b,
+           0x0007f33dbae36abc,0x0007f35f330f08d5,0x0007f37faaf2fa79,0x0007f39f2c805380,
+           0x0007f3bdc11f4f1c,0x0007f3db71b83850,0x0007f3f846bba121,0x0007f4144829f846,
+           0x0007f42f7d9a8b9d,0x0007f449ee420432,0x0007f463a0f8675e,0x0007f47c9c3ea77b,
+           0x0007f494e643cd8e,0x0007f4ac84e9c475,0x0007f4c37dc9cd50,0x0007f4d9d638a432,
+           0x0007f4ef934a5b6a,0x0007f504b9d5f33d,0x0007f5194e78b352,0x0007f52d55994a96,
+           0x0007f540d36aba0c,0x0007f553cbef0e77,0x0007f56642f9ec8f,0x0007f5783c32f31e,
+           0x0007f589bb17f609,0x0007f59ac2ff1525,0x0007f5ab5718b15a,0x0007f5bb7a71427c,
+           0x0007f5cb2ff31009,0x0007f5da7a67cebe,0x0007f5e95c7a24e7,0x0007f5f7d8b7171e,
+           0x0007f605f18f5ef4,0x0007f613a958ad0a,0x0007f621024ed7e9,0x0007f62dfe94f8cb,
            0x0007f63aa036777a,0x0007f646e928065a,0x0007f652db488f88,0x0007f65e786213ff,
-           0x0007f669c22a7d8b,0x0007f674ba44645a,0x0007f67f623fc8dc,0x0007f689bb9ac294,
-           0x0007f693c7c22482,0x0007f69d881217a6,0x0007f6a6fdd6ac37,0x0007f6b02a4c61ee,
-           0x0007f6b90ea0a7f4,0x0007f6c1abf254c1,0x0007f6ca03521664,0x0007f6d215c2db82,
-           0x0007f6d9e43a355a,0x0007f6e16fa0b329,0x0007f6e8b8d23729,0x0007f6efc09e4569,
-           0x0007f6f687c84cc0,0x0007f6fd0f07ea0a,0x0007f703570925e2,0x0007f709606cad03,
-           0x0007f70f2bc80370,0x0007f714b9a5b292,0x0007f71a0a85725e,0x0007f71f1edc4d9e,
-           0x0007f723f714c17a,0x0007f728938ed843,0x0007f72cf4a03fa1,0x0007f7311a945a17,
-           0x0007f73505ac4bf8,0x0007f738b61f03bd,0x0007f73c2c193dc1,0x0007f73f67bd835d,
-           0x0007f74269242559,0x0007f745305b31a1,0x0007f747bd666429,0x0007f74a103f12ed,
-           0x0007f74c28d414f6,0x0007f74e0709a42e,0x0007f74faab939fa,0x0007f75113b16657,
-           0x0007f75241b5a156,0x0007f753347e16b9,0x0007f753ebb76b7d,0x0007f75467027d05,
-           0x0007f754a5f4199e,0x0007f754a814b208,0x0007f7546ce003af,0x0007f753f3c4bb2a,
-           0x0007f7533c240e92,0x0007f75245514f41,0x0007f7510e91726d,0x0007f74f971a9012,
-           0x0007f74dde135797,0x0007f74be2927972,0x0007f749a39e051d,0x0007f747202aba8b,
-           0x0007f744571b4e3d,0x0007f741473f9efe,0x0007f73def53dc44,0x0007f73a4dff9c00,
-           0x0007f73661d4deaf,0x0007f732294f0040,0x0007f72da2d19444,0x0007f728cca72bdb,
-           0x0007f723a5000367,0x0007f71e29f09628,0x0007f7185970156c,0x0007f7123156c102,
-           0x0007f70baf5c1e2d,0x0007f704d1150a24,0x0007f6fd93f1a4e6,0x0007f6f5f53b10b6,
-           0x0007f6edf211023f,0x0007f6e587671cea,0x0007f6dcb202167a,0x0007f6d36e749c65,
-           0x0007f6c9b91bf4c6,0x0007f6bf8e1c541b,0x0007f6b4e95ce015,0x0007f6a9c6835700,
-           0x0007f69e20ef5211,0x0007f691f3b517ec,0x0007f6853997f322,0x0007f677ed03ff1a,
+           0x0007f669c22a7d8a,0x0007f674ba446459,0x0007f67f623fc8db,0x0007f689bb9ac294,
+           0x0007f693c7c22481,0x0007f69d881217a6,0x0007f6a6fdd6ac36,0x0007f6b02a4c61ee,
+           0x0007f6b90ea0a7f4,0x0007f6c1abf254c0,0x0007f6ca03521664,0x0007f6d215c2db82,
+           0x0007f6d9e43a3559,0x0007f6e16fa0b329,0x0007f6e8b8d23729,0x0007f6efc09e4569,
+           0x0007f6f687c84cbf,0x0007f6fd0f07ea09,0x0007f703570925e2,0x0007f709606cad03,
+           0x0007f70f2bc8036f,0x0007f714b9a5b292,0x0007f71a0a85725d,0x0007f71f1edc4d9e,
+           0x0007f723f714c179,0x0007f728938ed843,0x0007f72cf4a03fa0,0x0007f7311a945a16,
+           0x0007f73505ac4bf8,0x0007f738b61f03bd,0x0007f73c2c193dc0,0x0007f73f67bd835c,
+           0x0007f74269242559,0x0007f745305b31a1,0x0007f747bd666428,0x0007f74a103f12ed,
+           0x0007f74c28d414f5,0x0007f74e0709a42d,0x0007f74faab939f9,0x0007f75113b16657,
+           0x0007f75241b5a155,0x0007f753347e16b8,0x0007f753ebb76b7c,0x0007f75467027d05,
+           0x0007f754a5f4199d,0x0007f754a814b207,0x0007f7546ce003ae,0x0007f753f3c4bb29,
+           0x0007f7533c240e92,0x0007f75245514f41,0x0007f7510e91726c,0x0007f74f971a9012,
+           0x0007f74dde135797,0x0007f74be2927971,0x0007f749a39e051c,0x0007f747202aba8a,
+           0x0007f744571b4e3c,0x0007f741473f9efe,0x0007f73def53dc43,0x0007f73a4dff9bff,
+           0x0007f73661d4deaf,0x0007f732294f003f,0x0007f72da2d19444,0x0007f728cca72bda,
+           0x0007f723a5000367,0x0007f71e29f09627,0x0007f7185970156b,0x0007f7123156c102,
+           0x0007f70baf5c1e2c,0x0007f704d1150a23,0x0007f6fd93f1a4e5,0x0007f6f5f53b10b6,
+           0x0007f6edf211023e,0x0007f6e587671ce9,0x0007f6dcb2021679,0x0007f6d36e749c64,
+           0x0007f6c9b91bf4c6,0x0007f6bf8e1c541b,0x0007f6b4e95ce015,0x0007f6a9c68356ff,
+           0x0007f69e20ef5211,0x0007f691f3b517eb,0x0007f6853997f321,0x0007f677ed03ff19,
            0x0007f66a08075bdc,0x0007f65b844ab75a,0x0007f64c5b091860,0x0007f63c8506d4bc,
-           0x0007f62bfa8798ff,0x0007f61ab34364b1,0x0007f608a65a599a,0x0007f5f5ca4737e8,
-           0x0007f5e214d05b49,0x0007f5cd7af7066e,0x0007f5b7f0e4c2a1,0x0007f5a169d68fcf,
-           0x0007f589d80596a6,0x0007f5712c8d0174,0x0007f557574c912b,0x0007f53c46c77193,
-           0x0007f51fe7feb9f3,0x0007f5022646ecfb,0x0007f4e2eb17ab1d,0x0007f4c21dd4a3d2,
+           0x0007f62bfa8798fe,0x0007f61ab34364b0,0x0007f608a65a599a,0x0007f5f5ca4737e8,
+           0x0007f5e214d05b48,0x0007f5cd7af7066e,0x0007f5b7f0e4c2a1,0x0007f5a169d68fcf,
+           0x0007f589d80596a5,0x0007f5712c8d0174,0x0007f557574c912b,0x0007f53c46c77193,
+           0x0007f51fe7feb9f2,0x0007f5022646ecfb,0x0007f4e2eb17ab1d,0x0007f4c21dd4a3d1,
            0x0007f49fa38ea394,0x0007f47b5ebb62eb,0x0007f4552ee27473,0x0007f42cf03d58f5,
-           0x0007f4027b4854a0,0x0007f3d5a44119e0,0x0007f3a63a8fb553,0x0007f37408155101,
-           0x0007f33ed05b55ec,0x0007f3064f9c183f,0x0007f2ca399c7ba1,0x0007f28a384bb940,
-           0x0007f245ea1b7a2c,0x0007f1fcdffe8f1c,0x0007f1ae9af758ce,0x0007f15a8917f27f,
-           0x0007f10001ccaaac,0x0007f09e413c418a,0x0007f034627733d8,0x0007efc15815b8d5,
-           0x0007ef43e2bf7f55,0x0007eeba84e31dff,0x0007ee237294df89,0x0007ed7c7c170141,
-           0x0007ecc2f0d95d3b,0x0007ebf377a46782,0x0007eb09d6deb286,0x0007ea00a4f17809,
-           0x0007e8d0d3da63d6,0x0007e771023b0fcf,0x0007e5d46c2f08d9,0x0007e3e937669691,
-           0x0007e195978f1176,0x0007deb2c0e05c1d,0x0007db0362002a1a,0x0007d6202c15143a,
-           0x0007cf4b8f00a2cc,0x0007c4fd24520efe,0x0007b362fbf81816,0x00078d2d25998e25]
-const wi = 
+           0x0007f4027b48549f,0x0007f3d5a44119df,0x0007f3a63a8fb552,0x0007f37408155100,
+           0x0007f33ed05b55ec,0x0007f3064f9c183e,0x0007f2ca399c7ba1,0x0007f28a384bb940,
+           0x0007f245ea1b7a2b,0x0007f1fcdffe8f1b,0x0007f1ae9af758cd,0x0007f15a8917f27e,
+           0x0007f10001ccaaab,0x0007f09e413c418a,0x0007f034627733d7,0x0007efc15815b8d5,
+           0x0007ef43e2bf7f55,0x0007eeba84e31dfe,0x0007ee237294df89,0x0007ed7c7c170141,
+           0x0007ecc2f0d95d3a,0x0007ebf377a46782,0x0007eb09d6deb285,0x0007ea00a4f17808,
+           0x0007e8d0d3da63d6,0x0007e771023b0fcf,0x0007e5d46c2f08d8,0x0007e3e937669691,
+           0x0007e195978f1176,0x0007deb2c0e05c1c,0x0007db0362002a19,0x0007d6202c151439,
+           0x0007cf4b8f00a2cb,0x0007c4fd24520efd,0x0007b362fbf81816,0x00078d2d25998e24]
+const wi =
     [1.7367254121602630e-15,9.5586603514556339e-17,1.2708704834810623e-16,
      1.4909740962495474e-16,1.6658733631586268e-16,1.8136120810119029e-16,
      1.9429720153135588e-16,2.0589500628482093e-16,2.1646860576895422e-16,
@@ -399,7 +399,7 @@ const wi =
      1.3446300925011171e-15,1.3693606835128518e-15,1.3979436672775240e-15,
      1.4319989869661328e-15,1.4744848603597596e-15,1.5317872741611144e-15,
      1.6227698675312968e-15]
-const fi = 
+const fi =
     [1.0000000000000000e+00,9.7710170126767082e-01,9.5987909180010600e-01,
      9.4519895344229909e-01,9.3206007595922991e-01,9.1999150503934646e-01,
      9.0872644005213032e-01,8.9809592189834297e-01,8.8798466075583282e-01,
@@ -488,73 +488,73 @@ const fi =
      1.2602859304985975e-03]
 
 ## Tables for exponential variates
-const ke = 
-Uint64[0x000e290a13924be4,0                 ,0x0009beadebce18c0,0x000c377ac71f9e08,
-       0x000d4ddb99075857,0x000de893fb8ca23e,0x000e4a8e87c4328e,0x000e8dff16ae1cba,
-       0x000ebf2deab58c5a,0x000ee49a6e8b9639,0x000f0204efd64ee5,0x000f19bdb8ea3c1c,
-       0x000f2d458bbe5bd2,0x000f3da104b78236,0x000f4b86d784571f,0x000f577ad8a7784f,
-       0x000f61de83da32ac,0x000f6afb7843cce7,0x000f730a57372b44,0x000f7a37651b0e68,
-       0x000f80a5bb6eea52,0x000f867189d3cb5c,0x000f8bb1b4f8fbbd,0x000f9079062292b9,
-       0x000f94d70ca8d43a,0x000f98d8c7dcaa99,0x000f9c8928abe083,0x000f9ff175b734a6,
-       0x000fa319996bc47e,0x000fa6085f8e9d08,0x000fa8c3a62e1991,0x000fab5084e1f660,
-       0x000fadb36c84cccb,0x000faff041086846,0x000fb20a6ea22bb9,0x000fb404fb42cb3d,
-       0x000fb5e295158174,0x000fb7a59e99727a,0x000fb95038c8789d,0x000fbae44ba684ec,
-       0x000fbc638d822e60,0x000fbdcf89209ffb,0x000fbf29a303cfc5,0x000fc0731df1089d,
-       0x000fc1ad1ed6c8b1,0x000fc2d8b02b5c8a,0x000fc3f6c4d92131,0x000fc5083ac9ba7d,
-       0x000fc60ddd1e9cd7,0x000fc7086622e825,0x000fc7f881009f0c,0x000fc8decb41ac71,
-       0x000fc9bbd623d7ec,0x000fca9027c5b26e,0x000fcb5c3c319c49,0x000fcc20864b4449,
-       0x000fccdd70a35d41,0x000fcd935e34bf80,0x000fce42ab0db8bd,0x000fceebace7ec02,
-       0x000fcf8eb3b0d0e7,0x000fd02c0a049b60,0x000fd0c3f59d199d,0x000fd156b7b5e27e,
-       0x000fd1e48d670342,0x000fd26daff73552,0x000fd2f2552684bf,0x000fd372af7233c2,
-       0x000fd3eeee528f62,0x000fd4673e73543b,0x000fd4dbc9e72ff8,0x000fd54cb856dc2c,
-       0x000fd5ba2f2c4119,0x000fd62451ba02c3,0x000fd68b415fcff5,0x000fd6ef1dabc161,
-       0x000fd75004790eb7,0x000fd7ae120c583f,0x000fd809612dbd0a,0x000fd8620b40effa,
-       0x000fd8b8285b78fe,0x000fd90bcf594b1d,0x000fd95d15efd426,0x000fd9ac10bfa70c,
-       0x000fd9f8d364df06,0x000fda437086566c,0x000fda8bf9e3c9ff,0x000fdad28062fed5,
-       0x000fdb17141bff2d,0x000fdb59c4648085,0x000fdb9a9fda83cd,0x000fdbd9b46e3ed4,
-       0x000fdc170f6b5d05,0x000fdc52bd81a3fb,0x000fdc8ccacd07ba,0x000fdcc542dd3902,
-       0x000fdcfc30bcb794,0x000fdd319ef77143,0x000fdd6597a0f60c,0x000fdd98245a48a3,
-       0x000fddc94e575271,0x000fddf91e64014f,0x000fde279ce914cb,0x000fde54d1f0a06b,
-       0x000fde80c52a47d0,0x000fdeab7def394e,0x000fded50345eb36,0x000fdefd5be59fa1,
-       0x000fdf248e39b26f,0x000fdf4aa064b4b0,0x000fdf6f98435894,0x000fdf937b6f30bb,
-       0x000fdfb64f414572,0x000fdfd818d48262,0x000fdff8dd07fed8,0x000fe018a08122c5,
-       0x000fe03767adaa5a,0x000fe05536c58a14,0x000fe07211ccb4c5,0x000fe08dfc94c532,
-       0x000fe0a8fabe8ca2,0x000fe0c30fbb87a6,0x000fe0dc3ecf3a5a,0x000fe0f48b107522,
-       0x000fe10bf76a82ef,0x000fe122869e4200,0x000fe1383b4327e1,0x000fe14d17c83188,
-       0x000fe1611e74c023,0x000fe1745169635a,0x000fe186b2a09177,0x000fe19843ef4e08,
-       0x000fe1a90705bf64,0x000fe1b8fd6fb37c,0x000fe1c828951444,0x000fe1d689ba4bfd,
-       0x000fe1e4220099a5,0x000fe1f0f26655a0,0x000fe1fcfbc726d4,0x000fe2083edc2831,
-       0x000fe212bc3bfeb4,0x000fe21c745adfe4,0x000fe225678a8895,0x000fe22d95fa23f4,
-       0x000fe234ffb62282,0x000fe23ba4a800d9,0x000fe2418495fddd,0x000fe2469f22bffb,
-       0x000fe24af3cce90e,0x000fe24e81ee9859,0x000fe25148bcda1a,0x000fe253474703fe,
-       0x000fe2547c75fdc6,0x000fe254e70b7550,0x000fe25485a0fd1a,0x000fe25356a71450,
-       0x000fe2515864173b,0x000fe24e88f316f2,0x000fe24ae64296fb,0x000fe2466e132f61,
-       0x000fe2411df611bd,0x000fe23af34b6f73,0x000fe233eb40bf42,0x000fe22c02cee01c,
-       0x000fe22336b81711,0x000fe2198385e5cd,0x000fe20ee586b707,0x000fe20358cb5dfc,
-       0x000fe1f6d92465b1,0x000fe1e9621f2c9e,0x000fe1daef02c8da,0x000fe1cb7accb0a6,
-       0x000fe1bb002d22ca,0x000fe1a9798349b9,0x000fe196e0d9140d,0x000fe1832fdebc44,
-       0x000fe16e5fe5f932,0x000fe15869dccfcf,0x000fe1414647fe78,0x000fe128ed3cf8b2,
-       0x000fe10f565b69cf,0x000fe0f478c633ab,0x000fe0d84b1bdd9e,0x000fe0bac36e6688,
-       0x000fe09bd73a6b5c,0x000fe07b7b5d920b,0x000fe059a40c26d2,0x000fe03644c5d7f9,
-       0x000fe011504979b3,0x000fdfeab887b95d,0x000fdfc26e94a448,0x000fdf986297e306,
-       0x000fdf6c83bb8663,0x000fdf3ec0193eee,0x000fdf0f04a5d30a,0x000fdedd3d1aa204,
-       0x000fdea953dcfc13,0x000fde7331e3100e,0x000fde3abe9626f3,0x000fddffdfb1dbd5,
-       0x000fddc2791ff351,0x000fdd826cd068c7,0x000fdd3f9a8d3857,0x000fdcf9dfc95b0d,
-       0x000fdcb1176a55fe,0x000fdc65198ba50c,0x000fdc15bb3b2daa,0x000fdbc2ce2dc4ae,
-       0x000fdb6c206aaaca,0x000fdb117becb4a2,0x000fdab2a6379bf1,0x000fda4f5fdfb4e9,
-       0x000fd9e76401f3a4,0x000fd97a67a9ce20,0x000fd9081922142a,0x000fd8901f2d4b02,
-       0x000fd812182170e1,0x000fd78d98e23cd4,0x000fd7022bb3f083,0x000fd66f4edf96ba,
-       0x000fd5d473200306,0x000fd530f9ccff94,0x000fd48432b7b351,0x000fd3cd59a8469f,
-       0x000fd30b9368f90a,0x000fd23dea45f500,0x000fd16349e2e04b,0x000fd07a7a3ef98b,
-       0x000fcf8219b5df06,0x000fce7895bcfcdf,0x000fcd5c220ad5e3,0x000fcc2aadbc17dd,
-       0x000fcae1d5e81fbd,0x000fc97ed4e778fa,0x000fc7fe6d4d720f,0x000fc65ccf39c2fc,
-       0x000fc4957623cb04,0x000fc2a2fc826dc8,0x000fc07ee19b01ce,0x000fbe213c1cf493,
-       0x000fbb8051ac1566,0x000fb890078d120e,0x000fb5411a5b9a96,0x000fb18000547134,
-       0x000fad334827f1e2,0x000fa839276708b9,0x000fa263b32e37ee,0x000f9b72d1c52cd1,
-       0x000f930a1a281a05,0x000f889f023d820a,0x000f7b577d2be5f4,0x000f69c650c40a8f,
-       0x000f51530f0916d9,0x000f2cb0e3c5933e,0x000eeefb15d605d9,0x000e6da6ecf27460]
+const ke =
+    Uint64[0x000e290a13924be3,0x0000000000000000,0x0009beadebce18bf,0x000c377ac71f9e08,
+           0x000d4ddb99075857,0x000de893fb8ca23e,0x000e4a8e87c4328d,0x000e8dff16ae1cb9,
+           0x000ebf2deab58c59,0x000ee49a6e8b9638,0x000f0204efd64ee4,0x000f19bdb8ea3c1b,
+           0x000f2d458bbe5bd1,0x000f3da104b78236,0x000f4b86d784571f,0x000f577ad8a7784f,
+           0x000f61de83da32ab,0x000f6afb7843cce7,0x000f730a57372b44,0x000f7a37651b0e68,
+           0x000f80a5bb6eea52,0x000f867189d3cb5b,0x000f8bb1b4f8fbbd,0x000f9079062292b8,
+           0x000f94d70ca8d43a,0x000f98d8c7dcaa99,0x000f9c8928abe083,0x000f9ff175b734a6,
+           0x000fa319996bc47d,0x000fa6085f8e9d07,0x000fa8c3a62e1991,0x000fab5084e1f660,
+           0x000fadb36c84cccb,0x000faff041086846,0x000fb20a6ea22bb9,0x000fb404fb42cb3c,
+           0x000fb5e295158173,0x000fb7a59e99727a,0x000fb95038c8789d,0x000fbae44ba684eb,
+           0x000fbc638d822e60,0x000fbdcf89209ffa,0x000fbf29a303cfc5,0x000fc0731df1089c,
+           0x000fc1ad1ed6c8b1,0x000fc2d8b02b5c89,0x000fc3f6c4d92131,0x000fc5083ac9ba7d,
+           0x000fc60ddd1e9cd6,0x000fc7086622e825,0x000fc7f881009f0b,0x000fc8decb41ac70,
+           0x000fc9bbd623d7ec,0x000fca9027c5b26d,0x000fcb5c3c319c49,0x000fcc20864b4449,
+           0x000fccdd70a35d40,0x000fcd935e34bf80,0x000fce42ab0db8bd,0x000fceebace7ec01,
+           0x000fcf8eb3b0d0e7,0x000fd02c0a049b60,0x000fd0c3f59d199c,0x000fd156b7b5e27e,
+           0x000fd1e48d670341,0x000fd26daff73551,0x000fd2f2552684be,0x000fd372af7233c1,
+           0x000fd3eeee528f62,0x000fd4673e73543a,0x000fd4dbc9e72ff7,0x000fd54cb856dc2c,
+           0x000fd5ba2f2c4119,0x000fd62451ba02c2,0x000fd68b415fcff4,0x000fd6ef1dabc160,
+           0x000fd75004790eb6,0x000fd7ae120c583f,0x000fd809612dbd09,0x000fd8620b40effa,
+           0x000fd8b8285b78fd,0x000fd90bcf594b1d,0x000fd95d15efd425,0x000fd9ac10bfa70c,
+           0x000fd9f8d364df06,0x000fda437086566b,0x000fda8bf9e3c9fe,0x000fdad28062fed5,
+           0x000fdb17141bff2c,0x000fdb59c4648085,0x000fdb9a9fda83cc,0x000fdbd9b46e3ed4,
+           0x000fdc170f6b5d04,0x000fdc52bd81a3fb,0x000fdc8ccacd07ba,0x000fdcc542dd3902,
+           0x000fdcfc30bcb793,0x000fdd319ef77143,0x000fdd6597a0f60b,0x000fdd98245a48a2,
+           0x000fddc94e575271,0x000fddf91e64014f,0x000fde279ce914ca,0x000fde54d1f0a06a,
+           0x000fde80c52a47cf,0x000fdeab7def394e,0x000fded50345eb35,0x000fdefd5be59fa0,
+           0x000fdf248e39b26f,0x000fdf4aa064b4af,0x000fdf6f98435894,0x000fdf937b6f30ba,
+           0x000fdfb64f414571,0x000fdfd818d48262,0x000fdff8dd07fed8,0x000fe018a08122c4,
+           0x000fe03767adaa59,0x000fe05536c58a13,0x000fe07211ccb4c5,0x000fe08dfc94c532,
+           0x000fe0a8fabe8ca1,0x000fe0c30fbb87a5,0x000fe0dc3ecf3a5a,0x000fe0f48b107521,
+           0x000fe10bf76a82ef,0x000fe122869e41ff,0x000fe1383b4327e1,0x000fe14d17c83187,
+           0x000fe1611e74c023,0x000fe1745169635a,0x000fe186b2a09176,0x000fe19843ef4e07,
+           0x000fe1a90705bf63,0x000fe1b8fd6fb37c,0x000fe1c828951443,0x000fe1d689ba4bfd,
+           0x000fe1e4220099a4,0x000fe1f0f26655a0,0x000fe1fcfbc726d4,0x000fe2083edc2830,
+           0x000fe212bc3bfeb4,0x000fe21c745adfe3,0x000fe225678a8895,0x000fe22d95fa23f4,
+           0x000fe234ffb62282,0x000fe23ba4a800d9,0x000fe2418495fddc,0x000fe2469f22bffb,
+           0x000fe24af3cce90d,0x000fe24e81ee9858,0x000fe25148bcda19,0x000fe253474703fe,
+           0x000fe2547c75fdc6,0x000fe254e70b754f,0x000fe25485a0fd1a,0x000fe25356a71450,
+           0x000fe2515864173a,0x000fe24e88f316f1,0x000fe24ae64296fa,0x000fe2466e132f60,
+           0x000fe2411df611bd,0x000fe23af34b6f73,0x000fe233eb40bf41,0x000fe22c02cee01b,
+           0x000fe22336b81710,0x000fe2198385e5cc,0x000fe20ee586b707,0x000fe20358cb5dfb,
+           0x000fe1f6d92465b1,0x000fe1e9621f2c9e,0x000fe1daef02c8da,0x000fe1cb7accb0a6,
+           0x000fe1bb002d22c9,0x000fe1a9798349b8,0x000fe196e0d9140c,0x000fe1832fdebc44,
+           0x000fe16e5fe5f931,0x000fe15869dccfcf,0x000fe1414647fe78,0x000fe128ed3cf8b2,
+           0x000fe10f565b69cf,0x000fe0f478c633ab,0x000fe0d84b1bdd9e,0x000fe0bac36e6688,
+           0x000fe09bd73a6b5b,0x000fe07b7b5d920a,0x000fe059a40c26d2,0x000fe03644c5d7f8,
+           0x000fe011504979b2,0x000fdfeab887b95c,0x000fdfc26e94a447,0x000fdf986297e305,
+           0x000fdf6c83bb8663,0x000fdf3ec0193eed,0x000fdf0f04a5d30a,0x000fdedd3d1aa204,
+           0x000fdea953dcfc13,0x000fde7331e3100d,0x000fde3abe9626f2,0x000fddffdfb1dbd5,
+           0x000fddc2791ff351,0x000fdd826cd068c6,0x000fdd3f9a8d3856,0x000fdcf9dfc95b0c,
+           0x000fdcb1176a55fe,0x000fdc65198ba50b,0x000fdc15bb3b2daa,0x000fdbc2ce2dc4ae,
+           0x000fdb6c206aaaca,0x000fdb117becb4a1,0x000fdab2a6379bf0,0x000fda4f5fdfb4e9,
+           0x000fd9e76401f3a3,0x000fd97a67a9ce1f,0x000fd90819221429,0x000fd8901f2d4b02,
+           0x000fd812182170e1,0x000fd78d98e23cd3,0x000fd7022bb3f082,0x000fd66f4edf96b9,
+           0x000fd5d473200305,0x000fd530f9ccff94,0x000fd48432b7b351,0x000fd3cd59a8469e,
+           0x000fd30b9368f90a,0x000fd23dea45f500,0x000fd16349e2e04a,0x000fd07a7a3ef98a,
+           0x000fcf8219b5df05,0x000fce7895bcfcde,0x000fcd5c220ad5e2,0x000fcc2aadbc17dc,
+           0x000fcae1d5e81fbc,0x000fc97ed4e778f9,0x000fc7fe6d4d720e,0x000fc65ccf39c2fc,
+           0x000fc4957623cb03,0x000fc2a2fc826dc7,0x000fc07ee19b01cd,0x000fbe213c1cf493,
+           0x000fbb8051ac1566,0x000fb890078d120e,0x000fb5411a5b9a95,0x000fb18000547133,
+           0x000fad334827f1e2,0x000fa839276708b9,0x000fa263b32e37ed,0x000f9b72d1c52cd1,
+           0x000f930a1a281a05,0x000f889f023d820a,0x000f7b577d2be5f3,0x000f69c650c40a8f,
+           0x000f51530f0916d8,0x000f2cb0e3c5933e,0x000eeefb15d605d8,0x000e6da6ecf27460]
 
-const we = 
+const we =
     [1.9311480126418366e-15,1.4178028487910829e-17,2.3278824993382448e-17,
      3.0487830247064320e-17,3.6665697714474878e-17,4.2179302189289733e-17,
      4.7222561556862764e-17,5.1911915446217879e-17,5.6323471083955047e-17,
@@ -641,7 +641,7 @@ const we =
      1.2174462832361815e-15,1.2581958069755114e-15,1.3060984107128082e-15,
      1.3642786158057857e-15,1.4384889932178723e-15,1.5412190700064194e-15,
      1.7091034077168055e-15]
-const fe = 
+const fe =
     [1.0000000000000000e+00,9.3814368086217470e-01,9.0046992992574648e-01,
      8.7170433238120359e-01,8.4778550062398961e-01,8.2699329664305032e-01,
      8.0842165152300838e-01,7.9152763697249562e-01,7.7595685204011555e-01,
@@ -736,32 +736,33 @@ ziggurat_exp_r      = 7.6971174701310497140446280481
 rand(state::DSFMT_state) = dsfmt_genrand_close_open(state)
 randi() = reinterpret(Uint64,dsfmt_gv_genrand_close1_open2()) & 0x000fffffffffffff
 randi(state::DSFMT_state) = reinterpret(Uint64,dsfmt_genrand_close1_open2(state)) & 0x000fffffffffffff
-for (lhs, rhs) in (([], []), 
+for (lhs, rhs) in (([], []),
                   ([:(state::DSFMT_state)], [:state]))
-    @eval begin                
+    @eval begin
         function randmtzig_randn($(lhs...))
             @inbounds begin
-                while true
-                    r = randi($(rhs...))
-                    rabs = int64(r>>1) # One bit for the sign
-                    idx = rabs & 0xFF
-                    x = (r&1 != 0x000000000 ? -rabs : rabs)*wi[idx+1]
-                    if rabs < ki[idx+1]
-                        return x # 99.3% of the time we return here 1st try
-                    elseif idx == 0
-                        while true
-                            xx = -$(ziggurat_nor_inv_r)*log(rand($(rhs...)))
-                            yy = -log(rand($(rhs...)))
-                            if yy+yy > xx*xx
-                                return (rabs & 0x100) != 0x000000000 ? -$(ziggurat_nor_r)-xx : $(ziggurat_nor_r)+xx
-                            end
-                        end
-                    elseif (fi[idx] - fi[idx+1])*rand($(rhs...)) + fi[idx+1] < exp(-0.5*x*x)
-                        return x # return from the triangular area
-                    end
-                end
+                r = randi($(rhs...))
+                rabs = int64(r>>1) # One bit for the sign
+                idx = rabs & 0xFF
+                x = ifelse(r&1 != 0x000000000, -rabs, rabs)*wi[idx+1]
+                rabs < ki[idx+1] && return x # 99.3% of the time we return here 1st try
+                return randn_unlikely($(rhs...),idx,rabs,x)
             end
-        end    
+        end
+
+        function randn_unlikely($(lhs...),idx,rabs,x)
+            @inbounds if idx == 0
+                while true
+                    xx = -$(ziggurat_nor_inv_r)*log(rand($(rhs...)))
+                    yy = -log(rand($(rhs...)))
+                    yy+yy > xx*xx && return (rabs & 0x100) != 0x000000000 ? -$(ziggurat_nor_r)-xx : $(ziggurat_nor_r)+xx
+                end
+            elseif (fi[idx] - fi[idx+1])*rand($(rhs...)) + fi[idx+1] < exp(-0.5*x*x)
+                return x # return from the triangular area
+            else
+                return randmtzig_randn($(lhs...))
+            end
+        end
 
         function randmtzig_exprnd($(lhs...))
             @inbounds begin
@@ -778,7 +779,7 @@ for (lhs, rhs) in (([], []),
                     end
                 end
             end
-        end       
+        end
     end
 end
 
