@@ -20,7 +20,7 @@ let i = 2
              Module, UndefRefTag, Task, ASCIIString, UTF8String,
              UTF16String, UTF32String, Float16,
              :reserved9, :reserved10, :reserved11, :reserved12,
-             
+
              (), Bool, Any, :Any, None, Top, Undef, Type,
              :Array, :TypeVar, :Box,
              :lambda, :body, :return, :call, symbol("::"),
@@ -166,8 +166,9 @@ function serialize(s, e::Expr)
 end
 
 function serialize_mod_names(s, m::Module)
-    if m !== Main
-        serialize_mod_names(s, module_parent(m))
+    p = module_parent(m)
+    if m !== p
+        serialize_mod_names(s, p)
         serialize(s, module_name(m))
     end
 end
