@@ -11,18 +11,18 @@ function show(io::IO, x::ANY)
         recorded = false
         oid = object_id(x)
         shown_set = get(task_local_storage(), :SHOWNSET, nothing)
-        if shown_set == nothing 
+        if shown_set == nothing
             shown_set = Set()
-            task_local_storage(:SHOWNSET, shown_set) 
+            task_local_storage(:SHOWNSET, shown_set)
         end
-        
+
         try
             if oid in shown_set
                 print(io, "#= circular reference =#")
             else
                 push!(shown_set, oid)
                 recorded = true
-            
+
                 n = length(t.names)
                 for i=1:n
                     f = t.names[i]
@@ -38,7 +38,7 @@ function show(io::IO, x::ANY)
             end
         catch e
             rethrow(e)
-        
+
         finally
             if recorded delete!(shown_set, oid) end
         end
@@ -210,7 +210,7 @@ print(io::IO, n::Unsigned) = print(io, dec(n))
 #   print(io, ex) defers to show_unquoted(io, ex)
 #   show(io, ex) defers to show_unquoted(io, QuoteNode(ex))
 #   show_unquoted(io, ex) does the heavy lifting
-# 
+#
 # AST printing should follow two rules:
 #   1. parse(string(ex)) == ex
 #   2. eval(parse(repr(ex))) == ex
@@ -263,6 +263,7 @@ const bin_ops = Set{Symbol}(keys(bin_op_precs))
 const expr_infix_wide = Set([:(=), :(+=), :(-=), :(*=), :(/=), :(\=), :(&=),
     :(|=), :($=), :(>>>=), :(>>=), :(<<=), :(&&), :(||)])
 const expr_infix = Set([:(:), :(<:), :(->), :(=>), symbol("::")])
+const expr_infix_any = union(expr_infix, expr_infix_wide)
 const expr_calls  = [:call =>('(',')'), :calldecl =>('(',')'), :ref =>('[',']'), :curly =>('{','}')]
 const expr_parens = [:tuple=>('(',')'), :vcat=>('[',']'), :cell1d=>('{','}'),
                      :hcat =>('[',']'), :row =>('[',']')]
@@ -431,15 +432,13 @@ function show_unquoted(io::IO, ex::Expr, indent::Int, prec::Int)
         end
 
     # infix (i.e. "x<:y" or "x = y")
-    elseif (head in expr_infix && nargs==2) || (is(head,:(:)) && nargs==3)
-        show_list(io, args, head, indent)
-
-    elseif head in expr_infix_wide && nargs == 2
+    elseif (head in expr_infix_any && nargs==2) || (is(head,:(:)) && nargs==3)
         func_prec = get(bin_op_precs, head, 0)
+        head_ = head in expr_infix_wide ? " $head " : head
         if func_prec < prec
-            show_enclosed_list(io, '(', args, " $head ", ')', indent, func_prec)
+            show_enclosed_list(io, '(', args, head_, ')', indent, func_prec)
         else
-            show_list(io, args, " $head ", indent, func_prec)
+            show_list(io, args, head_, indent, func_prec)
         end
 
     # list (i.e. "(1,2,3)" or "[1,2,3]")
@@ -461,7 +460,7 @@ function show_unquoted(io::IO, ex::Expr, indent::Int, prec::Int)
     # (:calldecl is a "fake" expr node created when we find a :function expr)
     elseif head == :calldecl && nargs >= 1
         show_call(io, head, args[1], args[2:end], indent)
-        
+
     # function call
     elseif haskey(expr_calls, head) && nargs >= 1  # :call/:ref/:curly
         func = args[1]
@@ -619,7 +618,7 @@ function show_unquoted(io::IO, ex::Expr, indent::Int, prec::Int)
         a = map(args) do x
             if !isa(x,String)
                 if isa(x,Symbol) && !(x in quoted_syms)
-                    string("\$", x)
+                    string("\$(", x, ")")
                 else
                     string("\$(", sprint(show_unquoted,x), ")")
                 end
@@ -747,7 +746,7 @@ function dumptype(io::IO, x, n::Int, indent)
     # based on Jameson Nash's examples/typetree.jl
     println(io, x)
     if n == 0   # too deeply nested
-        return  
+        return
     end
     typargs(t) = split(string(t), "{")[1]
     # todo: include current module?
@@ -772,7 +771,7 @@ function dumptype(io::IO, x, n::Int, indent)
                     # type aliases
                     if string(s) != string(t.name)
                         println(io, indent, "  ", s, " = ", t.name)
-                    elseif t != Any 
+                    elseif t != Any
                         print(io, indent, "  ")
                         dump(io, t, n - 1, string(indent, "  "))
                     end
