@@ -9,10 +9,18 @@ type SparseMatrixCSC{Tv,Ti<:Integer} <: AbstractSparseMatrix{Tv,Ti}
     colptr::Vector{Ti}      # Column i is in colptr[i]:(colptr[i+1]-1)
     rowval::Vector{Ti}      # Row values of nonzeros
     nzval::Vector{Tv}       # Nonzero values
-end
 
-SparseMatrixCSC{Tv,Ti}(m::Integer, n::Integer, colptr::Vector{Ti}, rowval::Vector{Ti}, nzval::Vector{Tv}) =
-    SparseMatrixCSC(int(m), int(n), colptr, rowval, nzval)
+    function SparseMatrixCSC(m::Integer, n::Integer, colptr::Vector{Ti}, rowval::Vector{Ti}, nzval::Vector{Tv})
+        m < 0 && throw(ArgumentError("number of rows (m) must be ≥ 0, got $m"))
+        n < 0 && throw(ArgumentError("number of columns (n) must be ≥ 0, got $n"))
+        new(int(m), int(n), colptr, rowval, nzval)
+    end
+end
+function SparseMatrixCSC(m::Integer, n::Integer, colptr::Vector, rowval::Vector, nzval::Vector)
+    Tv = eltype(nzval)
+    Ti = promote_type(eltype(colptr), eltype(rowval))
+    SparseMatrixCSC{Tv,Ti}(m, n, colptr, rowval, nzval)
+end
 
 size(S::SparseMatrixCSC) = (S.m, S.n)
 nnz(S::SparseMatrixCSC) = int(S.colptr[end]-1)
@@ -56,7 +64,7 @@ function reinterpret{T,Tv,Ti}(::Type{T}, a::SparseMatrixCSC{Tv,Ti})
     colptr = copy(a.colptr)
     rowval = copy(a.rowval)
     nzval  = reinterpret(T, a.nzval)
-    return SparseMatrixCSC{T,Ti}(mA, nA, colptr, rowval, nzval)
+    return SparseMatrixCSC(mA, nA, colptr, rowval, nzval)
 end
 
 function sparse_compute_reshaped_colptr_and_rowval{Ti}(colptrS::Vector{Ti}, rowvalS::Vector{Ti}, mS::Int, nS::Int, colptrA::Vector{Ti}, rowvalA::Vector{Ti}, mA::Int, nA::Int)
@@ -107,7 +115,7 @@ function reinterpret{T,Tv,Ti,N}(::Type{T}, a::SparseMatrixCSC{Tv,Ti}, dims::NTup
 
     sparse_compute_reshaped_colptr_and_rowval(colptr, rowval, mS, nS, a.colptr, a.rowval, mA, nA)
 
-    return SparseMatrixCSC{T,Ti}(mS, nS, colptr, rowval, nzval)
+    return SparseMatrixCSC(mS, nS, colptr, rowval, nzval)
 end
 
 function reshape{Tv,Ti}(a::SparseMatrixCSC{Tv,Ti}, dims::NTuple{2,Int})
@@ -123,7 +131,7 @@ function reshape{Tv,Ti}(a::SparseMatrixCSC{Tv,Ti}, dims::NTuple{2,Int})
 
     sparse_compute_reshaped_colptr_and_rowval(colptr, rowval, mS, nS, a.colptr, a.rowval, mA, nA)
 
-    return SparseMatrixCSC{Tv,Ti}(mS, nS, colptr, rowval, nzval)
+    return SparseMatrixCSC(mS, nS, colptr, rowval, nzval)
 end
 
 ## Constructors
@@ -170,7 +178,9 @@ convert{T}(::Type{AbstractMatrix{T}}, A::SparseMatrixCSC) = convert(SparseMatrix
 convert(::Type{Matrix}, S::SparseMatrixCSC) = full(S)
 
 function full{Tv}(S::SparseMatrixCSC{Tv})
-    A = zeros(Tv, S.m, S.n)
+    # Handle cases where zero(Tv) is not defined but the array is dense.
+    # (Should we really worry about this?)
+    A = length(S) == nnz(S) ? Array(Tv, S.m, S.n) : zeros(Tv, S.m, S.n)
     for col = 1 : S.n, k = S.colptr[col] : (S.colptr[col+1]-1)
         A[S.rowval[k], col] = S.nzval[k]
     end
@@ -190,7 +200,7 @@ function vec{Tv,Ti}(S::SparseMatrixCSC{Tv,Ti})
     rowval = similar(S.rowval)
     lS = length(S)
     sparse_compute_reshaped_colptr_and_rowval(colptr, rowval, lS, 1, S.colptr, S.rowval, S.m, S.n)
-    SparseMatrixCSC{Tv,Ti}(lS, 1, colptr, rowval, copy(S.nzval))
+    SparseMatrixCSC(lS, 1, colptr, rowval, copy(S.nzval))
 end
 
 sparsevec(A::AbstractMatrix) = reshape(sparse(A), (length(A),1))
@@ -210,6 +220,7 @@ function sparsevec(I::AbstractVector, V, m::Integer, combine::Function)
     p = sortperm(I)
     @inbounds I = I[p]
     (nI==0 || m >= I[end]) || throw(DimensionMismatch("indices cannot be larger than length of vector"))
+    (nI==0 || I[1] > 0) || throw(BoundsError())
     V = V[p]
     sparse_IJ_sorted!(I, ones(eltype(I), nI), V, m, 1, combine)
 end
@@ -355,6 +366,7 @@ nonzeros(S::SparseMatrixCSC) = S.nzval
 
 function sprand{T}(m::Integer, n::Integer, density::FloatingPoint,
                    rng::Function,::Type{T}=eltype(rng(1)))
+    ((m < 0) || (n < 0)) && throw(ArgumentError("invalid Array dimensions"))
     0 <= density <= 1 || throw(ArgumentError("$density not in [0,1]"))
     N = n*m
     N == 0 && return spzeros(T,m,n)
@@ -366,10 +378,10 @@ function sprand{T}(m::Integer, n::Integer, density::FloatingPoint,
 
     # density of nonzero columns:
     L = log1p(-density)
-    coldensity = -expm1(m*L) # = 1 - (1-density)^m 
+    coldensity = -expm1(m*L) # = 1 - (1-density)^m
     colsparsity = exp(m*L) # = 1 - coldensity
     L = 1/L
-    
+
     rows = Array(Int, 0)
     for j in randsubseq(1:n, coldensity)
         # To get the right statistics, we *must* have a nonempty column j
@@ -401,10 +413,11 @@ spones{T}(S::SparseMatrixCSC{T}) =
      SparseMatrixCSC(S.m, S.n, copy(S.colptr), copy(S.rowval), ones(T, S.colptr[end]-1))
 
 spzeros(m::Integer, n::Integer) = spzeros(Float64, m, n)
-spzeros(Tv::Type, m::Integer, n::Integer) =
-    SparseMatrixCSC(m, n, ones(Int, n+1), Array(Int, 0), Array(Tv, 0))
-spzeros(Tv::Type, Ti::Type, m::Integer, n::Integer) =
+spzeros(Tv::Type, m::Integer, n::Integer) = spzeros(Tv, Int, m, n)
+function spzeros(Tv::Type, Ti::Type, m::Integer, n::Integer)
+    ((m < 0) || (n < 0)) && throw(ArgumentError("invalid Array dimensions"))
     SparseMatrixCSC(m, n, ones(Ti, n+1), Array(Ti, 0), Array(Tv, 0))
+end
 
 speye(n::Integer) = speye(Float64, n)
 speye(T::Type, n::Integer) = speye(T, n, n)
@@ -413,6 +426,7 @@ speye{T}(S::SparseMatrixCSC{T}) = speye(T, size(S, 1), size(S, 2))
 eye(S::SparseMatrixCSC) = speye(S)
 
 function speye(T::Type, m::Integer, n::Integer)
+    ((m < 0) || (n < 0)) && throw(ArgumentError("invalid Array dimensions"))
     x = min(m,n)
     rowval = [1:x]
     colptr = [rowval, fill(int(x+1), n+1-x)]
@@ -489,6 +503,16 @@ for op in (:-, :abs, :abs2, :log1p, :expm1)
     end
 end
 
+function conj!(A::SparseMatrixCSC)
+    nzvalA = A.nzval
+    @simd for i=1:length(nzvalA)
+        @inbounds nzvalA[i] = conj(nzvalA[i])
+    end
+    return A
+end
+
+conj(A::SparseMatrixCSC) = conj!(copy(A))
+
 # Operations that map nonzeros to nonzeros, and zeros to nonzeros
 # Result is dense
 for op in (:cos, :cosh, :acos, :sec, :csc, :cot, :acot, :sech,
@@ -508,7 +532,7 @@ for op in (:cos, :cosh, :acos, :sec, :csc, :cot, :acot, :sech,
             end
             return B
         end
-        
+
     end
 end
 
@@ -704,6 +728,7 @@ end
 
 function maximum{T}(A::SparseMatrixCSC{T})
     isempty(A) && throw(ArgumentError("argument must not be empty"))
+    (nnz(A) == 0) && (return zero(T))
     m = maximum(A.nzval)
     nnz(A)!=length(A) ? max(m,zero(T)) : m
 end
@@ -713,6 +738,7 @@ maximum{T}(A::SparseMatrixCSC{T}, region) =
 
 function minimum{T}(A::SparseMatrixCSC{T})
     isempty(A) && throw(ArgumentError("argument must not be empty"))
+    (nnz(A) == 0) && (return zero(T))
     m = minimum(A.nzval)
     nnz(A)!=length(A) ? min(m,zero(T)) : m
 end
@@ -726,6 +752,8 @@ sum{T}(A::SparseMatrixCSC{T}, region)  = reducedim(+,A,region,zero(T))
 prod{T}(A::SparseMatrixCSC{T})         = nnz(A)!=length(A) ? zero(T) : prod(A.nzval)
 prod{T}(A::SparseMatrixCSC{T}, region) = reducedim(*,A,region,one(T))
 
+mean(A::SparseMatrixCSC, region::Integer) = sum(A, region) / size(A, region)
+
 #all(A::SparseMatrixCSC{Bool}, region) = reducedim(all,A,region,true)
 #any(A::SparseMatrixCSC{Bool}, region) = reducedim(any,A,region,false)
 #sum(A::SparseMatrixCSC{Bool}, region) = reducedim(+,A,region,0,Int)
@@ -737,7 +765,7 @@ function rangesearch(haystack::Range, needle)
     (rem==0 && 1<=i+1<=length(haystack)) ? i+1 : 0
 end
 
-getindex(A::SparseMatrixCSC, i::Integer) = getindex(A, ind2sub(size(A),i))
+getindex(A::SparseMatrixCSC, i::Integer) = isempty(A) ? throw(BoundsError()) : getindex(A, ind2sub(size(A),i))
 getindex(A::SparseMatrixCSC, I::(Integer,Integer)) = getindex(A, I[1], I[2])
 
 function getindex{T}(A::SparseMatrixCSC{T}, i0::Integer, i1::Integer)
@@ -803,7 +831,7 @@ function getindex{Tv,Ti<:Integer}(A::SparseMatrixCSC{Tv,Ti}, I::Range, J::Abstra
     # Form the structure of the result and compute space
     for j = 1:nJ
         @inbounds col = J[j]
-        for k in colptrA[col]:colptrA[col+1]-1
+        @simd for k in colptrA[col]:colptrA[col+1]-1
             if rowvalA[k] in I; nnzS += 1 end # `in` is fast for ranges
         end
         @inbounds colptrS[j+1] = nnzS+1
@@ -834,17 +862,95 @@ function getindex_I_sorted{Tv,Ti}(A::SparseMatrixCSC{Tv,Ti}, I::AbstractVector,
     # Sorted vectors for indexing rows.
     # Similar to getindex_general but without the transpose trick.
     (m, n) = size(A)
+
+    !isempty(I) && ((I[1] < 1) || (I[end] > m)) && BoundsError()
+    if !isempty(J)
+        minj, maxj = extrema(J)
+        ((minj < 1) || (maxj > n)) && BoundsError()
+    end
+
     nI = length(I)
-    nJ = length(J)
+    avgM = div(nnz(A),m)
+    # heuristics based on experiments
+    alg = ((nI - avgM) > 2^8) ? 1 :
+          ((avgM - nI) > 2^10) ? 0 : 2
+
+    (alg == 0) ? getindex_I_sorted_bsearch_A(A, I, J) :
+    (alg == 1) ? getindex_I_sorted_bsearch_I(A, I, J) :
+    getindex_I_sorted_linear(A, I, J)
+end
+
+function getindex_I_sorted_bsearch_A{Tv,Ti}(A::SparseMatrixCSC{Tv,Ti}, I::AbstractVector, J::AbstractVector)
+    const nI = length(I)
+    const nJ = length(J)
 
     colptrA = A.colptr; rowvalA = A.rowval; nzvalA = A.nzval
     colptrS = Array(Ti, nJ+1)
     colptrS[1] = 1
-    nnzS = 0
 
-    # Form the structure of the result and compute space
-    for j = 1:nJ
-        @inbounds col = J[j]
+    ptrS = 1
+    # determine result size
+    @inbounds for j = 1:nJ
+        col = J[j]
+        ptrI::Int = 1 # runs through I
+        ptrA::Int = colptrA[col]
+        stopA::Int = colptrA[col+1]-1
+        if ptrA <= stopA
+            while ptrI <= nI
+                rowI = I[ptrI]
+                ptrI += 1
+                (rowvalA[ptrA] > rowI) && continue
+                ptrA = searchsortedfirst(rowvalA, rowI, ptrA, stopA, Base.Order.Forward)
+                (ptrA <= stopA) || break
+                if rowvalA[ptrA] == rowI
+                    ptrS += 1
+                end
+            end
+        end
+        colptrS[j+1] = ptrS
+    end
+
+    rowvalS = Array(Ti, ptrS-1)
+    nzvalS  = Array(Tv, ptrS-1)
+
+    # fill the values
+    ptrS = 1
+    @inbounds for j = 1:nJ
+        col = J[j]
+        ptrI::Int = 1 # runs through I
+        ptrA::Int = colptrA[col]
+        stopA::Int = colptrA[col+1]-1
+        if ptrA <= stopA
+            while ptrI <= nI
+                rowI = I[ptrI]
+                if rowvalA[ptrA] <= rowI
+                    ptrA = searchsortedfirst(rowvalA, rowI, ptrA, stopA, Base.Order.Forward)
+                    (ptrA <= stopA) || break
+                    if rowvalA[ptrA] == rowI
+                        rowvalS[ptrS] = ptrI
+                        nzvalS[ptrS] = nzvalA[ptrA]
+                        ptrS += 1
+                    end
+                end
+                ptrI += 1
+            end
+        end
+    end
+    return SparseMatrixCSC(nI, nJ, colptrS, rowvalS, nzvalS)
+end
+
+function getindex_I_sorted_linear{Tv,Ti}(A::SparseMatrixCSC{Tv,Ti}, I::AbstractVector, J::AbstractVector)
+    const nI = length(I)
+    const nJ = length(J)
+
+    colptrA = A.colptr; rowvalA = A.rowval; nzvalA = A.nzval
+    colptrS = Array(Ti, nJ+1)
+    colptrS[1] = 1
+
+    ptrS   = 1
+    # build the cache and determine result size
+    @inbounds for j = 1:nJ
+        col = J[j]
         ptrI::Int = 1 # runs through I
         ptrA::Int = colptrA[col]
         stopA::Int = colptrA[col+1]
@@ -857,24 +963,23 @@ function getindex_I_sorted{Tv,Ti}(A::SparseMatrixCSC{Tv,Ti}, I::AbstractVector,
             elseif rowI < rowA
                 ptrI += 1
             else
-                nnzS += 1
+                ptrS += 1
                 ptrI += 1
             end
         end
-        @inbounds colptrS[j+1] = nnzS+1
+        colptrS[j+1] = ptrS
     end
 
-    # Populate the values in the result
-    rowvalS = Array(Ti, nnzS)
-    nzvalS  = Array(Tv, nnzS)
-    ptrS    = 1
+    rowvalS = Array(Ti, ptrS-1)
+    nzvalS  = Array(Tv, ptrS-1)
 
+    # fill the values
+    ptrS = 1
     @inbounds for j = 1:nJ
         col = J[j]
         ptrI::Int = 1 # runs through I
         ptrA::Int = colptrA[col]
         stopA::Int = colptrA[col+1]
-
         while ptrI <= nI && ptrA < stopA
             rowA = rowvalA[ptrA]
             rowI = I[ptrI]
@@ -894,6 +999,75 @@ function getindex_I_sorted{Tv,Ti}(A::SparseMatrixCSC{Tv,Ti}, I::AbstractVector,
     return SparseMatrixCSC(nI, nJ, colptrS, rowvalS, nzvalS)
 end
 
+function getindex_I_sorted_bsearch_I{Tv,Ti}(A::SparseMatrixCSC{Tv,Ti}, I::AbstractVector, J::AbstractVector)
+    const nI = length(I)
+    const nJ = length(J)
+
+    colptrA = A.colptr; rowvalA = A.rowval; nzvalA = A.nzval
+    colptrS = Array(Ti, nJ+1)
+    colptrS[1] = 1
+
+    m = A.m
+
+    # cacheI is used first to store num occurrences of each row in columns of interest
+    # and later to store position of first occurrence of each row in I
+    cacheI = zeros(Int, m)
+
+    # count rows
+    @inbounds for j = 1:nJ
+        col = J[j]
+        for ptrA in colptrA[col]:(colptrA[col+1]-1)
+            cacheI[rowvalA[ptrA]] += 1
+        end
+    end
+
+    # fill cache and count nnz
+    ptrS::Int = 0
+    ptrI::Int = 1
+    @inbounds for j = 1:m
+        cval = cacheI[j]
+        (cval == 0) && continue
+        ptrI = searchsortedfirst(I, j, ptrI, nI, Base.Order.Forward)
+        cacheI[j] = ptrI
+        while ptrI <= nI && I[ptrI] == j
+            ptrS += cval
+            ptrI += 1
+        end
+        if ptrI > nI
+            @simd for i=(j+1):m; @inbounds cacheI[i]=ptrI; end
+            break
+        end
+    end
+    rowvalS = Array(Ti, ptrS)
+    nzvalS  = Array(Tv, ptrS)
+    colptrS[nJ+1] = ptrS+1
+
+    # fill the values
+    ptrS = 1
+    @inbounds for j = 1:nJ
+        col = J[j]
+        ptrA::Int = colptrA[col]
+        stopA::Int = colptrA[col+1]
+        while ptrA < stopA
+            rowA = rowvalA[ptrA]
+            ptrI = cacheI[rowA]
+            (ptrI > nI) && break
+            if ptrI > 0
+                while I[ptrI] == rowA
+                    rowvalS[ptrS] = ptrI
+                    nzvalS[ptrS] = nzvalA[ptrA]
+                    ptrS += 1
+                    ptrI += 1
+                    (ptrI > nI) && break
+                end
+            end
+            ptrA += 1
+        end
+        colptrS[j+1] = ptrS
+    end
+    return SparseMatrixCSC(nI, nJ, colptrS, rowvalS, nzvalS)
+end
+
 
 function getindex_general{Tv,Ti}(A::SparseMatrixCSC{Tv,Ti}, I::AbstractVector, J::AbstractVector)
     # Anything for indexing rows.
@@ -913,9 +1087,7 @@ function getindex_general{Tv,Ti}(A::SparseMatrixCSC{Tv,Ti}, I::AbstractVector, J
     # Form the structure of the result and compute space
     for j = 1:nJ
         @inbounds col = J[j]
-
         ptrI::Int = 1
-
         ptrA::Int = colptrA[col]
         stopA::Int = colptrA[col+1]
 
@@ -945,9 +1117,7 @@ function getindex_general{Tv,Ti}(A::SparseMatrixCSC{Tv,Ti}, I::AbstractVector, J
 
     @inbounds for j = 1:nJ
         col = J[j]
-
         ptrI::Int = 1
-
         ptrA::Int = colptrA[col]
         stopA::Int = colptrA[col+1]
 
@@ -1038,7 +1208,11 @@ function getindex{Tv}(A::SparseMatrixCSC{Tv}, I::AbstractArray{Bool})
 end
 
 function getindex{Tv}(A::SparseMatrixCSC{Tv}, I::AbstractArray)
-    szA = size(A); colptrA = A.colptr; rowvalA = A.rowval; nzvalA = A.nzval
+    szA = size(A)
+    nA = szA[1]*szA[2]
+    colptrA = A.colptr
+    rowvalA = A.rowval
+    nzvalA = A.nzval
 
     n = length(I)
     outm = size(I,1)
@@ -1054,6 +1228,7 @@ function getindex{Tv}(A::SparseMatrixCSC{Tv}, I::AbstractArray)
     idxB = 1
 
     for i in 1:n
+        ((I[i] < 1) | (I[i] > nA)) && throw(BoundsError())
         row,col = ind2sub(szA, I[i])
         for r in colptrA[col]:(colptrA[col+1]-1)
             @inbounds if rowvalA[r] == row
@@ -1221,7 +1396,7 @@ function spset!{Tv,Ti<:Integer}(A::SparseMatrixCSC{Tv}, x::Tv, I::AbstractVector
         colptrA[n+1] = rowidx
         deleteat!(rowvalA, rowidx:nnzA)
         deleteat!(nzvalA, rowidx:nnzA)
-        
+
         A.colptr = colptrA
         A.rowval = rowvalA
         A.nzval = nzvalA
@@ -1278,7 +1453,7 @@ function spdelete!{Tv,Ti<:Integer}(A::SparseMatrixCSC{Tv}, I::AbstractVector{Ti}
         colptrA[n+1] = rowidx
         deleteat!(rowvalA, rowidx:nnzA)
         deleteat!(nzvalA, rowidx:nnzA)
-        
+
         A.colptr = colptrA
         A.rowval = rowvalA
         A.nzval = nzvalA
@@ -1329,7 +1504,7 @@ function setindex!{Tv,Ti,T<:Integer}(A::SparseMatrixCSC{Tv,Ti}, B::SparseMatrixC
     colptrS = Array(Ti, n+1)
     rowvalS = Array(Ti, nnzS)
     nzvalS = Array(Tv, nnzS)
-    
+
     colptrS[1] = 1
     colB = 1
     asgn_col = J[colB]
@@ -1344,7 +1519,7 @@ function setindex!{Tv,Ti,T<:Integer}(A::SparseMatrixCSC{Tv,Ti}, B::SparseMatrixC
         # Copy column of A if it is not being assigned into
         if colB > nJ || col != J[colB]
             colptrS[col+1] = colptrS[col] + (colptrA[col+1]-colptrA[col])
-            
+
             for k = colptrA[col]:colptrA[col+1]-1
                 rowvalS[ptrS] = rowvalA[k]
                 nzvalS[ptrS] = nzvalA[k]
@@ -1637,7 +1812,7 @@ end
 
 function vcat(X::SparseMatrixCSC...)
     num = length(X)
-    mX = [ size(x, 1) for x in X ] 
+    mX = [ size(x, 1) for x in X ]
     nX = [ size(x, 2) for x in X ]
     n = nX[1]
     for i = 2 : num
@@ -1755,6 +1930,8 @@ function blkdiag(X::SparseMatrixCSC...)
     SparseMatrixCSC(m, n, colptr, rowval, nzval)
 end
 
+squeeze(S::SparseMatrixCSC, dims::Dims) = throw(ArgumentError("squeeze is not available for sparse matrices"))
+
 ## Structure query functions
 
 function issym(A::SparseMatrixCSC)
@@ -1808,7 +1985,7 @@ function istril{Tv}(A::SparseMatrixCSC{Tv})
     return true
 end
 
-# Create a sparse diagonal matrix by specifying multiple diagonals 
+# Create a sparse diagonal matrix by specifying multiple diagonals
 # packed into a tuple, alongside their diagonal offsets and matrix shape
 
 function spdiagm_internal(B, d)
@@ -1820,7 +1997,7 @@ function spdiagm_internal(B, d)
     end
     I = Array(Int, ncoeffs)
     J = Array(Int, ncoeffs)
-    V = Array(eltype(B[1]), ncoeffs)
+    V = Array(promote_type(map(eltype, B)...), ncoeffs)
     id = 0
     i = 0
     for vec in B
@@ -1944,5 +2121,96 @@ function diagm{Tv,Ti}(v::SparseMatrixCSC{Tv,Ti})
         end
     end
 
-    return SparseMatrixCSC{Tv,Ti}(n, n, colptr, rowval, nzval)
+    return SparseMatrixCSC(n, n, colptr, rowval, nzval)
+end
+
+# Sort all the indices in each column of a CSC sparse matrix
+# sortSparseMatrixCSC!(A, sortindices = :sortcols)        # Sort each column with sort()
+# sortSparseMatrixCSC!(A, sortindices = :doubletranspose) # Sort with a double transpose
+function sortSparseMatrixCSC!{Tv,Ti}(A::SparseMatrixCSC{Tv,Ti}; sortindices::Symbol = :sortcols)
+    if sortindices == :doubletranspose
+        nB, mB = size(A)
+        B = SparseMatrixCSC(mB, nB, Array(Ti, nB+1), similar(A.rowval), similar(A.nzval))
+        transpose!(B, A)
+        transpose!(A, B)
+        return A
+    end
+
+    m, n = size(A)
+    colptr = A.colptr; rowval = A.rowval; nzval = A.nzval
+
+    index = zeros(Ti, m)
+    row = zeros(Ti, m)
+    val = zeros(Tv, m)
+
+    for i = 1:n
+        @inbounds col_start = colptr[i]
+        @inbounds col_end = (colptr[i+1] - 1)
+
+        numrows = col_end - col_start + 1
+        if numrows <= 1
+            continue
+        elseif numrows == 2
+            f = col_start
+            s = f+1
+            if rowval[f] > rowval[s]
+                @inbounds rowval[f], rowval[s] = rowval[s], rowval[f]
+                @inbounds nzval[f],  nzval[s]  = nzval[s],  nzval[f]
+            end
+            continue
+        end
+
+        jj = 1
+        @simd for j = col_start:col_end
+            @inbounds row[jj] = rowval[j]
+            @inbounds val[jj] = nzval[j]
+            jj += 1
+        end
+
+        sortperm!(pointer_to_array(pointer(index), numrows),
+                  pointer_to_array(pointer(row), numrows))
+
+        jj = 1;
+        @simd for j = col_start:col_end
+            @inbounds rowval[j] = row[index[jj]]
+            @inbounds nzval[j] = val[index[jj]]
+            jj += 1
+        end
+    end
+
+    return A
+end
+
+## rotations
+
+function rot180(A::SparseMatrixCSC)
+    I,J,V = findnz(A)
+    m,n = size(A)
+    for i=1:length(I)
+        I[i] = m - I[i] + 1
+        J[i] = n - J[i] + 1
+    end
+    return sparse(I,J,V,m,n)
+end
+
+function rotr90(A::SparseMatrixCSC)
+    I,J,V = findnz(A)
+    m,n = size(A)
+    #old col inds are new row inds
+    newJ = similar(J)
+    for i=1:length(newJ)
+        newJ[i] = m - I[i] + 1
+    end
+    return sparse(J, newJ, V, n, m)
+end
+
+function rotl90(A::SparseMatrixCSC)
+    I,J,V = findnz(A)
+    m,n = size(A)
+    #old row inds are new col inds
+    newI = similar(I)
+    for i=1:length(newI)
+        newI[i] = n - J[i] + 1
+    end
+    return sparse(newI, I, V, n, m)
 end
