@@ -2,7 +2,7 @@
 
 length(t::Tuple) = tuplelen(t)
 endof(t::Tuple) = tuplelen(t)
-size(t::Tuple, d) = d==1 ? tuplelen(t) : error("invalid tuple dimension")
+size(t::Tuple, d) = d==1 ? tuplelen(t) : error("invalid tuple dimension $(d)")
 getindex(t::Tuple, i::Int) = tupleref(t, i)
 getindex(t::Tuple, i::Real) = tupleref(t, convert(Int, i))
 getindex(t::Tuple, r::AbstractArray) = tuple([t[ri] for ri in r]...)
@@ -20,6 +20,10 @@ indexed_next(t::Tuple, i::Int, state) = (t[i], i+1)
 indexed_next(a::Array, i::Int, state) = (a[i], i+1)
 indexed_next(I, i, state) = done(I,state) ? throw(BoundsError()) : next(I, state)
 
+# eltype
+
+eltype{T}(x::(T...)) = T
+
 ## mapping ##
 
 ntuple(n::Integer, f::Function) = ntuple(f, n) # TODO: deprecate this?
@@ -32,6 +36,9 @@ ntuple(f::Function, n::Integer) =
     n==5 ? (f(1),f(2),f(3),f(4),f(5),) :
     tuple(ntuple(n-2,f)..., f(n-1), f(n))
 
+argtail(x, rest...) = rest
+tail(x::Tuple) = argtail(x...)
+
 # 0 argument function
 map(f::Callable) = f()
 # 1 argument function
@@ -39,28 +46,19 @@ map(f::Callable, t::())                   = ()
 map(f::Callable, t::(Any,))               = (f(t[1]),)
 map(f::Callable, t::(Any, Any))           = (f(t[1]), f(t[2]))
 map(f::Callable, t::(Any, Any, Any))      = (f(t[1]), f(t[2]), f(t[3]))
-map(f::Callable, t::(Any, Any, Any, Any)) = (f(t[1]), f(t[2]), f(t[3]), f(t[4]))
-map(f::Callable, t::Tuple)                = tuple([f(ti) for ti in t]...)
+map(f::Callable, t::Tuple)                = tuple(f(t[1]), map(f,tail(t))...)
 # 2 argument function
 map(f::Callable, t::(),        s::())        = ()
 map(f::Callable, t::(Any,),    s::(Any,))    = (f(t[1],s[1]),)
 map(f::Callable, t::(Any,Any), s::(Any,Any)) = (f(t[1],s[1]), f(t[2],s[2]))
-map(f::Callable, t::(Any,Any,Any), s::(Any,Any,Any)) =
-    (f(t[1],s[1]), f(t[2],s[2]), f(t[3],s[3]))
-map(f::Callable, t::(Any,Any,Any,Any), s::(Any,Any,Any,Any)) =
-    (f(t[1],s[1]), f(t[2],s[2]), f(t[3],s[3]), f(t[4],s[4]))
 # n argument function
-map(f::Callable, ts::Tuple...) = tuple([f(map(t->t[n],ts)...) for n=1:length_checked_equal(ts...)]...)
-
-function length_checked_equal(args...) 
-    n = length(args[1])
-    for i=2:length(args)
-        if length(args[i]) != n
-            error("argument dimensions must match")
-        end
-    end
-    n
-end
+heads() = ()
+heads(t::Tuple, ts::Tuple...) = tuple(t[1], heads(ts...)...)
+tails() = ()
+tails(t::Tuple, ts::Tuple...) = tuple(tail(t), tails(ts...)...)
+map(f::Callable, ::(), ts::Tuple...) = ()
+map(f::Callable, ts::Tuple...) =
+    tuple(f(heads(ts...)...), map(f, tails(ts...)...)...)
 
 ## comparison ##
 
@@ -88,6 +86,11 @@ function ==(t1::Tuple, t2::Tuple)
     return true
 end
 
+hash(::(), h::Uint) = h + uint(0x77cfa1eef01bca90)
+hash(x::(Any,), h::Uint)    = hash(x[1], hash((), h))
+hash(x::(Any,Any), h::Uint) = hash(x[1], hash(x[2], hash((), h)))
+hash(x::Tuple, h::Uint)     = hash(x[1], hash(x[2], hash(tupletail(x), h)))
+
 function isless(t1::Tuple, t2::Tuple)
     n1, n2 = length(t1), length(t2)
     for i = 1:min(n1, n2)
@@ -104,15 +107,18 @@ end
 isempty(x::()) = true
 isempty(x::Tuple) = false
 
-reverse(x::Tuple) = (n=length(x); tuple([x[n-k+1] for k=1:n]...))
+revargs() = ()
+revargs(x, r...) = tuple(revargs(r...)..., x)
+
+reverse(t::Tuple) = revargs(t...)
 
 ## specialized reduction ##
 
 # TODO: these definitions cannot yet be combined, since +(x...)
 # where x might be any tuple matches too many methods.
-sum(x::()) = 0
 sum(x::(Any, Any...)) = +(x...)
 
+# NOTE: should remove, but often used on array sizes
 prod(x::()) = 1
 prod(x::(Any, Any...)) = *(x...)
 
