@@ -12,8 +12,8 @@
 extern "C" {
 #endif
 
-jl_value_t *jl_true;
-jl_value_t *jl_false;
+DLLEXPORT jl_value_t *jl_true;
+DLLEXPORT jl_value_t *jl_false;
 
 jl_datatype_t *jl_undef_type;
 jl_tvar_t     *jl_typetype_tvar;
@@ -58,11 +58,11 @@ jl_datatype_t *jl_pointer_type;
 jl_datatype_t *jl_voidpointer_type;
 jl_value_t *jl_an_empty_cell=NULL;
 jl_value_t *jl_stackovf_exception;
-jl_value_t *jl_diverror_exception;
-jl_value_t *jl_domain_exception;
-jl_value_t *jl_overflow_exception;
-jl_value_t *jl_inexact_exception;
-jl_value_t *jl_undefref_exception;
+DLLEXPORT jl_value_t *jl_diverror_exception;
+DLLEXPORT jl_value_t *jl_domain_exception;
+DLLEXPORT jl_value_t *jl_overflow_exception;
+DLLEXPORT jl_value_t *jl_inexact_exception;
+DLLEXPORT jl_value_t *jl_undefref_exception;
 jl_value_t *jl_interrupt_exception;
 jl_value_t *jl_bounds_exception;
 jl_value_t *jl_memory_exception;
@@ -101,7 +101,7 @@ typedef struct {
 static size_t jl_new_bits_align(jl_value_t *dt)
 {
     if (jl_is_tuple(dt)) {
-        size_t i, l = jl_tuple_len(dt), align = 0;
+        size_t i, l = jl_tuple_len(dt), align = 1;
         for (i = 0; i < l; i++) {
             size_t l = jl_new_bits_align(jl_tupleref(dt,i));
             if (l > align)
@@ -127,8 +127,11 @@ static jl_value_t *jl_new_bits_internal(jl_value_t *dt, void *data, size_t *len)
         return v;
     }
 
+    assert(jl_is_datatype(dt));
     jl_datatype_t *bt = (jl_datatype_t*)dt;
     size_t nb = jl_datatype_size(bt);
+    if (nb == 0)
+        return jl_new_struct_uninit(bt);
     *len = LLT_ALIGN(*len, bt->alignment);
     data = (char*)data + (*len);
     *len += nb;
@@ -138,7 +141,7 @@ static jl_value_t *jl_new_bits_internal(jl_value_t *dt, void *data, size_t *len)
     if (bt == jl_int32_type)   return jl_box_int32(*(int32_t*)data);
     if (bt == jl_float64_type) return jl_box_float64(*(double*)data);
 
-    jl_value_t *v = 
+    jl_value_t *v =
         (jl_value_t*)allocobj((NWORDS(LLT_ALIGN(nb,sizeof(void*)))+1)*
                               sizeof(void*));
     v->type = (jl_value_t*)bt;
@@ -457,7 +460,7 @@ static jl_sym_t *mk_symbol(const char *str)
 static void unmark_symbols_(jl_sym_t *root)
 {
     while (root != NULL) {
-        root->type = (jl_value_t*)(((uptrint_t)root->type)&~1UL);
+        root->type = (jl_value_t*)(((ptrint_t)root->type)&~((ptrint_t)1));
         unmark_symbols_(root->left);
         root = root->right;
     }
@@ -577,7 +580,7 @@ void jl_add_constructors(jl_datatype_t *t)
         if (t != (jl_datatype_t*)t->name->primary) {
             // instantiating
             assert(jl_is_function(t->ctor_factory));
-            
+
             // add type's static parameters to the ctor factory
             size_t np = jl_tuple_len(t->parameters);
             jl_tuple_t *sparams = jl_alloc_tuple_uninit(np*2);
@@ -593,7 +596,7 @@ void jl_add_constructors(jl_datatype_t *t)
                                              sparams);
             cfactory->linfo->ast = jl_prepare_ast(cfactory->linfo,
                                                   cfactory->linfo->sparams);
-            
+
             // call user-defined constructor factory on (type,)
             jl_value_t *cfargs[1] = { (jl_value_t*)t };
             jl_apply(cfactory, cfargs, 1);
@@ -618,19 +621,21 @@ jl_datatype_t *jl_new_uninitialized_datatype(size_t nfields)
 
 void jl_compute_field_offsets(jl_datatype_t *st)
 {
-    size_t sz = 0, alignm = 0;
+    size_t sz = 0, alignm = 1;
     int ptrfree = 1;
 
     for(size_t i=0; i < jl_tuple_len(st->types); i++) {
         jl_value_t *ty = jl_tupleref(st->types, i);
         size_t fsz, al;
-        if (jl_isbits(ty) && (al=((jl_datatype_t*)ty)->alignment)!=0 &&
+        if (jl_isbits(ty) && (fsz = jl_datatype_size(ty))!=0 &&
             jl_is_leaf_type(ty)) {
-            fsz = jl_datatype_size(ty);
+            al = ((jl_datatype_t*)ty)->alignment;
             st->fields[i].isptr = 0;
         }
         else {
             fsz = sizeof(void*);
+            if (fsz > MAX_ALIGN)
+                fsz = MAX_ALIGN;
             al = fsz;
             st->fields[i].isptr = 1;
             ptrfree = 0;
@@ -688,7 +693,7 @@ jl_datatype_t *jl_new_datatype(jl_sym_t *name, jl_datatype_t *super,
     t->instance = NULL;
     t->struct_decl = NULL;
     t->size = 0;
-    t->alignment = 0;
+    t->alignment = 1;
 
     if (tn == NULL) {
         t->name = NULL;
