@@ -324,8 +324,15 @@ JL_CALLABLE(jl_f_apply)
         }
         else {
             size_t al = jl_array_len(args[i]);
-            for(j=0; j < al; j++)
-                newargs[n++] = jl_cellref(args[i], j);
+            for (j = 0;j < al;j++) {
+                jl_value_t *arg = jl_cellref(args[i], j);
+                // apply with array splatting may have embedded NULL value
+                // #11772
+                if (__unlikely(arg == NULL)) {
+                    jl_throw(jl_undefref_exception);
+                }
+                newargs[n++] = arg;
+            }
         }
     }
     result = jl_apply((jl_function_t*)args[0], newargs, n);
@@ -627,9 +634,18 @@ DLLEXPORT int jl_substrtod(char *str, size_t offset, int len, double *out)
         pend = bstr+len;
     }
     *out = strtod_c(bstr, &p);
-    if ((p == bstr) || (p != pend) ||
+    if (p == bstr ||
         (errno==ERANGE && (*out==0 || *out==HUGE_VAL || *out==-HUGE_VAL)))
         err = 1;
+    // Deal with case where the substring might be something like "1 ",
+    // which is OK, and "1 X", which we don't allow.
+    while (p != pend) {
+        if (!isspace((unsigned char)*p)) {
+            err = 1;
+            break;
+        }
+        p++;
+    }
     if (bstr != str+offset)
         free(bstr);
     return err;
@@ -677,9 +693,18 @@ DLLEXPORT int jl_substrtof(char *str, int offset, int len, float *out)
     *out = strtof_c(bstr, &p);
 #endif
 
-    if ((p == bstr) || (p != pend) ||
+    if (p == bstr ||
         (errno==ERANGE && (*out==0 || *out==HUGE_VALF || *out==-HUGE_VALF)))
         err = 1;
+    // Deal with case where the substring might be something like "1 ",
+    // which is OK, and "1 X", which we don't allow.
+    while (p != pend) {
+        if (!isspace((unsigned char)*p)) {
+            err = 1;
+            break;
+        }
+        p++;
+    }
     if (bstr != str+offset)
         free(bstr);
     return err;
@@ -737,7 +762,7 @@ void jl_show(jl_value_t *stream, jl_value_t *v)
         }
         if (jl_show_gf==NULL || stream==NULL) {
             JL_PRINTF(JL_STDERR, " could not show value of type %s",
-                      jl_is_tuple(v) ? "Tuple" : 
+                      jl_is_tuple(v) ? "Tuple" :
                       ((jl_datatype_t*)jl_typeof(v))->name->name->name);
             return;
         }
@@ -1000,7 +1025,7 @@ void jl_init_primitives(void)
     add_builtin_func("eval", jl_f_top_eval);
     add_builtin_func("isdefined", jl_f_isdefined);
     add_builtin_func("yieldto", jl_f_yieldto);
-    
+
     // functions for internal use
     add_builtin_func("convert_default", jl_f_convert_default);
     add_builtin_func("tupleref",  jl_f_tupleref);
