@@ -11,7 +11,7 @@ static GlobalVariable *prepare_global(GlobalVariable *G)
                                     G->isConstant(), GlobalVariable::ExternalLinkage,
                                     NULL, G->getName());
         }
-        return gv;     
+        return gv;
     }
 #endif
     return G;
@@ -42,10 +42,46 @@ static llvm::Value *prepare_call(llvm::Value* Callee)
 static inline void add_named_global(GlobalValue *gv, void *addr)
 {
 #ifdef USE_MCJIT
-    sys::DynamicLibrary::AddSymbol(gv->getName(),addr);
-#else
-    jl_ExecutionEngine->addGlobalMapping(gv,addr);
+
+    StringRef name = gv->getName();
+#ifdef _OS_WINDOWS_
+    std::string imp_name;
+    // setting DLLEXPORT correctly only matters when building a binary
+    if (jl_compileropts.build_path != NULL) {
+        // add the __declspec(dllimport) attribute
+        gv->setDLLStorageClass(GlobalValue::DLLImportStorageClass);
+        // this will cause llvm to rename it, so we do the same
+        imp_name = Twine("__imp_", name).str();
+        name = StringRef(imp_name);
+        // __imp_ functions are jmp stubs (no additional work needed)
+        // __imp_ variables are indirection pointers, so use malloc to simulate that too
+        if (isa<GlobalVariable>(gv)) {
+            void** imp_addr = (void**)malloc(sizeof(void**));
+            *imp_addr = addr;
+            addr = (void*)imp_addr;
+        }
+    }
 #endif
+    sys::DynamicLibrary::AddSymbol(name, addr);
+
+#else // USE_MCJIT
+
+#ifdef _OS_WINDOWS_
+    // setting DLLEXPORT correctly only matters when building a binary
+    if (jl_compileropts.build_path != NULL) {
+        if (gv->getLinkage() == GlobalValue::ExternalLinkage)
+            gv->setLinkage(GlobalValue::DLLImportLinkage);
+#ifdef _P64
+        // the following is correct by observation,
+        // as long as everything stays within a 32-bit offset :/
+        void** imp_addr = (void**)malloc(sizeof(void**));
+        *imp_addr = addr;
+        addr = (void*)imp_addr;
+#endif
+    }
+#endif // _OS_WINDOWS_
+    jl_ExecutionEngine->addGlobalMapping(gv, addr);
+#endif // USE_MCJIT
 }
 
 // --- string constants ---
@@ -100,7 +136,7 @@ public:
         ValueMaterializer(), VMap(), destModule(dest), srcModule(src)
     {
 
-    } 
+    }
     ValueToValueMapTy VMap;
     llvm::Module *destModule;
     llvm::Module *srcModule;
@@ -119,11 +155,11 @@ public:
                     uint64_t addr = jl_mcjmm->getSymbolAddress(F->getName());
                     if (addr == 0) {
                         return clone_llvm_function(shadow,this);
-                    } 
+                    }
                     else {
                         return destModule->getOrInsertFunction(F->getName(),F->getFunctionType());
                     }
-                } 
+                }
                 else if (!F->isDeclaration()) {
                     return clone_llvm_function(F,this);
                 }
@@ -133,7 +169,7 @@ public:
                 // Create forward declaration in current module
                 return destModule->getOrInsertFunction(F->getName(),F->getFunctionType());
             }
-        } 
+        }
         else if (isa<GlobalVariable>(V)) {
             GlobalVariable *GV = cast<GlobalVariable>(V);
             assert(GV != NULL);
@@ -156,7 +192,7 @@ public:
             if (it != llvm_to_jl_value.end()) {
                 newGV->setInitializer(Constant::getIntegerValue(GV->getType()->getElementType(),APInt(sizeof(void*)*8,(ptrint_t)it->second)));
                 newGV->setConstant(true);
-            } 
+            }
             else if (GV->hasInitializer()) {
                 Value *C = MapValue(GV->getInitializer(),VMap,RF_None,NULL,this);
                 newGV->setInitializer(cast<Constant>(C));
@@ -240,7 +276,7 @@ static void jl_gen_llvm_gv_array()
             ConstantInt::get(T_size,globalUnique+1),
             "jl_globalUnique");
 
-    Constant *feature_string = ConstantDataArray::getString(jl_LLVMContext, jl_cpu_string);
+    Constant *feature_string = ConstantDataArray::getString(jl_LLVMContext, jl_compileropts.cpu_target);
     new GlobalVariable(*jl_Module,
                        feature_string->getType(),
                        true,
@@ -249,7 +285,7 @@ static void jl_gen_llvm_gv_array()
                        "jl_sysimg_cpu_target");
 
     // For native also store the cpuid
-    if (strcmp(jl_cpu_string,"native") == 0) {
+    if (strcmp(jl_compileropts.cpu_target,"native") == 0) {
         uint32_t info[4];
 
         jl_cpuid((int32_t*)info, 1);
