@@ -1,18 +1,15 @@
-#if USE_MCJIT
+#ifdef USE_MCJIT
 typedef object::SymbolRef SymRef;
 #endif
 
 // --- storing and accessing source location metadata ---
 
 #ifndef USE_MCJIT
-struct FuncInfo{
+struct FuncInfo {
     const Function* func;
     size_t lengthAdr;
     std::string name;
     std::string filename;
-#if defined(_OS_WINDOWS_) && defined(_CPU_X86_64_)
-    PRUNTIME_FUNCTION fnentry;
-#endif
     std::vector<JITEvent_EmittedFunctionDetails::LineStart> lines;
 };
 #else
@@ -24,11 +21,83 @@ struct ObjectInfo {
 #endif
 
 #if defined(_OS_WINDOWS_)
-#include <dbghelp.h>
 #if defined(_CPU_X86_64_)
 extern "C" EXCEPTION_DISPOSITION _seh_exception_handler(PEXCEPTION_RECORD ExceptionRecord,void *EstablisherFrame, PCONTEXT ContextRecord, void *DispatcherContext);
-extern "C" volatile int jl_in_stackwalk;
 #endif
+#include <dbghelp.h>
+static void create_PRUNTIME_FUNCTION(uint8_t *Code, size_t Size, StringRef fnname,
+        uint8_t *Section, size_t Allocated)
+{
+    DWORD mod_size = 0;
+#if defined(_CPU_X86_64_)
+    uint8_t *catchjmp = Section+Allocated;
+    uint8_t *UnwindData = (uint8_t*)(((uintptr_t)catchjmp+12+3)&~(uintptr_t)3);
+    if (!catchjmp[0]) {
+        catchjmp[0] = 0x48;
+        catchjmp[1] = 0xb8; // mov RAX, QWORD PTR [...]
+        *(uint64_t*)(&catchjmp[2]) = (uint64_t)&_seh_exception_handler;
+        catchjmp[10] = 0xff;
+        catchjmp[11] = 0xe0; // jmp RAX
+        UnwindData[0] = 0x09; // version info, UNW_FLAG_EHANDLER
+        UnwindData[1] = 4;    // size of prolog (bytes)
+        UnwindData[2] = 2;    // count of unwind codes (slots)
+        UnwindData[3] = 0x05; // frame register (rbp) = rsp
+        UnwindData[4] = 4;    // second instruction
+        UnwindData[5] = 0x03; // mov RBP, RSP
+        UnwindData[6] = 1;    // first instruction
+        UnwindData[7] = 0x50; // push RBP
+        *(DWORD*)&UnwindData[8] = (DWORD)Allocated; // relative location of catchjmp
+        mod_size = (DWORD)Allocated+48;
+    }
+#if !defined(USE_MCJIT)
+    PRUNTIME_FUNCTION tbl = (PRUNTIME_FUNCTION)(UnwindData+12);
+#else
+    PRUNTIME_FUNCTION tbl = (PRUNTIME_FUNCTION)malloc(sizeof(RUNTIME_FUNCTION));
+#endif
+    tbl->BeginAddress = (DWORD)(Code - Section);
+    tbl->EndAddress = (DWORD)(intptr_t)(Code + Size - Section);
+    tbl->UnwindData = (DWORD)(intptr_t)(UnwindData - Section);
+#else // defined(_CPU_X86_64_)
+    Section = Code;
+    mod_size = Size;
+#endif
+    if (0) {
+        assert(!jl_in_stackwalk);
+        jl_in_stackwalk = 1;
+        if (mod_size && !SymLoadModuleEx(GetCurrentProcess(), NULL, NULL, NULL, (DWORD64)Section, mod_size, NULL, SLMFLAG_VIRTUAL)) {
+#if defined(_CPU_X86_64_)
+            catchjmp[0] = 0;
+#endif
+            static int warned = 0;
+            if (!warned) {
+                JL_PRINTF(JL_STDERR, "WARNING: failed to insert module info for backtrace: %d\n", GetLastError());
+                warned = 1;
+            }
+        }
+        else {
+            size_t len = fnname.size()+1;
+            if (len > MAX_SYM_NAME)
+                len = MAX_SYM_NAME;
+            char *name = (char*)alloca(len);
+            memcpy(name, fnname.data(), len-1);
+            name[len-1] = 0;
+            if (!SymAddSymbol(GetCurrentProcess(), (ULONG64)Section, name,
+                        (DWORD64)Code, (DWORD)Size, 0)) {
+                JL_PRINTF(JL_STDERR, "WARNING: failed to insert function name %s into debug info: %d\n", name, GetLastError());
+            }
+        }
+        jl_in_stackwalk = 0;
+    }
+#if defined(_CPU_X86_64_)
+    if (!RtlAddFunctionTable(tbl, 1, (DWORD64)Section)) {
+        static int warned = 0;
+        if (!warned) {
+            JL_PRINTF(JL_STDERR, "WARNING: failed to insert function stack unwind info: %d\n", GetLastError());
+            warned = 1;
+        }
+    }
+#endif
+}
 #endif
 
 struct revcomp {
@@ -52,50 +121,10 @@ public:
     virtual void NotifyFunctionEmitted(const Function &F, void *Code,
                                        size_t Size, const EmittedFunctionDetails &Details)
     {
-#if defined(_OS_WINDOWS_) && defined(_CPU_X86_64_)
-        assert(!jl_in_stackwalk);
-        jl_in_stackwalk = 1;
-        uintptr_t catchjmp = (uintptr_t)Code+Size;
-        *(uint8_t*)(catchjmp+0) = 0x48;
-        *(uint8_t*)(catchjmp+1) = 0xb8; // mov RAX, QWORD PTR [...]
-        *(uint64_t*)(catchjmp+2) = (uint64_t)&_seh_exception_handler;
-        *(uint8_t*)(catchjmp+10) = 0xff;
-        *(uint8_t*)(catchjmp+11) = 0xe0; // jmp RAX
-        PRUNTIME_FUNCTION tbl = (PRUNTIME_FUNCTION)((catchjmp+12+3)&~(uintptr_t)3);
-        uint8_t *UnwindData = (uint8_t*)((((uintptr_t)&tbl[1])+3)&~(uintptr_t)3);
-        RUNTIME_FUNCTION fn = {0,(DWORD)Size+13,(DWORD)(intptr_t)(UnwindData-(uint8_t*)Code)};
-        tbl[0] = fn;
-        UnwindData[0] = 0x09; // version info, UNW_FLAG_EHANDLER
-        UnwindData[1] = 4;    // size of prolog (bytes)
-        UnwindData[2] = 2;    // count of unwind codes (slots)
-        UnwindData[3] = 0x05; // frame register (rbp) = rsp
-        UnwindData[4] = 4;    // second instruction
-        UnwindData[5] = 0x03; // mov RBP, RSP
-        UnwindData[6] = 1;    // first instruction
-        UnwindData[7] = 0x50; // push RBP
-        *(DWORD*)&UnwindData[8] = (DWORD)(catchjmp-(intptr_t)Code);
-        DWORD mod_size = (DWORD)(size_t)(&UnwindData[8]-(uint8_t*)Code);
-        if (!SymLoadModuleEx(GetCurrentProcess(), NULL, NULL, NULL, (DWORD64)Code, mod_size, NULL, SLMFLAG_VIRTUAL)) {
-            static int warned = 0;
-            if (!warned) {
-                JL_PRINTF(JL_STDERR, "WARNING: failed to insert function info for backtrace\n");
-                warned = 1;
-            }
-        }
-        else {
-            if (!SymAddSymbol(GetCurrentProcess(), (ULONG64)Code, F.getName().data(), (DWORD64)Code, mod_size, 0)) {
-                JL_PRINTF(JL_STDERR, "WARNING: failed to insert function name into debug info\n");
-            }
-            if (!RtlAddFunctionTable(tbl,1,(DWORD64)Code)) {
-                JL_PRINTF(JL_STDERR, "WARNING: failed to insert function stack unwind info\n");
-            }
-        }
-        jl_in_stackwalk = 0;
-
-        FuncInfo tmp = {&F, Size, std::string(), std::string(), tbl, Details.LineStarts};
-#else
-        FuncInfo tmp = {&F, Size, std::string(F.getName().data()), std::string(), Details.LineStarts};
+#if defined(_OS_WINDOWS_)
+        create_PRUNTIME_FUNCTION((uint8_t*)Code, Size, F.getName(), (uint8_t*)Code, Size);
 #endif
+        FuncInfo tmp = {&F, Size, F.getName().str(), std::string(), Details.LineStarts};
         info[(size_t)(Code)] = tmp;
     }
 
@@ -103,24 +132,41 @@ public:
     {
         return info;
     }
-#endif // ndef USE_MCJIT
+#endif // ifndef USE_MCJIT
 
-#if USE_MCJIT
+#ifdef USE_MCJIT
     virtual void NotifyObjectEmitted(const ObjectImage &obj)
     {
         uint64_t Addr;
+        uint64_t Size;
         object::SymbolRef::Type SymbolType;
+#ifdef _OS_WINDOWS_
+        StringRef Name;
+        object::section_iterator Section = obj.begin_sections();
+        uint64_t SectionAddr;
+        uint64_t SectionSize;
+#endif
 
         #ifdef LLVM35
         for (const object::SymbolRef &sym_iter : obj.symbols()) {
             sym_iter.getType(SymbolType);
             if (SymbolType != object::SymbolRef::ST_Function) continue;
             sym_iter.getAddress(Addr);
-            ObjectInfo tmp = {obj.getObjectFile(), sym_iter, obj.getData().size()};
+            sym_iter.getSize(Size);
+#ifdef _OS_WINDOWS_
+            sym_iter.getName(Name);
+            sym_iter.getSection(Section);
+            Section->getAddress(SectionAddr);
+            Section->getSize(SectionSize);
+            create_PRUNTIME_FUNCTION(
+                   (uint8_t*)(intptr_t)Addr, (size_t)Size, Name,
+                   (uint8_t*)(intptr_t)SectionAddr, (size_t)SectionSize);
+#endif
+            ObjectInfo tmp = {obj.getObjectFile(), sym_iter, (size_t)Size};
             objectmap[Addr] = tmp;
         }
         #else
-        error_code itererr; 
+        error_code itererr;
         object::symbol_iterator sym_iter = obj.begin_symbols();
         object::symbol_iterator sym_end = obj.end_symbols();
         for (; sym_iter != sym_end; sym_iter.increment(itererr)) {
@@ -147,16 +193,19 @@ public:
 extern "C"
 const char *jl_demangle(const char *name)
 {
-    const char *start = name;
-    const char *end = start;
+    const char *start = name + 6;
+    const char *end = name + strlen(name);
     char *ret;
-    while ((*start++ != '_') && (*start != '\0'));
-    if (*name == '\0') goto done;
-    while ((*end++ != ';') && (*end != '\0'));
-    if (*name == '\0') goto done;
-    ret = (char*)malloc(end-start);
-    memcpy(ret,start,end-start-1);
-    ret[end-start-1] = '\0';
+    if (strncmp(name, "julia_", 6)) goto done;
+    if (*start == '\0') goto done;
+    while (*(--end) != '_') {
+        char c = *end;
+        if (c < '0' || c > '9') goto done;
+    }
+    if (end <= start) goto done;
+    ret = (char*)malloc(end-start+1);
+    memcpy(ret,start,end-start);
+    ret[end-start] = '\0';
     return ret;
  done:
     return strdup(name);
@@ -171,17 +220,17 @@ void lookup_pointer(DIContext *context, const char **name, size_t *line, const c
     DILineInfo info;
     if (demangle && *name != NULL)
         *name = jl_demangle(*name);
-    #ifdef LLVM35
+#ifdef LLVM35
     DILineInfoSpecifier infoSpec(DILineInfoSpecifier::FileLineInfoKind::AbsoluteFilePath,
                                  DILineInfoSpecifier::FunctionNameKind::ShortName);
-    #else
+#else
     int infoSpec = DILineInfoSpecifier::FileLineInfo |
                    DILineInfoSpecifier::AbsoluteFilePath |
                    DILineInfoSpecifier::FunctionName;
-    #endif
+#endif
     if (context == NULL) goto done;
     info = context->getLineInfoForAddress(pointer, infoSpec);
-    #ifndef LLVM35 // LLVM <= 3.4
+#ifndef LLVM35 // LLVM <= 3.4
     if (strcmp(info.getFunctionName(), "<invalid>") == 0) goto done;
     if (demangle)
         *name = jl_demangle(info.getFunctionName());
@@ -189,13 +238,13 @@ void lookup_pointer(DIContext *context, const char **name, size_t *line, const c
         *name = strdup(info.getFunctionName());
     *line = info.getLine();
     *filename = strdup(info.getFileName());
-    #else
+#else
     if (strcmp(info.FunctionName.c_str(), "<invalid>") == 0) goto done;
     *name = strdup(info.FunctionName.c_str());
     *line = info.Line;
     *filename = strdup(info.FileName.c_str());
-    #endif
-    done:
+#endif
+done:
     // If this is a jlcall wrapper, set fromC to match JIT behavior
     if (*name == NULL || memcmp(*name,"jlcall_",7) == 0)
         *fromC = true;
@@ -245,78 +294,106 @@ bool getObjUUID(llvm::object::MachOObjectFile *obj, uint8_t uuid[16])
 }
 #endif
 
-extern char *jl_sysimage_name;
+extern "C" uint64_t jl_sysimage_base;
 
-bool jl_is_sysimg(const char *path)
-{
-    if (!jl_sysimage_name)
-        return 0;
-    const char *filename = strrchr(path,'/');
-    if (filename == NULL)
-        filename = path;
-    const char *sysimgname = strrchr(jl_sysimage_name,'/');
-    if (sysimgname == NULL)
-        sysimgname = jl_sysimage_name;
-    return strncmp(filename,sysimgname,strrchr(path,'.')-filename) == 0;
-}
-
-#if defined(_OS_WINDOWS_) && !defined(USE_MCJIT)
-void jl_getDylibFunctionInfo(const char **name, size_t *line, const char **filename, size_t pointer, int *fromC, int skipC)
-{
-    return;
-}
-#else
 void jl_getDylibFunctionInfo(const char **name, size_t *line, const char **filename, size_t pointer, int *fromC, int skipC)
 {
 #ifdef _OS_WINDOWS_
-    DWORD fbase = SymGetModuleBase64(GetCurrentProcess(),(DWORD)pointer);
-    char *fname = 0;
-    if (fbase != 0) {
-#else
+    IMAGEHLP_MODULE64 ModuleInfo;
+    BOOL isvalid;
+    if (jl_in_stackwalk) {
+        *fromC = 1;
+        return;
+    }
+    ModuleInfo.SizeOfStruct = sizeof(IMAGEHLP_MODULE64);
+    jl_in_stackwalk = 1;
+    isvalid = SymGetModuleInfo64(GetCurrentProcess(), (DWORD64)pointer, &ModuleInfo);
+    jl_in_stackwalk = 0;
+    if (isvalid) {
+        char *fname = ModuleInfo.LoadedImageName;
+        DWORD64 fbase = ModuleInfo.BaseOfImage;
+        size_t msize = ModuleInfo.ImageSize;
+        *fromC = (fbase != jl_sysimage_base);
+        if (skipC && *fromC) {
+            return;
+        }
+        static char frame_info_func[
+            sizeof(SYMBOL_INFO) +
+            MAX_SYM_NAME * sizeof(TCHAR)];
+        static IMAGEHLP_LINE64 frame_info_line;
+        DWORD dwDisplacement = 0;
+        DWORD64 dwDisplacement64 = 0;
+        DWORD64 dwAddress = pointer;
+        PSYMBOL_INFO pSymbol = (PSYMBOL_INFO)frame_info_func;
+        pSymbol->SizeOfStruct = sizeof(SYMBOL_INFO);
+        pSymbol->MaxNameLen = MAX_SYM_NAME;
+        jl_in_stackwalk = 1;
+        if (SymFromAddr(GetCurrentProcess(), dwAddress, &dwDisplacement64, pSymbol)) {
+            // SymFromAddr returned success
+            *name = strdup(pSymbol->Name);
+        }
+        else {
+            // SymFromAddr failed
+            //fprintf(stderr,"SymFromAddr returned error : %lu\n", GetLastError());
+        }
+
+        frame_info_line.SizeOfStruct = sizeof(IMAGEHLP_LINE64);
+        if (SymGetLineFromAddr64(GetCurrentProcess(), dwAddress, &dwDisplacement, &frame_info_line)) {
+            // SymGetLineFromAddr64 returned success
+            // record source file name and line number
+            if (frame_info_line.FileName)
+                *filename = strdup(frame_info_line.FileName);
+            *line = frame_info_line.LineNumber;
+        } else if (*fromC) {
+            // No debug info, use dll name instead
+            *filename = fname;
+        } else {
+            *filename = "";
+        }
+        jl_in_stackwalk = 0;
+#else // ifdef _OS_WINDOWS_
     Dl_info dlinfo;
-    const char *fname = 0;
     if ((dladdr((void*)pointer, &dlinfo) != 0) && dlinfo.dli_fname) {
-        *fromC = !jl_is_sysimg(dlinfo.dli_fname);
+        const char *fname;
+        uint64_t fbase = (uint64_t)dlinfo.dli_fbase;
+        size_t msize = (size_t)(((uint64_t)-1)-fbase);
+        *fromC = (fbase != jl_sysimage_base);
         if (skipC && *fromC)
             return;
         // In case we fail with the debug info lookup, we at least still
         // have the function name, even if we don't have line numbers
         *name = dlinfo.dli_sname;
         *filename = dlinfo.dli_fname;
-        uint64_t fbase = (uint64_t)dlinfo.dli_fbase;
-#endif
-        obfiletype::iterator it = objfilemap.find(fbase);
-        llvm::object::ObjectFile *obj = NULL;
+        fname = dlinfo.dli_fname;
+#endif // ifdef _OS_WINDOWS_
         DIContext *context = NULL;
         int64_t slide = 0;
-#ifndef _OS_WINDOWS_
-        fname = dlinfo.dli_fname;
-#else
-        IMAGEHLP_MODULE64 ModuleInfo;
-        ModuleInfo.SizeOfStruct = sizeof(IMAGEHLP_MODULE64);
-        SymGetModuleInfo64(GetCurrentProcess(), (DWORD64)pointer, &ModuleInfo);
-        fname = ModuleInfo.LoadedImageName;
-        *fromC = !jl_is_sysimg(fname);
-        if (skipC && *fromC)
-            return;
-#endif
+#if !defined(_OS_WINDOWS_) || defined(LLVM35)
+        obfiletype::iterator it = objfilemap.find(fbase);
+        llvm::object::ObjectFile *obj = NULL;
         if (it == objfilemap.end()) {
-#ifdef _OS_DARWIN_
-            // First find the uuid of the object file (we'll use this to make sure we find the
-            // correct debug symbol file).
-            uint8_t uuid[16], uuid2[16];
-
+#if defined(_OS_DARWIN_) || defined(_OS_WINDOWS_)
+#if defined(_OS_WINDOWS_)
+#define origerrorobj errorobj
+#endif
+#ifdef LLVM36
+           std::unique_ptr<MemoryBuffer> membuf = MemoryBuffer::getMemBuffer(
+                    StringRef((const char *)fbase, msize)), "", false);
+           auto origerrorobj = llvm::object::ObjectFile::createObjectFile(
+                membuf->getMemBufferRef(), sys::fs::file_magic::unknown);
+#elif defined(LLVM35)
             MemoryBuffer *membuf = MemoryBuffer::getMemBuffer(
-                StringRef((const char *)fbase, (size_t)(((uint64_t)-1)-fbase)),"",false);
-
-#ifdef LLVM35
+                StringRef((const char *)fbase, msize), "", false);
             std::unique_ptr<MemoryBuffer> buf(membuf);
             auto origerrorobj = llvm::object::ObjectFile::createObjectFile(
                 buf, sys::fs::file_magic::unknown);
 #else
+            MemoryBuffer *membuf = MemoryBuffer::getMemBuffer(
+                StringRef((const char *)fbase, msize), "", false);
             llvm::object::ObjectFile *origerrorobj = llvm::object::ObjectFile::createObjectFile(
                 membuf);
 #endif
+#if defined(_OS_DARWIN_)
             if (!origerrorobj) {
                 objfileentry_t entry = {obj,context,slide};
                 objfilemap[fbase] = entry;
@@ -329,6 +406,9 @@ void jl_getDylibFunctionInfo(const char **name, size_t *line, const char **filen
 #else
             llvm::object::MachOObjectFile *morigobj = (llvm::object::MachOObjectFile *)origerrorobj;
 #endif
+            // First find the uuid of the object file (we'll use this to make sure we find the
+            // correct debug symbol file).
+            uint8_t uuid[16], uuid2[16];
             if (!getObjUUID(morigobj,uuid)) {
                 objfileentry_t entry = {obj,context,slide};
                 objfilemap[fbase] = entry;
@@ -340,31 +420,29 @@ void jl_getDylibFunctionInfo(const char **name, size_t *line, const char **filen
             // as the shared library. In the future we may use DBGCopyFullDSYMURLForUUID from CoreFoundation to make
             // use of spotlight to find the .dSYM file.
             char dsympath[PATH_MAX];
-            strlcpy(dsympath, dlinfo.dli_fname, sizeof(dsympath));
+            strlcpy(dsympath, fname, sizeof(dsympath));
             strlcat(dsympath, ".dSYM/Contents/Resources/DWARF/", sizeof(dsympath));
-            strlcat(dsympath, strrchr(dlinfo.dli_fname,'/')+1, sizeof(dsympath));
+            strlcat(dsympath, strrchr(fname,'/')+1, sizeof(dsympath));
 #ifdef LLVM35
             auto errorobj = llvm::object::ObjectFile::createObjectFile(dsympath);
 #else
             llvm::object::ObjectFile *errorobj = llvm::object::ObjectFile::createObjectFile(dsympath);
 #endif
-#else
-            // On non OS X systems we need to mmap another copy because of the permissions on the mmaped
-            // shared library.
+#endif // ifdef _OS_DARWIN_
+#else // ifdef  _OS_DARWIN_ || _OS_WINDOWS_
+            // On Linux systems we need to mmap another copy because of the permissions on the mmap'ed shared library.
 #ifdef LLVM35
             auto errorobj = llvm::object::ObjectFile::createObjectFile(fname);
 #else
             llvm::object::ObjectFile *errorobj = llvm::object::ObjectFile::createObjectFile(fname);
 #endif
-#endif
+#endif // ifdef _OS_DARWIN_
+            if (errorobj) {
 #ifdef LLVM36
-            if (errorobj) {
                 obj = errorobj.get().release();
-#elif LLVM35
-            if (errorobj) {
+#elif defined(LLVM35)
                 obj = errorobj.get();
 #else
-            if (errorobj != NULL) {
                 obj = errorobj;
 #endif
 #ifdef _OS_DARWIN_
@@ -380,24 +458,26 @@ void jl_getDylibFunctionInfo(const char **name, size_t *line, const char **filen
                 }
 #endif
 #ifdef _OS_WINDOWS_
+#ifdef LLVM35
                 assert(obj->isCOFF());
                 llvm::object::COFFObjectFile *coffobj = (llvm::object::COFFObjectFile *)obj;
                 const llvm::object::pe32plus_header *pe32plus;
                 coffobj->getPE32PlusHeader(pe32plus);
                 if (pe32plus != NULL) {
-                    slide = pe32plus->ImageBase-fbase;
+                    slide = pe32plus->ImageBase - fbase;
                 }
                 else {
                     const llvm::object::pe32_header *pe32;
-                    coffobj->getPE32Header(pe32); 
+                    coffobj->getPE32Header(pe32);
                     if (pe32 == NULL) {
                         obj = NULL;
                         context = NULL;
                     }
                     else {
-                        slide = pe32->ImageBase-fbase;
+                        slide = pe32->ImageBase - fbase;
                     }
                 }
+#endif
 #endif
 
             }
@@ -409,16 +489,16 @@ void jl_getDylibFunctionInfo(const char **name, size_t *line, const char **filen
             context = it->second.ctx;
             slide = it->second.slide;
         }
+#endif // ifdef _OS_WINDOWS && !LLVM35
 #ifdef _OS_DARWIN_
-    lookup:
+lookup:
 #endif
-        lookup_pointer(context, name, line, filename, pointer+slide, jl_is_sysimg(fname), fromC);
-        return;
+        lookup_pointer(context, name, line, filename, pointer+slide, fbase == jl_sysimage_base, fromC);
+    }
+    else {
+        *fromC = 1;
     }
-    *fromC = 1;
-    return;
 }
-#endif
 
 void jl_getFunctionInfo(const char **name, size_t *line, const char **filename, size_t pointer, int *fromC, int skipC)
 {
@@ -427,14 +507,12 @@ void jl_getFunctionInfo(const char **name, size_t *line, const char **filename,
     *filename = "no file";
     *fromC = 0;
 
-#if USE_MCJIT
+#ifdef USE_MCJIT
 // With MCJIT we can get function information directly from the ObjectFile
     std::map<size_t, ObjectInfo, revcomp> &objmap = jl_jit_events->getObjectMap();
     std::map<size_t, ObjectInfo, revcomp>::iterator it = objmap.lower_bound(pointer);
 
-    if (it == objmap.end())
-        return jl_getDylibFunctionInfo(name,line,filename,pointer,fromC,skipC);
-    if ((pointer - it->first) > it->second.size)
+    if (it == objmap.end() || (pointer - it->first) > it->second.size)
         return jl_getDylibFunctionInfo(name,line,filename,pointer,fromC,skipC);
 
 #ifdef LLVM36
@@ -500,21 +578,84 @@ void jl_getFunctionInfo(const char **name, size_t *line, const char **filename,
 #endif // USE_MCJIT
 }
 
+#if defined(_OS_WINDOWS_)
+#ifdef USE_MCJIT
+#if defined(_CPU_X86_64_)
+class RTDyldMemoryManagerWin : public RTDyldMemoryManager {
+public:
+  RTDyldMemoryManagerWin(RTDyldMemoryManager *MM)
+    : ClientMM(MM) {}
 
-#if defined(_OS_WINDOWS_) && defined(_CPU_X86_64_)
-extern "C" void *CALLBACK jl_getUnwindInfo(HANDLE hProcess, ULONG64 AddrBase, ULONG64 UserContext);
-#ifndef USE_MCJIT
+  // Functions deferred to client memory manager
+  uint8_t *allocateCodeSection(uintptr_t Size, unsigned Alignment,
+                               unsigned SectionID,
+                               StringRef SectionName) override {
+    uint8_t *mem = ClientMM->allocateCodeSection(Size+48, Alignment, SectionID, SectionName);
+    mem[Size] = 0;
+    return mem;
+  }
 
-void *CALLBACK jl_getUnwindInfo(HANDLE hProcess, ULONG64 AddrBase, ULONG64 UserContext)
+  uint8_t *allocateDataSection(uintptr_t Size, unsigned Alignment,
+                               unsigned SectionID, StringRef SectionName,
+                               bool IsReadOnly) override {
+    return ClientMM->allocateDataSection(Size, Alignment,
+                                         SectionID, SectionName, IsReadOnly);
+  }
+
+  void reserveAllocationSpace(uintptr_t CodeSize, uintptr_t DataSizeRO,
+                              uintptr_t DataSizeRW) override {
+    return ClientMM->reserveAllocationSpace(CodeSize+48, DataSizeRO, DataSizeRW);
+  }
+
+  bool needsToReserveAllocationSpace() override {
+    return ClientMM->needsToReserveAllocationSpace();
+  }
+
+  void registerEHFrames(uint8_t *Addr, uint64_t LoadAddr,
+                        size_t Size) override {
+    ClientMM->registerEHFrames(Addr, LoadAddr, Size);
+  }
+
+  void deregisterEHFrames(uint8_t *Addr, uint64_t LoadAddr,
+                          size_t Size) override {
+    ClientMM->deregisterEHFrames(Addr, LoadAddr, Size);
+  }
+
+  uint64_t getSymbolAddress(const std::string &Name) override {
+    return ClientMM->getSymbolAddress(Name);
+  }
+
+  void notifyObjectLoaded(ExecutionEngine *EE,
+                          const ObjectImage *Obj) override {
+    ClientMM->notifyObjectLoaded(EE, Obj);
+  }
+
+  void *getPointerToNamedFunction(const std::string &Name,
+                                  bool AbortOnFailure = true) override {
+      return ClientMM->getPointerToNamedFunction(Name,AbortOnFailure);
+  }
+
+  bool finalizeMemory(std::string *ErrMsg = nullptr) override {
+    return ClientMM->finalizeMemory(ErrMsg);
+  }
+
+private:
+  std::unique_ptr<RTDyldMemoryManager> ClientMM;
+};
+#else
+extern "C"
+DWORD64 jl_getUnwindInfo(ULONG64 dwAddr)
 {
-    std::map<size_t, FuncInfo, revcomp> &info = jl_jit_events->getMap();
-    std::map<size_t, FuncInfo, revcomp>::iterator it = info.lower_bound(AddrBase);
-    if (it != info.end() && (size_t)(*it).first + (*it).second.lengthAdr >= AddrBase) {
-        return (void*)(*it).second.fnentry;
+    std::map<size_t, ObjectInfo, revcomp> &objmap = jl_jit_events->getObjectMap();
+    std::map<size_t, ObjectInfo, revcomp>::iterator it = objmap.lower_bound(dwAddr);
+    if (it != objmap.end() && (intptr_t)(*it).first + (*it).second.size > dwAddr) {
+        return (DWORD64)(intptr_t)(*it).first;
     }
-    return NULL;
+    return 0;
 }
-
+#endif
+#else //ifdef USE_MCJIT
+#if defined(_CPU_X86_64_)
 // Custom memory manager for exception handling on Windows
 // we overallocate 48 bytes at the end of each function
 // for unwind information (see NotifyFunctionEmitted)
@@ -531,11 +672,21 @@ public:
   virtual void AllocateGOT() { JMM->AllocateGOT(); HasGOT = true; }
   virtual uint8_t *getGOTBase() const { return JMM->getGOTBase(); }
   virtual uint8_t *startFunctionBody(const Function *F,
-                                     uintptr_t &ActualSize) { ActualSize += 48; uint8_t *ret = JMM->startFunctionBody(F,ActualSize); ActualSize -= 48; return ret; }
+                                     uintptr_t &ActualSize) {
+      if (ActualSize == 0)
+          ActualSize += 64;
+      ActualSize += 48;
+      uint8_t *mem = JMM->startFunctionBody(F,ActualSize);
+      ActualSize -= 48;
+      return mem;
+  }
   virtual uint8_t *allocateStub(const GlobalValue* F, unsigned StubSize,
                                 unsigned Alignment)  { return JMM->allocateStub(F,StubSize,Alignment); }
   virtual void endFunctionBody(const Function *F, uint8_t *FunctionStart,
-                               uint8_t *FunctionEnd) { return JMM->endFunctionBody(F,FunctionStart,FunctionEnd+48); }
+                               uint8_t *FunctionEnd) {
+      FunctionEnd[0] = 0;
+      JMM->endFunctionBody(F,FunctionStart,FunctionEnd+48);
+  }
   virtual uint8_t *allocateSpace(intptr_t Size, unsigned Alignment) { return JMM->allocateSpace(Size,Alignment); }
   virtual uint8_t *allocateGlobal(uintptr_t Size, unsigned Alignment) { return JMM->allocateGlobal(Size,Alignment); }
   virtual void deallocateFunctionBody(void *Body) { return JMM->deallocateFunctionBody(Body); }
@@ -554,12 +705,22 @@ public:
 
 #ifdef LLVM35
   virtual uint8_t *allocateCodeSection(uintptr_t Size, unsigned Alignment,
-                                       unsigned SectionID, llvm::StringRef SectionName) { return JMM->allocateCodeSection(Size,Alignment,SectionID,SectionName); }
+                                       unsigned SectionID, llvm::StringRef SectionName) {
+    uint8_t *mem = JMM->allocateCodeSection(Size+48, Alignment, SectionID, SectionName);
+    mem[Size] = 0;
+    return mem;
+  }
   virtual uint8_t *allocateDataSection(uintptr_t Size, unsigned Alignment,
-                                       unsigned SectionID, llvm::StringRef SectionName, bool IsReadOnly) { return JMM->allocateDataSection(Size,Alignment,SectionID,SectionName,IsReadOnly); }
+                                       unsigned SectionID, llvm::StringRef SectionName, bool IsReadOnly) {
+    return JMM->allocateDataSection(Size,Alignment,SectionID,SectionName,IsReadOnly);
+  }
 #else
   virtual uint8_t *allocateCodeSection(uintptr_t Size, unsigned Alignment,
-                                       unsigned SectionID) { return JMM->allocateCodeSection(Size,Alignment,SectionID); }
+                                       unsigned SectionID) {
+    uint8_t *mem = JMM->allocateCodeSection(Size+48, Alignment, SectionID);
+    mem[Size] = 0;
+    return mem;
+  }
   virtual uint8_t *allocateDataSection(uintptr_t Size, unsigned Alignment,
                                        unsigned SectionID, bool IsReadOnly) { return JMM->allocateDataSection(Size,Alignment,SectionID,IsReadOnly); }
 #endif
@@ -568,14 +729,20 @@ public:
   virtual bool applyPermissions(std::string *ErrMsg = 0) { return JMM->applyPermissions(ErrMsg); }
   virtual void registerEHFrames(StringRef SectionData) { return JMM->registerEHFrames(SectionData); }
 };
-
-#else 
-void *CALLBACK jl_getUnwindInfo(HANDLE hProcess, ULONG64 AddrBase, ULONG64 UserContext)
+#else
+extern "C"
+DWORD64 jl_getUnwindInfo(ULONG64 dwAddr)
 {
-    return NULL;
+    std::map<size_t, FuncInfo, revcomp> &info = jl_jit_events->getMap();
+    std::map<size_t, FuncInfo, revcomp>::iterator it = info.lower_bound(dwAddr);
+    if (it != info.end() && (intptr_t)(*it).first + (*it).second.lengthAdr > dwAddr) {
+        return (DWORD64)(intptr_t)(*it).first;
+    }
+    return 0;
 }
 #endif
 #endif
+#endif
 
 // Code coverage
 
@@ -602,6 +769,8 @@ static void coverageVisitLine(std::string filename, int line)
                         v);
 }
 
+extern "C" int isabspath(const char *in);
+
 void write_log_data(logdata_t logData, const char *extension)
 {
     std::string base = std::string(julia_home);
@@ -611,21 +780,21 @@ void write_log_data(logdata_t logData, const char *extension)
         std::string filename = (*it).first;
         std::vector<GlobalVariable*> &values = (*it).second;
         if (values.size() > 1) {
-	    if (filename[0] != '/')
-		filename = base + filename;
+            if (!isabspath(filename.c_str()))
+                filename = base + filename;
             std::ifstream inf(filename.c_str());
             if (inf.is_open()) {
-		std::string outfile = filename + extension;
+                std::string outfile = filename + extension;
                 std::ofstream outf(outfile.c_str(), std::ofstream::trunc | std::ofstream::out);
                 char line[1024];
                 int l = 1;
                 while (!inf.eof()) {
                     inf.getline(line, sizeof(line));
-		    if (inf.fail() && !inf.bad()) {
-			// Read through lines longer than sizeof(line)
-			inf.clear();
-			inf.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
-		    }
+                    if (inf.fail() && !inf.bad()) {
+                        // Read through lines longer than sizeof(line)
+                        inf.clear();
+                        inf.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
+                    }
                     int value = -1;
                     if ((size_t)l < values.size()) {
                         GlobalVariable *gv = values[l];
@@ -662,7 +831,7 @@ static logdata_t mallocData;
 static void mallocVisitLine(std::string filename, int line)
 {
     if (filename == "" || filename == "none" || filename == "no file") {
-	sync_gc_total_bytes();
+        sync_gc_total_bytes();
         return;
     }
     logdata_t::iterator it = mallocData.find(filename);
@@ -674,7 +843,7 @@ static void mallocVisitLine(std::string filename, int line)
         vec.resize(line+1, NULL);
     if (vec[line] == NULL)
         vec[line] = new GlobalVariable(*jl_Module, T_int64, false,
-				       GlobalVariable::InternalLinkage,
+                                       GlobalVariable::InternalLinkage,
                                        ConstantInt::get(T_int64,0), "bytecnt");
     GlobalVariable *v = vec[line];
     builder.CreateStore(builder.CreateAdd(builder.CreateLoad(v, true),
@@ -689,13 +858,13 @@ extern "C" DLLEXPORT void jl_clear_malloc_data(void)
     logdata_t::iterator it = mallocData.begin();
     for (; it != mallocData.end(); it++) {
         std::vector<GlobalVariable*> &bytes = (*it).second;
-	std::vector<GlobalVariable*>::iterator itb;
-	for (itb = bytes.begin(); itb != bytes.end(); itb++) {
-	    if (*itb) {
-		int64_t *p = (int64_t*) jl_ExecutionEngine->getPointerToGlobal(*itb);
-		*p = 0;
-	    }
-	}
+        std::vector<GlobalVariable*>::iterator itb;
+        for (itb = bytes.begin(); itb != bytes.end(); itb++) {
+            if (*itb) {
+                int64_t *p = (int64_t*) jl_ExecutionEngine->getPointerToGlobal(*itb);
+                *p = 0;
+            }
+        }
     }
     sync_gc_total_bytes();
 }
