@@ -8,6 +8,7 @@
 #include <string.h>
 #include <setjmp.h>
 #include <assert.h>
+#include <fcntl.h>
 
 #if defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__)
 #include <sys/types.h>
@@ -80,6 +81,7 @@ DLLEXPORT void gdblookup(ptrint_t ip);
 
 char *julia_home = NULL;
 jl_compileropts_t jl_compileropts = { NULL, // build_path
+                                      NULL, // cpu_target ("native", "core2", etc...)
                                       0,    // code_coverage
                                       0,    // malloc_log
                                       JL_COMPILEROPT_CHECK_BOUNDS_DEFAULT,
@@ -207,6 +209,7 @@ volatile sig_atomic_t jl_signal_pending = 0;
 volatile sig_atomic_t jl_defer_signal = 0;
 
 #ifdef _OS_WINDOWS_
+BOOL (*pSetThreadStackGuarantee)(PULONG);
 void restore_signals(void)
 {
     SetConsoleCtrlHandler(NULL, 0); //turn on ctrl-c handler
@@ -215,18 +218,23 @@ void restore_signals(void)
 void jl_throw_in_ctx(jl_value_t *excpt, CONTEXT *ctxThread, int bt)
 {
     assert(excpt != NULL);
+#if defined(_CPU_X86_64_)
+    DWORD64 Rsp = (ctxThread->Rsp&(DWORD64)-16) - 8;
+#elif defined(_CPU_X86_)
+    DWORD32 Esp = (ctxThread->Esp&(DWORD32)-16) - 4;
+#else
+#error WIN16 not supported :P
+#endif
     bt_size = bt ? rec_backtrace_ctx(bt_data, MAX_BT_SIZE, ctxThread) : 0;
     jl_exception_in_transit = excpt;
 #if defined(_CPU_X86_64_)
+    *(DWORD64*)Rsp = 0;
+    ctxThread->Rsp = Rsp;
     ctxThread->Rip = (DWORD64)&jl_rethrow;
-    ctxThread->Rsp &= (DWORD64)-16;
-    ctxThread->Rsp -= 8; //fix up the stack pointer -- this seems to be correct by observation
 #elif defined(_CPU_X86_)
+    *(DWORD32*)Esp = 0;
+    ctxThread->Esp = Esp;
     ctxThread->Eip = (DWORD)&jl_rethrow;
-    ctxThread->Esp &= (DWORD)-16;
-    ctxThread->Esp -= 4; //fix up the stack pointer
-#else
-#error WIN16 not supported :P
 #endif
 }
 
@@ -286,8 +294,7 @@ static LONG WINAPI _exception_handler(struct _EXCEPTION_POINTERS *ExceptionInfo,
                 jl_throw_in_ctx(jl_diverror_exception, ExceptionInfo->ContextRecord,in_ctx);
                 return EXCEPTION_CONTINUE_EXECUTION;
             case EXCEPTION_STACK_OVERFLOW:
-                bt_size = 0;
-                jl_throw_in_ctx(jl_stackovf_exception, ExceptionInfo->ContextRecord,0);
+                jl_throw_in_ctx(jl_stackovf_exception, ExceptionInfo->ContextRecord,in_ctx&&pSetThreadStackGuarantee);
                 return EXCEPTION_CONTINUE_EXECUTION;
         }
         ios_puts("\nPlease submit a bug report with steps to reproduce this fault, and any error messages that follow (in their entirety). Thanks.\nException: ", ios_stderr);
@@ -339,6 +346,11 @@ static LONG WINAPI _exception_handler(struct _EXCEPTION_POINTERS *ExceptionInfo,
         gdblookup((ptrint_t)ExceptionInfo->ExceptionRecord->ExceptionAddress);
         bt_size = rec_backtrace_ctx(bt_data, MAX_BT_SIZE, ExceptionInfo->ContextRecord);
         jlbacktrace();
+        static int recursion = 0;
+        if (recursion++)
+            exit(1);
+        else
+            jl_exit(1);
     }
     return EXCEPTION_CONTINUE_SEARCH;
 }
@@ -354,7 +366,7 @@ EXCEPTION_DISPOSITION _seh_exception_handler(PEXCEPTION_RECORD ExceptionRecord,
     EXCEPTION_POINTERS ExceptionInfo;
     ExceptionInfo.ExceptionRecord = ExceptionRecord;
     ExceptionInfo.ContextRecord = ContextRecord;
-    
+
     EXCEPTION_DISPOSITION rval;
     switch (_exception_handler(&ExceptionInfo,1)) {
         case EXCEPTION_CONTINUE_EXECUTION:
@@ -368,10 +380,8 @@ EXCEPTION_DISPOSITION _seh_exception_handler(PEXCEPTION_RECORD ExceptionRecord,
     }
 
     return rval;
-} 
-void* CALLBACK jl_getUnwindInfo(HANDLE hProcess, ULONG64 AddrBase, ULONG64 UserContext);
+}
 #endif
-
 #else // #ifdef _OS_WINDOWS_
 
 void restore_signals(void)
@@ -449,8 +459,10 @@ DLLEXPORT void uv_atexit_hook()
     uv_walk(loop, jl_uv_exitcleanup_walk, &queue);
     // close stdout and stderr last, since we like being
     // able to show stuff (incl. printf's)
-    jl_uv_exitcleanup_add((uv_handle_t*)jl_uv_stdout, &queue);
-    jl_uv_exitcleanup_add((uv_handle_t*)jl_uv_stderr, &queue);
+    if (((uv_handle_t*)jl_uv_stdout)->type < UV_HANDLE_TYPE_MAX)
+        jl_uv_exitcleanup_add((uv_handle_t*)jl_uv_stdout, &queue);
+    if (((uv_handle_t*)jl_uv_stderr)->type < UV_HANDLE_TYPE_MAX)
+        jl_uv_exitcleanup_add((uv_handle_t*)jl_uv_stderr, &queue);
     //uv_unref((uv_handle_t*)jl_uv_stdout);
     //uv_unref((uv_handle_t*)jl_uv_stderr);
     struct uv_shutdown_queue_item *item = queue.first;
@@ -506,7 +518,7 @@ DLLEXPORT void uv_atexit_hook()
 
 void jl_get_builtin_hooks(void);
 
-uv_lib_t *jl_dl_handle;
+DLLEXPORT uv_lib_t *jl_dl_handle;
 uv_lib_t _jl_RTLD_DEFAULT_handle;
 uv_lib_t *jl_RTLD_DEFAULT_handle=&_jl_RTLD_DEFAULT_handle;
 #ifdef _OS_WINDOWS_
@@ -516,12 +528,13 @@ uv_lib_t _jl_kernel32_handle;
 uv_lib_t _jl_crtdll_handle;
 uv_lib_t _jl_winsock_handle;
 
+DLLEXPORT uv_lib_t *jl_exe_handle=&_jl_exe_handle;
 uv_lib_t *jl_ntdll_handle=&_jl_ntdll_handle;
-uv_lib_t *jl_exe_handle=&_jl_exe_handle;
 uv_lib_t *jl_kernel32_handle=&_jl_kernel32_handle;
 uv_lib_t *jl_crtdll_handle=&_jl_crtdll_handle;
 uv_lib_t *jl_winsock_handle=&_jl_winsock_handle;
 #endif
+
 uv_loop_t *jl_io_loop;
 
 void *init_stdio_handle(uv_file fd,int readable)
@@ -529,7 +542,7 @@ void *init_stdio_handle(uv_file fd,int readable)
     void *handle;
     uv_handle_type type = uv_guess_handle(fd);
     jl_uv_file_t *file;
-#ifndef _OS_WINDOWS_    
+#ifndef _OS_WINDOWS_
     // Duplicate the file descriptor so we can later dup it over if we want to redirect
     // STDIO without having to worry about closing the associated libuv object.
     // On windows however, libuv objects remember streams by their HANDLE, so this is
@@ -547,7 +560,15 @@ void *init_stdio_handle(uv_file fd,int readable)
             ((uv_tty_t*)handle)->data=0;
             uv_tty_set_mode((uv_tty_t*)handle,0); //cooked stdio
             break;
-        case UV_FILE: 
+        case UV_UNKNOWN_HANDLE:
+            // dup the descriptor with a new one pointing at the bit bucket ...
+#if defined(_OS_WINDOWS_)
+            _dup2(_open("NUL", O_RDWR | O_BINARY, _S_IREAD | _S_IWRITE), fd);
+#else
+            dup2(open("/dev/null", O_RDWR, S_IRUSR | S_IWUSR /* 0600 */ | S_IRGRP | S_IROTH /* 0644 */), fd);
+#endif
+            // ...and continue on as in the UV_FILE case
+        case UV_FILE:
             file = (jl_uv_file_t*)malloc(sizeof(jl_uv_file_t));
             file->loop = jl_io_loop;
             file->type = UV_FILE;
@@ -600,7 +621,9 @@ char jl_using_intel_jitevents; // Non-zero if running under Intel VTune Amplifie
 #endif
 
 #if defined(JL_USE_INTEL_JITEVENTS) && defined(__linux__)
-unsigned sig_stack_size = SIGSTKSZ; 
+unsigned sig_stack_size = SIGSTKSZ;
+#elif defined(_OS_WINDOWS_)
+#define sig_stack_size 131072 // 128k
 #else
 #define sig_stack_size SIGSTKSZ
 #endif
@@ -723,12 +746,41 @@ kern_return_t catch_exception_raise(mach_port_t            exception_port,
 
 #endif
 
+int isabspath(const char *in)
+{
+#ifdef _OS_WINDOWS_
+    char c0 = in[0];
+    if (c0 == '/' || c0 == '\\') {
+        return 1; // absolute path relative to %CD% (current drive), or UNC
+    }
+    else {
+        int s = strlen(in);
+        if (s > 2) {
+            char c1 = in[1];
+            char c2 = in[2];
+            if (c1 == ':' && (c2 == '/' || c2 == '\\')) return 1; // absolute path
+        }
+    }
+#else
+    if (in[0] == '/') return 1; // absolute path
+#endif
+    return 0; // relative path
+}
+
 void julia_init(char *imageFile)
 {
     jl_io_loop = uv_default_loop(); // this loop will internal events (spawning process etc.),
                                     // best to call this first, since it also initializes libuv
+    restore_signals();
+    // If we are able to load the sysimg and get a cpu_target, use that unless user has overridden
+    if (jl_compileropts.cpu_target == NULL) {
+        const char * sysimg_cpu_target = jl_get_system_image_cpu_target(imageFile);
+
+        // If we can't load anything from the sysimg, default to native
+        jl_compileropts.cpu_target = sysimg_cpu_target ? sysimg_cpu_target : "native";
+    }
     jl_page_size = jl_getpagesize();
-    jl_arr_xtralloc_limit = uv_get_total_memory() / 100;  // Extra allocation limited to 1% of total RAM 
+    jl_arr_xtralloc_limit = uv_get_total_memory() / 100;  // Extra allocation limited to 1% of total RAM
     jl_find_stack_bottom();
     jl_dl_handle = jl_load_dynamic_library(NULL, JL_RTLD_DEFAULT);
 #ifdef RTLD_DEFAULT
@@ -751,14 +803,14 @@ void julia_init(char *imageFile)
     if (!SymInitialize(GetCurrentProcess(), NULL, 1)) {
         JL_PRINTF(JL_STDERR, "WARNING: failed to initalize stack walk info\n");
     }
-#if defined(_CPU_X86_64_)
-    if (!SymRegisterFunctionEntryCallback64(GetCurrentProcess(), jl_getUnwindInfo, 0)) JL_PRINTF(JL_STDERR, "WARNING: failed to install backtrace info callback\n");
-#endif
     needsSymRefreshModuleList = 0;
     uv_lib_t jl_dbghelp;
     uv_dlopen("dbghelp.dll",&jl_dbghelp);
     if (uv_dlsym(&jl_dbghelp, "SymRefreshModuleList", (void**)&hSymRefreshModuleList))
         hSymRefreshModuleList = 0;
+    ULONG StackSizeInBytes = sig_stack_size;
+    if (uv_dlsym(jl_kernel32_handle, "SetThreadStackGuarantee", (void**)&pSetThreadStackGuarantee) || !pSetThreadStackGuarantee(&StackSizeInBytes))
+        pSetThreadStackGuarantee = NULL;
 #endif
     init_stdio();
 
@@ -768,8 +820,8 @@ void julia_init(char *imageFile)
         jl_using_intel_jitevents = 1;
 #if defined(__linux__)
         // Intel VTune Amplifier needs at least 64k for alternate stack.
-        if (SIGSTKSZ < 1<<16) 
-            sig_stack_size = 1<<16; 
+        if (SIGSTKSZ < 1<<16)
+            sig_stack_size = 1<<16;
 #endif
     }
 #endif
@@ -800,14 +852,15 @@ void julia_init(char *imageFile)
 
     if (!imageFile) {
         jl_core_module = jl_new_module(jl_symbol("Core"));
+        jl_init_intrinsic_functions();
+        jl_init_primitives();
+
         jl_new_main_module();
         jl_internal_main_module = jl_main_module;
 
         jl_current_module = jl_core_module;
         jl_root_task->current_module = jl_current_module;
 
-        jl_init_intrinsic_functions();
-        jl_init_primitives();
         jl_load("boot.jl");
         jl_get_builtin_hooks();
         jl_boot_file_loaded = 1;
@@ -884,13 +937,13 @@ void julia_init(char *imageFile)
     pthread_attr_t attr;
     if (pthread_attr_init(&attr) != 0) {
         JL_PRINTF(JL_STDERR, "pthread_attr_init failed");
-        jl_exit(1);  
+        jl_exit(1);
     }
     pthread_attr_setdetachstate(&attr,PTHREAD_CREATE_DETACHED);
     if (pthread_create(&thread,&attr,mach_segv_listener,NULL) != 0) {
         JL_PRINTF(JL_STDERR, "pthread_create failed");
-        jl_exit(1);  
-    }     
+        jl_exit(1);
+    }
     pthread_attr_destroy(&attr);
 
     ret = thread_set_exception_ports(mach_thread_self(),EXC_MASK_BAD_ACCESS,segv_port,EXCEPTION_DEFAULT,MACHINE_THREAD_STATE);
@@ -948,10 +1001,6 @@ void julia_init(char *imageFile)
         JL_PRINTF(JL_STDERR, "sigaction: %s\n", strerror(errno));
         jl_exit(1);
     }
-    if (sigaction(SIGPIPE, &act_die, NULL) < 0) {
-        JL_PRINTF(JL_STDERR, "sigaction: %s\n", strerror(errno));
-        jl_exit(1);
-    }
 #else // defined(_OS_WINDOWS_)
     if (signal(SIGFPE, (void (__cdecl *)(int))fpe_handler) == SIG_ERR) {
         JL_PRINTF(JL_STDERR, "Couldn't set SIGFPE\n");
