@@ -257,7 +257,7 @@ static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl)
             }
             gf = eval((jl_value_t*)fname, locals, nl);
             assert(jl_is_function(gf));
-            assert(jl_is_gf(gf));
+            if (!jl_is_gf(gf)) jl_error("cannot redefine non-generic function");
             if (jl_is_expr(fname))
                 fname = (jl_sym_t*)jl_fieldref(jl_exprarg(fname, 2), 0);
             if (!kw)
@@ -423,6 +423,7 @@ static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl)
         jl_sym_t *nm = (jl_sym_t*)args[0];
         assert(jl_is_symbol(nm));
         jl_function_t *f = (jl_function_t*)eval(args[1], locals, nl);
+        JL_GC_PUSH1(&f);
         assert(jl_is_function(f));
         if (jl_boot_file_loaded &&
             f->linfo && f->linfo->ast && jl_is_expr(f->linfo->ast)) {
@@ -431,6 +432,7 @@ static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl)
             li->name = nm;
         }
         jl_set_global(jl_current_module, nm, (jl_value_t*)f);
+        JL_GC_POP();
         return (jl_value_t*)jl_nothing;
     }
     else if (ex->head == line_sym) {
