@@ -809,6 +809,7 @@ STATIC_INLINE jl_function_t *jl_get_function(jl_module_t *m, const char *name)
 }
 DLLEXPORT int jl_module_has_initializer(jl_module_t *m);
 DLLEXPORT void jl_module_run_initializer(jl_module_t *m);
+int jl_is_submodule(jl_module_t *child, jl_module_t *parent);
 
 // eq hash tables
 DLLEXPORT jl_array_t *jl_eqtable_put(jl_array_t *h, void *key, void *val);
@@ -867,6 +868,7 @@ DLLEXPORT void jl_init_with_image(char *julia_home_dir, char *image_relative_pat
 DLLEXPORT int jl_is_initialized(void);
 DLLEXPORT extern char *julia_home;
 
+DLLEXPORT const char * jl_get_system_image_cpu_target(const char *fname);
 DLLEXPORT void jl_save_system_image(char *fname);
 DLLEXPORT void jl_restore_system_image(char *fname);
 void jl_init_restored_modules();
@@ -992,11 +994,12 @@ DLLEXPORT void jl_handle_stack_switch();
 
 #ifdef COPY_STACKS
 // initialize base context of root task
+extern DLLEXPORT jl_jmp_buf jl_base_ctx;
 #define JL_SET_STACK_BASE                               \
     {                                                   \
         int __stk;                                      \
         jl_root_task->stackbase = (char*)&__stk;        \
-        if (jl_setjmp(jl_root_task->base_ctx, 1)) {     \
+        if (jl_setjmp(jl_base_ctx, 1)) {                \
             jl_handle_stack_switch();                   \
         }                                               \
     }
@@ -1145,7 +1148,6 @@ typedef struct _jl_task_t {
         void *stackbase;
         void *stack;
     };
-    jl_jmp_buf base_ctx;
     size_t bufsz;
     void *stkbuf;
     size_t ssize;
@@ -1242,7 +1244,7 @@ DLLEXPORT int jl_spawn(char *name, char **argv, uv_loop_t *loop,
                        uv_process_t *proc, jl_value_t *julia_struct,
                        uv_handle_type stdin_type,uv_pipe_t *stdin_pipe,
                        uv_handle_type stdout_type,uv_pipe_t *stdout_pipe,
-                       uv_handle_type stderr_type,uv_pipe_t *stderr_pipe, 
+                       uv_handle_type stderr_type,uv_pipe_t *stderr_pipe,
                        int detach, char **env, char *cwd);
 DLLEXPORT void jl_run_event_loop(uv_loop_t *loop);
 DLLEXPORT int jl_run_once(uv_loop_t *loop);
@@ -1322,6 +1324,7 @@ void show_execution_point(char *filename, int lno);
 
 typedef struct {
     char *build_path;
+    const char *cpu_target;
     int8_t code_coverage;
     int8_t malloc_log;
     int8_t check_bounds;
