@@ -73,9 +73,12 @@ static int _enonfatal(int err)
 
 #define SLEEP_TIME 5//ms
 
-#ifdef __APPLE__
+#if defined(__APPLE__)
 #define MAXSIZE ((1l << 31) - 1)   // OSX cannot handle blocks larger than this
 #define LIMIT_IO_SIZE(n) ((n) < MAXSIZE ? (n) : MAXSIZE)
+#elif defined(_OS_WINDOWS_)
+#define MAXSIZE (0x7fffffff)       // Windows read() takes a uint
+#define LIMIT_IO_SIZE(n) ((n) < (size_t)MAXSIZE ? (unsigned int)(n) : MAXSIZE)
 #else
 #define LIMIT_IO_SIZE(n) (n)
 #endif
@@ -246,7 +249,7 @@ static size_t _ios_read(ios_t *s, char *dest, size_t n, int all)
 
     while (n > 0) {
         avail = s->size - s->bpos;
-        
+
         if (avail > 0) {
             size_t ncopy = (avail >= n) ? n : avail;
             memcpy(dest, s->buf + s->bpos, ncopy);
@@ -261,14 +264,14 @@ static size_t _ios_read(ios_t *s, char *dest, size_t n, int all)
                 s->_eof = 1;
             return avail;
         }
-        
+
         dest += avail;
         n -= avail;
         tot += avail;
-        
+
         ios_flush(s);
         s->bpos = s->size = 0;
-        
+
         s->fpos = -1;
         if (n > MOST_OF(s->maxsize)) {
             // doesn't fit comfortably in buffer; go direct
