@@ -23,7 +23,7 @@
 #include "timefuncs.h"
 
 #if defined(_OS_WINDOWS_)
-double floattime(void)
+static double floattime(void)
 {
     struct timeb tstruct;
 
@@ -31,34 +31,12 @@ double floattime(void)
     return (double)tstruct.time + (double)tstruct.millitm/1.0e3;
 }
 #else
-double tv2float(struct timeval *tv)
+static double tv2float(struct timeval *tv)
 {
     return (double)tv->tv_sec + (double)tv->tv_usec/1.0e6;
 }
-
-double diff_time(struct timeval *tv1, struct timeval *tv2)
-{
-    return tv2float(tv1) - tv2float(tv2);
-}
 #endif
 
-// return as many bits of system randomness as we can get our hands on
-u_int64_t i64time(void)
-{
-    u_int64_t a;
-#if defined(_OS_WINDOWS_)
-    struct timeb tstruct;
-    ftime(&tstruct);
-    a = (((u_int64_t)tstruct.time)<<32) + (u_int64_t)tstruct.millitm;
-#else
-    struct timeval now;
-    gettimeofday(&now, NULL);
-    a = (((u_int64_t)now.tv_sec)<<32) + (u_int64_t)now.tv_usec;
-#endif
-
-    return a;
-}
-
 double clock_now(void)
 {
 #if defined(_OS_WINDOWS_)
@@ -87,4 +65,3 @@ void sleep_ms(int ms)
     select(0, NULL, NULL, NULL, &timeout);
 #endif
 }
-
