@@ -30,6 +30,8 @@ volatile int jl_in_stackwalk = 0;
 #include <dlfcn.h>   // for dladdr
 #endif
 
+DLLEXPORT jl_jmp_buf jl_base_ctx;
+
 /* This probing code is derived from Douglas Jones' user thread library */
 
 /* true if stack grows up, false if down */
@@ -148,10 +150,10 @@ extern size_t jl_page_size;
 jl_datatype_t *jl_task_type;
 DLLEXPORT jl_task_t * volatile jl_current_task;
 jl_task_t *jl_root_task;
-jl_value_t * volatile jl_task_arg_in_transit;
-jl_value_t *jl_exception_in_transit;
+DLLEXPORT jl_value_t * volatile jl_task_arg_in_transit;
+DLLEXPORT jl_value_t *jl_exception_in_transit;
 #ifdef JL_GC_MARKSWEEP
-jl_gcframe_t *jl_pgcstack = NULL;
+DLLEXPORT jl_gcframe_t *jl_pgcstack = NULL;
 #endif
 
 static void start_task(jl_task_t *t);
@@ -159,11 +161,16 @@ static void start_task(jl_task_t *t);
 #ifdef COPY_STACKS
 jl_jmp_buf * volatile jl_jmp_target;
 
-static void save_stack(jl_task_t *t)
+#if defined(_OS_WINDOWS_) && !defined(_COMPILER_MINGW_)
+static void __declspec(noinline)
+#else
+static void __attribute__((noinline))
+#endif
+save_stack(jl_task_t *t)
 {
     if (t->state == done_sym || t->state == failed_sym)
         return;
-    volatile int _x;
+    volatile char *_x;
     size_t nb = (char*)t->stackbase - (char*)&_x;
     char *buf;
     if (t->stkbuf == NULL || t->bufsz < nb) {
@@ -189,14 +196,12 @@ void __attribute__((noinline)) restore_stack(jl_task_t *t, jl_jmp_buf *where, ch
         p = _x;
         if ((char*)&_x > _x) {
             p = (char*)alloca((char*)&_x - _x);
-    	}
+        }
         restore_stack(t, where, p);
     }
     jl_jmp_target = where;
-
-    if (t->stkbuf != NULL) {
-        memcpy(_x, t->stkbuf, t->ssize);
-    }
+    assert(t->stkbuf != NULL);
+    memcpy(_x, t->stkbuf, t->ssize);
     jl_longjmp(*jl_jmp_target, 1);
 }
 
@@ -211,11 +216,6 @@ static void switch_stack(jl_task_t *t, jl_jmp_buf *where)
         restore_stack(t, where, NULL);
     }
 }
-
-void jl_switch_stack(jl_task_t *t, jl_jmp_buf *where)
-{
-    switch_stack(t, where);
-}
 #endif
 
 static void ctx_switch(jl_task_t *t, jl_jmp_buf *where)
@@ -262,7 +262,7 @@ static void ctx_switch(jl_task_t *t, jl_jmp_buf *where)
 
 #ifdef COPY_STACKS
         jl_jmp_target = where;
-        jl_longjmp(lastt->base_ctx, 1);
+        jl_longjmp(jl_base_ctx, 1);
 #else
         jl_longjmp(*where, 1);
 #endif
@@ -418,10 +418,6 @@ static void start_task(jl_task_t *t)
     local_sp += sizeof(jl_gcframe_t);
     local_sp += 12*sizeof(void*);
     t->stackbase = (void*)(local_sp + _frame_offset);
-    if (jl_setjmp(t->base_ctx, 0)) {
-        // we get here to remove our data from the process stack
-        switch_stack(jl_current_task, jl_jmp_target);
-    }
 #endif
     res = jl_apply(t->start, NULL, 0);
     JL_GC_POP();
@@ -431,7 +427,7 @@ static void start_task(jl_task_t *t)
 
 DLLEXPORT void jl_handle_stack_switch()
 {
-    jl_switch_stack(jl_current_task, jl_jmp_target);
+    switch_stack(jl_current_task, jl_jmp_target);
 }
 
 #ifndef COPY_STACKS
@@ -472,70 +468,59 @@ static int frame_info_from_ip(
     int skipC)
 {
     static const char *name_unknown = "???";
-#if defined(_OS_WINDOWS_)
-    static char frame_info_func[
-        sizeof(SYMBOL_INFO) +
-        MAX_SYM_NAME * sizeof(TCHAR)];
-    static IMAGEHLP_LINE64 frame_info_line;
-#endif
     int fromC = 0;
 
     jl_getFunctionInfo(func_name, line_num, file_name, ip, &fromC, skipC);
-    if (*func_name == NULL) {
-#if defined(_OS_WINDOWS_)
-        fromC = 1;
-        if (jl_in_stackwalk) {
-            *func_name = name_unknown;
-            *file_name = name_unknown;
-            *line_num = ip;
-        }
-        else {
-            jl_in_stackwalk = 1;
-            DWORD64 dwDisplacement64 = 0;
-            DWORD64 dwAddress = ip;
-
-            PSYMBOL_INFO pSymbol = (PSYMBOL_INFO)frame_info_func;
-            pSymbol->SizeOfStruct = sizeof(SYMBOL_INFO);
-            pSymbol->MaxNameLen = MAX_SYM_NAME;
-
-            if (SymFromAddr(GetCurrentProcess(), dwAddress, &dwDisplacement64, pSymbol)) {
-                // SymFromAddr returned success
-                *func_name = pSymbol->Name;
-            }
-            else {
-                *func_name = name_unknown;
-                // SymFromAddr failed
-                //DWORD error = GetLastError();
-                //printf("SymFromAddr returned error : %d\n", error);
-            }
-
-            DWORD dwDisplacement = 0;
-            frame_info_line.SizeOfStruct = sizeof(IMAGEHLP_LINE64);
-
-            if (SymGetLineFromAddr64(GetCurrentProcess(), dwAddress, &dwDisplacement, &frame_info_line)) {
-                // SymGetLineFromAddr64 returned success
-                *file_name = frame_info_line.FileName;
-                *line_num = frame_info_line.LineNumber;
-            }
-            else {
-                *file_name = name_unknown;
-                *line_num = ip;
-                // SymGetLineFromAddr64 failed
-                //DWORD error = GetLastError();
-                //printf("SymGetLineFromAddr64 returned error : %d\n", error);
-            }
-            jl_in_stackwalk = 0;
-        }
-#else
+    if (!*func_name) {
         *func_name = name_unknown;
         *file_name = name_unknown;
         *line_num = ip;
-#endif
     }
     return fromC;
 }
 
 #if defined(_OS_WINDOWS_)
+#ifdef _CPU_X86_64_
+static UNWIND_HISTORY_TABLE HistoryTable;
+#else
+static struct {
+    DWORD64 dwAddr;
+    DWORD64 ImageBase;
+} HistoryTable;
+#endif
+static PVOID CALLBACK JuliaFunctionTableAccess64(
+        _In_  HANDLE hProcess,
+        _In_  DWORD64 AddrBase)
+{
+    //printf("lookup %d\n", AddrBase);
+#ifdef _CPU_X86_64_
+    DWORD64 ImageBase;
+    PRUNTIME_FUNCTION fn = RtlLookupFunctionEntry(AddrBase, &ImageBase, &HistoryTable);
+    if (fn) return fn;
+#endif
+    return SymFunctionTableAccess64(hProcess, AddrBase);
+}
+static DWORD64 WINAPI JuliaGetModuleBase64(
+        _In_  HANDLE hProcess,
+        _In_  DWORD64 dwAddr)
+{
+    //printf("lookup base %d\n", dwAddr);
+#ifdef _CPU_X86_64_
+    DWORD64 ImageBase;
+    PRUNTIME_FUNCTION fn = RtlLookupFunctionEntry(dwAddr, &ImageBase, &HistoryTable);
+    if (fn) return ImageBase;
+#else
+    if (dwAddr == HistoryTable.dwAddr) return HistoryTable.ImageBase;
+    DWORD64 ImageBase = jl_getUnwindInfo(dwAddr);
+    if (ImageBase) {
+        HistoryTable.dwAddr = dwAddr;
+        HistoryTable.ImageBase = ImageBase;
+        return ImageBase;
+    }
+#endif
+    return SymGetModuleBase64(hProcess, dwAddr);
+}
+
 int needsSymRefreshModuleList;
 BOOL (WINAPI *hSymRefreshModuleList)(HANDLE);
 DLLEXPORT size_t rec_backtrace(ptrint_t *data, size_t maxsize)
@@ -561,7 +546,7 @@ DLLEXPORT size_t rec_backtrace_ctx(ptrint_t *data, size_t maxsize, CONTEXT *Cont
         jl_in_stackwalk = 0;
         needsSymRefreshModuleList = 0;
     }
-#if defined(_CPU_X86_64_) 
+#if defined(_CPU_X86_64_)
     DWORD MachineType = IMAGE_FILE_MACHINE_AMD64;
     stk.AddrPC.Offset = Context->Rip;
     stk.AddrStack.Offset = Context->Rsp;
@@ -577,22 +562,17 @@ DLLEXPORT size_t rec_backtrace_ctx(ptrint_t *data, size_t maxsize, CONTEXT *Cont
     stk.AddrPC.Mode = AddrModeFlat;
     stk.AddrStack.Mode = AddrModeFlat;
     stk.AddrFrame.Mode = AddrModeFlat;
-    
+
     size_t n = 0;
-    intptr_t lastsp = stk.AddrStack.Offset;
+    jl_in_stackwalk = 1;
     while (n < maxsize) {
-        jl_in_stackwalk = 1;
         BOOL result = StackWalk64(MachineType, GetCurrentProcess(), hMainThread,
-            &stk, Context, NULL, SymFunctionTableAccess64, SymGetModuleBase64, NULL);
-        jl_in_stackwalk = 0;
+            &stk, Context, NULL, JuliaFunctionTableAccess64, JuliaGetModuleBase64, NULL);
         data[n++] = (intptr_t)stk.AddrPC.Offset;
-        intptr_t sp = (intptr_t)stk.AddrStack.Offset;
-        if (!result || sp == 0 || 
-            (_stack_grows_up ? sp < lastsp : sp > lastsp) ||
-            stk.AddrReturn.Offset == 0)
+        if (!result)
             break;
-        lastsp = sp;
     }
+    jl_in_stackwalk = 0;
     return n;
 }
 #else
@@ -608,7 +588,7 @@ DLLEXPORT size_t rec_backtrace_ctx(ptrint_t *data, size_t maxsize, unw_context_t
     unw_cursor_t cursor;
     unw_word_t ip;
     size_t n=0;
-    
+
     unw_init_local(&cursor, uc);
     do {
         if (n >= maxsize)
@@ -626,7 +606,7 @@ size_t rec_backtrace_ctx_dwarf(ptrint_t *data, size_t maxsize, unw_context_t *uc
     unw_cursor_t cursor;
     unw_word_t ip;
     size_t n=0;
-    
+
     unw_init_local_dwarf(&cursor, uc);
     do {
         if (n >= maxsize)
@@ -734,7 +714,7 @@ void NORETURN throw_internal(jl_value_t *e)
             JL_PRINTF(JL_STDERR, "fatal: error thrown and no exception handler available.\n");
             jl_static_show(JL_STDERR, e);
             JL_PRINTF(JL_STDERR, "\n");
-            exit(1);
+            jl_exit(1);
         }
         jl_current_task->exception = e;
         finish_task(jl_current_task, e);
