@@ -224,6 +224,9 @@ v = pop!(l)
 v = pop!(l)
 @test v == 3
 @test length(l)==2
+m = Any[]
+@test_throws ErrorException pop!(m)
+@test_throws ErrorException shift!(m)
 unshift!(l,4,7,5)
 @test l[1]==4 && l[2]==7 && l[3]==5 && l[4]==1 && l[5]==2
 v = shift!(l)
@@ -708,6 +711,9 @@ a = [1,3,5]
 b = [3,1,2]
 a[b] = a
 @test a == [3,5,1]
+a = [3,2,1]
+a[a] = [4,5,6]
+@test a == [6,5,4]
 
 # lexicographic comparison
 @test lexcmp([1.0], [1]) == 0
@@ -779,8 +785,11 @@ for idx in {1, 2, 5, 9, 10, 1:0, 2:1, 1:1, 2:2, 1:2, 2:4, 9:8, 10:9, 9:9, 10:10,
     @test deleteat!(a, idx) == [acopy[1:(first(idx)-1)], acopy[(last(idx)+1):end]]
 end
 a = [1:10]
+@test deleteat!(a, 11:10) == [1:10;]
 @test deleteat!(a, [1,3,5,7:10...]) == [2,4,6]
-
+@test_throws BoundsError deleteat!(a, 13)
+@test_throws BoundsError deleteat!(a, [1,13])
+@test_throws ErrorException deleteat!(a, [5,3])
 
 # comprehensions
 X = [ i+2j for i=1:5, j=1:5 ]
@@ -894,3 +903,30 @@ function i7197()
     ind2sub(size(S), 5)
 end
 @test i7197() == (2,2)
+
+# commit b718cbc72e90, getindex(::Number, ::Real)
+b718cbc = 5
+@test b718cbc[1.0] == 5
+@test_throws InexactError b718cbc[1.1]
+
+#rotates
+
+a = [ [ 1 0 0 ], [ 0 0 0 ] ]
+@test rotr90(a,1) == [ [ 0 1 ], [ 0 0 ], [ 0 0 ] ]
+@test rotr90(a,2) == rot180(a,1)
+@test rotr90(a,3) == rotl90(a,1)
+@test rotl90(a,3) == rotr90(a,1)
+@test rotl90(a,1) == rotr90(a,3)
+@test rotl90(a,4) == a
+@test rotr90(a,4) == a
+@test rot180(a,2) == a
+
+# issue #9648
+let x = fill(1.5f0, 10^7)
+    @test abs(1.5f7 - cumsum(x)[end]) < 3*eps(1.5f7)
+end
+
+# PR #11080
+let x = fill(0.9, 1000)
+    @test_approx_eq prod(x) cumprod(x)[end]
+end
