@@ -175,7 +175,7 @@ end
 
 # multidimensional
 
-rand_m1m2() = rand(1:n1), rand(1:n2) 
+rand_m1m2() = rand(1:n1), rand(1:n2)
 
 b1 = randbool(n1, n2)
 
@@ -845,6 +845,37 @@ b1 = randbool(n1, n2)
 
 timesofar("nnz&find")
 
+## Findnext (backported from #9735) ##
+B = falses(1000)
+B[77] = true
+B[777] = true
+B′ = ~B
+@test_throws BoundsError findnext(B, -1)
+@test_throws BoundsError Base.findnextnot(B′, -1)
+@test_throws BoundsError findnext(!, B′, -1)
+@test_throws BoundsError findnext(identity, B, -1)
+@test_throws BoundsError findnext(x->false, B, -1)
+@test_throws BoundsError findnext(x->true, B, -1)
+@test findnext(B, 1)    == Base.findnextnot(B′, 1)    == findnext(!, B′, 1)    == 77
+@test findnext(B, 77)   == Base.findnextnot(B′, 77)   == findnext(!, B′, 77)   == 77
+@test findnext(B, 78)   == Base.findnextnot(B′, 78)   == findnext(!, B′, 78)   == 777
+@test findnext(B, 777)  == Base.findnextnot(B′, 777)  == findnext(!, B′, 777)  == 777
+@test findnext(B, 778)  == Base.findnextnot(B′, 778)  == findnext(!, B′, 778)  == 0
+@test findnext(B, 1001) == Base.findnextnot(B′, 1001) == findnext(!, B′, 1001) == 0
+@test findnext(identity, B, 1001) == findnext(x->false, B, 1001) == findnext(x->true, B, 1001) == 0
+@test findfirst(B) == Base.findfirstnot(B′) == 77
+
+emptyvec = BitVector(0)
+@test_throws BoundsError findnext(x->true, emptyvec, -1)
+@test findnext(x->true, emptyvec, 1) == 0
+
+B = falses(10)
+@test findnext(x->true, B, 5) == 5
+@test findnext(x->true, B, 11) == 0
+@test findnext(x->false, B, 5) == 0
+@test findnext(x->false, B, 11) == 0
+@test_throws BoundsError findnext(x->true, B, -1)
+
 ## Reductions ##
 
 b1 = randbool(s1, s2, s3, s4)
@@ -975,6 +1006,10 @@ b1 = triu(randbool(n1, n2))
 b1 = triu(randbool(n2, n1))
 @check_bit_operation istriu(b1) Bool
 
+b1 = randbool(n1,n1)
+b1 |= b1.'
+@check_bit_operation issym(b1) Bool
+
 b1 = randbool(n1)
 b2 = randbool(n2)
 @check_bit_operation kron(b1, b2) BitVector
@@ -993,3 +1028,64 @@ timesofar("linalg")
 # issue #7515
 @test sizeof(BitArray(64)) == 8
 @test sizeof(BitArray(65)) == 16
+
+#one
+@test bitunpack(one(BitMatrix(2,2))) == eye(2,2)
+@test_throws DimensionMismatch one(BitMatrix(2,3))
+
+#reshape
+a = trues(2,5)
+b = reshape(a,(5,2))
+@test b == trues(5,2)
+@test_throws DimensionMismatch reshape(a, (1,5))
+
+#resize!
+
+a = trues(5)
+@test_throws BoundsError resize!(a,-1)
+resize!(a, 3)
+@test a == trues(3)
+resize!(a, 5)
+@test a == append!(trues(3),falses(2))
+
+#flipbits!
+
+a = trues(5,5)
+flipbits!(a)
+@test a == falses(5,5)
+
+# findmax, findmin
+a = trues(0)
+@test_throws ErrorException findmax(a)
+@test_throws ErrorException findmin(a)
+
+a = falses(6)
+@test findmax(a) == (false,1)
+a = trues(6)
+@test findmin(a) == (true,1)
+a = bitpack([1,0,1,1,0])
+@test findmin(a) == (false,2)
+@test findmax(a) == (true,1)
+a = bitpack([0,0,1,1,0])
+@test findmin(a) == (false,1)
+@test findmax(a) == (true,3)
+
+#qr and svd
+
+A = randbool(10,10)
+uA = bitunpack(A)
+@test svd(A) == svd(uA)
+@test qr(A) == qr(uA)
+
+#diag and diagm
+
+v = randbool(10)
+uv = bitunpack(v)
+@test bitunpack(diagm(v)) == diagm(uv)
+v = randbool(10,2)
+uv = bitunpack(v)
+@test_throws DimensionMismatch diagm(v)
+
+B = randbool(10,10)
+uB = bitunpack(B)
+@test diag(uB) == bitunpack(diag(B))
