@@ -177,6 +177,7 @@ end
 @test_approx_eq lgamma(-1/2) log(abs(gamma(-1/2)))
 @test_approx_eq lgamma(1.4+3.7im) -3.7094025330996841898 + 2.4568090502768651184im
 @test_approx_eq lgamma(1.4+3.7im) log(gamma(1.4+3.7im))
+@test_approx_eq lgamma(-4.2+0im) lgamma(-4.2)-pi*im
 
 # digamma
 for elty in (Float32, Float64)
@@ -305,6 +306,10 @@ end
 @test 1e-13 > errc(zeta(2 + 1im, -1.1), -1525.8095173321060982383023516086563741006869909580583246557 + 1719.4753293650912305811325486980742946107143330321249869576im)
 
 @test @evalpoly(2,3,4,5,6) == 3+2*(4+2*(5+2*6)) == @evalpoly(2+0im,3,4,5,6)
+a0 = 1
+a1 = 2
+c = 3
+@test @evalpoly(c, a0, a1) === 7
 
 @test 1e-14 > err(eta(1+1e-9), 0.693147180719814213126976796937244130533478392539154928250926)
 @test 1e-14 > err(eta(1+5e-3), 0.693945708117842473436705502427198307157819636785324430166786)
@@ -324,3 +329,32 @@ end
 for z in (1.234, 1.234 + 5.678im, [1.234, 5.678])
     @test_approx_eq cis(z) exp(im*z)
 end
+
+# modf
+for elty in (Float32, Float64)
+    @test_approx_eq modf( convert(elty,1.2) )[1] convert(elty,0.2)
+    @test_approx_eq modf( convert(elty,1.2) )[2] convert(elty,1.0)
+    @test_approx_eq modf( convert(elty,1.0) )[1] convert(elty,0.0)
+    @test_approx_eq modf( convert(elty,1.0) )[2] convert(elty,1.0)
+end
+
+# frexp
+for elty in (Float32, Float64)
+    @test frexp( convert(elty,0.5) ) == (convert(elty,0.5),0)
+    @test frexp( convert(elty,4.0) ) == (convert(elty,0.5),3)
+    @test_approx_eq frexp( convert(elty,10.5) )[1] convert(elty,0.65625)
+    @test frexp( convert(elty,10.5) )[2] == 4
+    @test_approx_eq frexp( [ convert(elty,4.0) convert(elty,10.5) ] )[1][1] convert(elty,0.5)
+    @test_approx_eq frexp( [ convert(elty,4.0) convert(elty,10.5) ] )[1][2] convert(elty,0.65625)
+    @test frexp( [ convert(elty,4.0) convert(elty,10.5) ] )[2] == [ 3 4 ]
+end
+
+for n = 0:28
+    @test log(2,2^n) == n
+end
+with_bigfloat_precision(10_000) do
+    @test log(2,big(2)^100) == 100
+    @test log(2,big(2)^200) == 200
+    @test log(2,big(2)^300) == 300
+    @test log(2,big(2)^400) == 400
+end
