@@ -261,6 +261,20 @@ end
 @test base(12,typemin(Int128)) == "-2a695925806818735399a37a20a31b3534a8"
 @test base(12,typemax(Int128)) == "2a695925806818735399a37a20a31b3534a7"
 
+@test hex2num("3ff0000000000000") == 1.
+@test hex2num("bff0000000000000") == -1.
+@test hex2num("4000000000000000") == 2.
+@test hex2num("7ff0000000000000") == Inf
+@test hex2num("fff0000000000000") == -Inf
+@test isnan(hex2num("7ff8000000000000"))
+@test isnan(hex2num("fff8000000000000"))
+@test hex2num("3f800000") == 1.0f0
+@test hex2num("bf800000") == -1.0f0
+@test hex2num("7f800000") == Inf32
+@test hex2num("ff800000") == -Inf32
+@test isnan(hex2num("7fc00000"))
+@test isnan(hex2num("ffc00000"))
+
 # floating-point printing
 @test repr(1.0) == "1.0"
 @test repr(-1.0) == "-1.0"
@@ -1827,3 +1841,6 @@ ndigf(n) = float64(log(float32(n)))
 # issue #7911
 @test sum([int128(1) int128(2)]) == int128(3)
 
+let x = big(-0.0)
+    @test signbit(x) && !signbit(abs(x))
+end
