@@ -29,8 +29,6 @@ map!(x->1, d)
 @test reduce(+, d) == 100
 
 
-@unix_only begin
-
 dims = (20,20,20)
 
 @linux_only begin
@@ -77,6 +75,8 @@ copy!(s, d)
 s = Base.shmem_rand(dims)
 copy!(s, sdata(d))
 @test s == d
+a = rand(dims)
+@test sdata(a) == a
 
 d = SharedArray(Int, dims; init = D->fill!(D.loc_subarr_1d, myid()))
 for p in procs(d)
@@ -87,6 +87,24 @@ for p in procs(d)
     @test d[idxl] == p
 end
 
+# reshape
+
+d = Base.shmem_fill(1.0, (10,10,10))
+@test ones(100, 10) == reshape(d,(100,10))
+d = Base.shmem_fill(1.0, (10,10,10))
+@test_throws ErrorException reshape(d,(50,))
+
+# rand, randn
+d = Base.shmem_rand(dims)
+@test size(rand!(d)) == dims
+d = Base.shmem_fill(1.0, dims)
+@test size(randn!(d)) == dims
+
+# similar
+d = Base.shmem_rand(dims)
+@test size(similar(d, Complex128)) == dims
+@test size(similar(d, dims)) == dims
+
 # issue #6362
 d = Base.shmem_rand(dims)
 s = copy(sdata(d))
@@ -108,7 +126,7 @@ d[5,1:2:4,8] = 19
 AA = rand(4,2)
 A = convert(SharedArray, AA)
 B = convert(SharedArray, AA')
-@test B*A == AA'*AA
+@test B*A == ctranspose(AA)*AA
 
 d=SharedArray(Int64, (10,10); init = D->fill!(D.loc_subarr_1d, myid()), pids=[id_me, id_other])
 d2 = map(x->1, d)
@@ -123,9 +141,6 @@ map!(x->1, d)
 @test 3.0 == remotecall_fetch(id_other, D->D[1], Base.shmem_fill(3.0, 1; pids=[id_me, id_other]))
 
 
-end # @unix_only(SharedArray tests)
-
-
 # Test @parallel load balancing - all processors should get either M or M+1
 # iterations out of the loop range for some M.
 if nprocs() < 4
@@ -153,7 +168,7 @@ end
 et=toq()
 
 # assuming that 0.5 seconds is a good enough buffer on a typical modern CPU
-try 
+try
     @test (et >= 1.0) && (et <= 1.5)
     @test !isready(rr3)
 catch
