@@ -6,6 +6,15 @@
 @test Base.mapfoldl(abs2, -, 2:5) == -46
 @test Base.mapfoldl(abs2, -, 10, 2:5) == -44
 
+@test_approx_eq Base.mapfoldl(abs2, /, 2:5 ) 1/900
+@test_approx_eq Base.mapfoldl(abs2, /, 10, 2:5 ) 1/1440
+
+@test Base.mapfoldl((x)-> x $ true, &, true, [true false true false false]) == false
+@test Base.mapfoldl((x)-> x $ true, &, [true false true false false]) == false
+
+@test Base.mapfoldl((x)-> x $ true, |, [true false true false false]) == true
+@test Base.mapfoldl((x)-> x $ true, |, false, [true false true false false]) == true
+
 @test foldr(-, 1:5) == 3
 @test foldr(-, 10, 1:5) == -7
 
@@ -91,22 +100,25 @@ end
 
 # prod
 
-prod(Int[]) === 0
-prod(Int8[]) === 0
-prod(Float64[]) === 0.0
+@test prod(Int[]) === 1
+@test prod(Int8[]) === 1
+@test prod(Float64[]) === 1.0
 
-prod([3]) === 0
-prod([int8(3)]) === 0
-prod([3.0]) === 0.0
+@test prod([3]) === 3
+@test prod([int8(3)]) === 3
+@test prod([3.0]) === 3.0
 
-prod(z) === 120
-prod(fz) === 120.0
+@test prod(z) === 120
+@test prod(fz) === 120.0
+
+@test prod(1:big(16)) == big(20922789888000)
+@test prod(big(typemax(Int64)):big(typemax(Int64))+16) == BigInt("25300281663413827620486300433089141956148633919452440329174083959168114253708467653081909888307573358090001734956158476311046124934597861626299416732205795533726326734482449215730132757595422510465791525610410023802664753402501982524443370512346073948799084936298007821432734720004795146875180123558814648586972474376192000")
 
 # check type-stability
 prod2(itr) = invoke(prod, (Any,), itr)
 @test prod(Int[]) === prod2(Int[]) === 1
 @test prod(Int[7]) === prod2(Int[7]) === 7
-@test typeof(prod(Int8[])) == typeof(prod(Int8[1])) == typeof(prod(Int8[1, 7])) == Int 
+@test typeof(prod(Int8[])) == typeof(prod(Int8[1])) == typeof(prod(Int8[1, 7])) == Int
 @test typeof(prod2(Int8[])) == typeof(prod2(Int8[1])) == typeof(prod2(Int8[1 7])) == Int
 
 # maximum & minimum & extrema
@@ -177,6 +189,11 @@ prod2(itr) = invoke(prod, (Any,), itr)
 @test all(x->x>0, [4]) == true
 @test all(x->x>0, [-3, 4, 5]) == false
 
+@test reduce(|, fill(trues(5), 24))  == trues(5)
+@test reduce(|, fill(falses(5), 24)) == falses(5)
+@test reduce(&, fill(trues(5), 24))  == trues(5)
+@test reduce(&, fill(falses(5), 24)) == falses(5)
+
 # in
 
 @test in(1, Int[]) == false
@@ -186,6 +203,11 @@ prod2(itr) = invoke(prod, (Any,), itr)
 @test in(1, 1:3) == true
 @test in(2, 1:3) == true
 
+# contains
+
+@test contains("quick fox", "fox") == true
+@test contains("quick fox", "lazy dog") == false
+
 # count & countnz
 
 @test count(x->x>0, Int[]) == 0
