@@ -3,18 +3,17 @@ testnames = [
     "linalg", "core", "keywordargs", "numbers", "strings",
     "collections", "hashing", "remote", "iobuffer", "arrayops", "reduce", "reducedim",
     "simdloop", "blas", "fft", "dsp", "sparse", "bitarray", "random", "math",
-    "functional", "bigint", "sorting", "statistics", "spawn",
+    "functional", "bigint", "sorting", "statistics", "spawn", "intfuncs",
     "backtrace", "priorityqueue", "arpack", "file", "suitesparse", "version",
     "resolve", "pollfd", "mpfr", "broadcast", "complex", "socket",
     "floatapprox", "readdlm", "regex", "float16", "combinatorics",
-    "sysinfo", "rounding", "ranges", "mod2pi", "euler", "show",
-    "lineedit", "replcompletions", "repl", "test", "goto"
+    "sysinfo", "rounding", "ranges", "mod2pi", "euler", "show", "unicode",
+    "lineedit", "replcompletions", "repl", "test", "goto", "parser"
 ]
 
 if isdir(joinpath(dirname(@__FILE__), "..", "examples"))
     push!(testnames, "examples")
 end
-@unix_only push!(testnames, "unicode")
 
 # parallel tests depend on other workers - do them last
 push!(testnames, "parallel")
@@ -24,7 +23,7 @@ tests = (ARGS==["all"] || isempty(ARGS)) ? testnames : ARGS
 if "linalg" in tests
     # specifically selected case
     filter!(x -> x != "linalg", tests)
-    prepend!(tests, ["linalg1", "linalg2", "linalg3", "linalg4"])
+    prepend!(tests, ["linalg1", "linalg2", "linalg3", "linalg4", "linalg/cholmod", "linalg/umfpack"])
 end
 
 net_required_for = ["socket", "parallel"]
@@ -38,7 +37,7 @@ end
 
 cd(dirname(@__FILE__)) do
     n = 1
-    if net_on 
+    if net_on
         n = min(8, CPU_CORES, length(tests))
         n > 1 && addprocs(n; exeflags=`--check-bounds=yes`)
         blas_set_num_threads(1)
