@@ -1034,11 +1034,11 @@ end
 
 # isvalid(), chr2ind() and ind2chr() for SubString{DirectIndexString}
 let s="lorem ipsum",
-    sdict=[SubString(s,1,11)=>s, 
+    sdict=[SubString(s,1,11)=>s,
         SubString(s,1,6)=>"lorem ",
-        SubString(s,1,0)=>"", 
-        SubString(s,2,4)=>"ore", 
-        SubString(s,2,16)=>"orem ipsum", 
+        SubString(s,1,0)=>"",
+        SubString(s,2,4)=>"ore",
+        SubString(s,2,16)=>"orem ipsum",
         SubString(s,12,14)=>""
     ]
     for (ss,s) in sdict
@@ -1097,3 +1097,31 @@ let
     @test srep[7] == 'β'
     @test_throws BoundsError srep[8]
 end
+
+# Tests of join()
+@test join([]) == ""
+@test join(["a"],"?") == "a"
+@test join("HELLO",'-') == "H-E-L-L-O"
+@test join(1:5, ", ", " and ") == "1, 2, 3, 4 and 5"
+@test join(["apples", "bananas", "pineapples"], ", ", " and ") == "apples, bananas and pineapples"
+
+# issue #9178 `join` calls `done()` twice on the iterables
+type i9178
+    nnext::Int64
+    ndone::Int64
+end
+Base.start(jt::i9178) = (jt.nnext=0 ; jt.ndone=0 ; 0)
+Base.done(jt::i9178, n) = (jt.ndone += 1 ; n > 3)
+Base.next(jt::i9178, n) = (jt.nnext += 1 ; ("$(jt.nnext),$(jt.ndone)", n+1))
+@test join(i9178(0,0), ";") == "1,1;2,2;3,3;4,4"
+
+# issue #9781
+# float(SubString) wasn't tolerant of trailing whitespace, which was different
+# to "normal" strings. This also checks we aren't being too tolerant and allowing
+# any arbitrary trailing characters.
+@test float64("1\n") == 1.0
+@test float64(split("0,1\n",","))[2] == 1.0
+@test_throws ArgumentError float64(split("0,1 X\n",","))[2]
+@test float32("1\n") == 1.0
+@test float32(split("0,1\n",","))[2] == 1.0
+@test_throws ArgumentError float32(split("0,1 X\n",","))[2]
