@@ -1,41 +1,3 @@
-se33 = speye(3)
-do33 = ones(3)
-@test isequal(se33 \ do33, do33)
-
-# based on deps/Suitesparse-4.0.2/UMFPACK/Demo/umfpack_di_demo.c
-
-using Base.LinAlg.UMFPACK.increment!
-
-A = sparse(increment!([0,4,1,1,2,2,0,1,2,3,4,4]),
-           increment!([0,4,0,2,1,2,1,4,3,2,1,2]),
-           [2.,1.,3.,4.,-1.,-3.,3.,6.,2.,1.,4.,2.], 5, 5)
-lua = lufact(A)
-L,U,p,q,Rs = lua[:(:)]
-@test_approx_eq scale(Rs,A)[p,q] L*U
-
-@test_approx_eq det(lua) det(full(A))
-
-b = [8., 45., -3., 3., 19.]
-x = lua\b
-@test_approx_eq x float([1:5])
-
-@test norm(A*x-b,1) < eps(1e4)
-
-b = [8., 20., 13., 6., 17.]
-x = lua'\b
-@test_approx_eq x float([1:5])
-
-@test norm(A'*x-b,1) < eps(1e4)
-
-Ac = complex(A,A)
-lua = lufact(Ac)
-L,U,p,q,Rs = lua[:(:)]
-@test_approx_eq scale(Rs,Ac)[p,q] L*U 
-
-#4523 - complex sparse \
-x = speye(2) + im * speye(2)
-@test_approx_eq ((lufact(x) \ ones(2)) * x) (complex(ones(2)))
-
 using Base.LinAlg.CHOLMOD
 
 # based on deps/SuiteSparse-4.0.2/CHOLMOD/Demo/
@@ -160,6 +122,6 @@ pred = afiro'*sol
 @test norm(afiro * (y.mat - pred.mat)) < 1e-8
 
 # explicit zeros
-a = SparseMatrixCSC(2,2,[1,3,5],[1,2,1,2],[1.0,0.0,0.0,1.0]) 
+a = SparseMatrixCSC(2,2,[1,3,5],[1,2,1,2],[1.0,0.0,0.0,1.0])
 @test_approx_eq lufact(a)\[2.0,3.0] [2.0,3.0]
 @test_approx_eq cholfact(a)\[2.0,3.0] [2.0,3.0]
