@@ -1,27 +1,28 @@
 # UTF16
-u8 = "\U1d565\U1d7f6\U00066\U2008a"
+u8 = "\U10ffff\U1d565\U1d7f6\U00066\U2008a"
 u16 = utf16(u8)
-@test sizeof(u16) == 14
-@test length(u16.data) == 8 && u16.data[end] == 0
-@test length(u16) == 4
+@test sizeof(u16) == 18
+@test length(u16.data) == 10 && u16.data[end] == 0
+@test length(u16) == 5
 @test utf8(u16) == u8
 @test collect(u8) == collect(u16)
-@test u8 == utf16(u16.data[1:end-1]) == utf16(copy!(Array(Uint8, 14), 1, reinterpret(Uint8, u16.data), 1, 14))
+@test u8 == utf16(u16.data[1:end-1]) == utf16(copy!(Array(Uint8, 18), 1, reinterpret(Uint8, u16.data), 1, 18))
 @test u8 == utf16(pointer(u16)) == utf16(convert(Ptr{Int16}, pointer(u16)))
+@test_throws ArgumentError utf16(utf32(char(0x120000)))
 
 # UTF32
 u32 = utf32(u8)
-@test sizeof(u32) == 16
-@test length(u32.data) == 5 && u32.data[end] == 0
-@test length(u32) == 4
+@test sizeof(u32) == 20
+@test length(u32.data) == 6 && u32.data[end] == char(0)
+@test length(u32) == 5
 @test utf8(u32) == u8
 @test collect(u8) == collect(u32)
-@test u8 == utf32(u32.data[1:end-1]) == utf32(copy!(Array(Uint8, 16), 1, reinterpret(Uint8, u32.data), 1, 16))
+@test u8 == utf32(u32.data[1:end-1]) == utf32(copy!(Array(Uint8, 20), 1, reinterpret(Uint8, u32.data), 1, 20))
 @test u8 == utf32(pointer(u32)) == utf32(convert(Ptr{Int32}, pointer(u32)))
 
 # Wstring
 w = wstring(u8)
-@test length(w) == 4 && utf8(w) == u8 && collect(u8) == collect(w)
+@test length(w) == 5 && utf8(w) == u8 && collect(u8) == collect(w)
 @test u8 == WString(w.data)
 
 if !success(`iconv --version`)
@@ -30,7 +31,7 @@ if !success(`iconv --version`)
 else
     # Create unicode test data directory
     unicodedir = mktempdir()
-    
+
     # Use perl to generate the primary data
     primary_encoding = "UTF-32BE"
     primary_path = replace(joinpath(unicodedir, primary_encoding*".unicode"),"\\","\\\\\\\\")
@@ -40,7 +41,7 @@ else
         binmode(UNICODEF);
         print UNICODEF pack \"N*\", 0xfeff, 0..0xd7ff, 0xe000..0x10ffff;
         close(UNICODEF);"` )
-    
+
     # Use iconv to generate the other data
     for encoding in ["UTF-32LE", "UTF-16BE", "UTF-16LE", "UTF-8"]
         output_path = joinpath(unicodedir, encoding*".unicode")
@@ -48,34 +49,34 @@ else
         run(`iconv -f $primary_encoding -t $encoding $primary_path` |> f)
         Base.FS.close(f)
     end
-    
+
     f=open(joinpath(unicodedir,"UTF-32LE.unicode"))
     str1 = utf32(read(f, Uint32, 1112065)[2:end])
     close(f)
-    
+
     f=open(joinpath(unicodedir,"UTF-8.unicode"))
     str2 = UTF8String(read(f, Uint8, 4382595)[4:end])
     close(f)
     @test str1 == str2
-    
+
     @test str1 == open(joinpath(unicodedir,"UTF-16LE.unicode")) do f
         utf16(read(f, Uint16, 2160641)[2:end])
     end
-    
+
     @test str1 == open(joinpath(unicodedir,"UTF-16LE.unicode")) do f
         utf16(read(f, Uint8, 2160641*2))
     end
     @test str1 == open(joinpath(unicodedir,"UTF-16BE.unicode")) do f
         utf16(read(f, Uint8, 2160641*2))
     end
-    
+
     @test str1 == open(joinpath(unicodedir,"UTF-32LE.unicode")) do f
         utf32(read(f, Uint8, 1112065*4))
     end
     @test str1 == open(joinpath(unicodedir,"UTF-32BE.unicode")) do f
         utf32(read(f, Uint8, 1112065*4))
     end
-    
+
     str1 = "∀ ε > 0, ∃ δ > 0: |x-y| < δ ⇒ |f(x)-f(y)| < ε"
     str2 = utf32(
                  8704, 32, 949, 32, 62, 32, 48, 44, 32, 8707, 32,
@@ -84,7 +85,7 @@ else
                  41, 45, 102, 40, 121, 41, 124, 32, 60, 32, 949
                  )
     @test str1 == str2
-    
+
     # Cleanup unicode data
     for encoding in ["UTF-32BE", "UTF-32LE", "UTF-16BE", "UTF-16LE", "UTF-8"]
         rm(joinpath(unicodedir,encoding*".unicode"))
@@ -99,3 +100,8 @@ let c_ll = 'β', c_cn = '\u038B'
     # check codepoint with category code CN
     @test Base.UTF8proc.category_code(c_cn) == Base.UTF8proc.UTF8PROC_CATEGORY_CN
 end
+
+# handling of embedded NUL chars (#10958)
+@test length("\0w") == length("\0α") == 2
+@test strwidth("\0w") == strwidth("\0α") == 1
+@test normalize_string("\0W", casefold=true) == "\0w"
